#!/usr/bin/env bash

set -e

while ! pidof syslog-ng; do
  echo "waiting for syslog.."
  sleep 5
done

while ! nc -z mysql 3306; do
  echo "waiting for mysql.."
  sleep 1
done

cd /code/iqportal

echo 'resetting db..'
python related/bin/manage.py iqresetdb --settings=$DJANGO_SETTINGS_MODULE

echo 'syncdb etc..'
/code/iqportal/related/bin/docker_syncdb_etc.sh

# loop all installed projects (iqportal, pvcycle etc)
app_dirs="$(find /code -maxdepth 2 -type d)"
for dir in $app_dirs; do
    if [ -e "$dir/__init__.py" ]; then

        echo "Working on dir $dir"
        echo " - compiling .pyc"
        python -m compileall $dir > /dev/null

        echo " - compiling javascript ES6"
        ./related/bin/compile_es6.sh $dir

        echo " - compressing JS"
        ./related/yuicompressor/compress.sh $dir > /dev/null

        echo " - compiling .po"
        for po_file in $(find $dir -name '*.po'); do
            mo_file=$(echo $po_file | sed -e 's/\.po$/.mo/g')
            msgfmt -o $mo_file $po_file
        done

        echo

    fi
done
