# Copyright (c) 2008 BaseSoft Solutions B.V.
# See LICENSE for details.

"""
To handle start-up and configuration of your Twisted application, the Twisted
Application infrastructure uses .tac files.

.tac are Python files which configure an Application object and assign this
object to the top-level variableapplication.

Also see http://twistedmatrix.com/projects/core/documentation/howto/application.html

Run the service with: twistd -noy server.tac
"""

import os
import sqlalchemy

from twisted.application import service, internet
from twisted.web import static, server
from twisted.python import log

from django.conf import settings

from basesoft.scheduler.server import SchedulerWebSite, SchedulerXmlRpc, SchedulerService

# first we create the connection to the iqube db to be used in services
log.msg('IQube Services using DB_CONNSTR: %s' % settings.DB_CONNSTR)
db_engine=sqlalchemy.create_engine(settings.DB_CONNSTR, echo=False)

# this is the core part of any .TAC file, the creation of the root-level application object
application = service.Application("Basesoft Scheduler Server")
serviceCollection = service.IServiceCollection(application)

# attach scheduler service
sched_service = SchedulerService(db_engine)
sched_service.setServiceParent(serviceCollection)

# attach xmlrpc service and pass in the sched_service reference
rpc_resource = SchedulerXmlRpc(sched_service)
rpc_factory = SchedulerWebSite(rpc_resource)
rpc_service = internet.TCPServer(settings.SCHEDULER_SERVER_PORT, rpc_factory)
rpc_service.setServiceParent(serviceCollection)
