#!/usr/bin/python

# tests all installed iqportal apps

from django.conf import settings
from twisted.python.reflect import namedAny

import getopt, sys
import subprocess
import time


def run_tests(testtype):

    assert testtype in ('unit', 'selenium')

    trial_apps = settings.TRIAL_APPS
    if trial_apps is None:
        trial_apps = settings.INSTALLED_APPS

    #print 'running tests for apps: %s' % unicode(trial_apps)

    failed = 0

    for app in trial_apps:

        if app.endswith('.test'):

            try:
                module = namedAny(app+'.'+testtype)
            except AttributeError:
                continue

            test_module = '%s.%s' % (app, testtype)

            print "\n*************************\n** %s **\n*************************" % test_module

            try:
                subprocess.check_call([
                '/usr/bin/trial',
                # '-n',
                test_module
            ])
            except subprocess.CalledProcessError:
                failed += 1

    print
    print '-------'
    print 'Doner! There were %s test failures' % failed
    sys.exit(failed)

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "t:", ["testtype="])
    except getopt.GetoptError, err:
        print str(err)
        sys.exit(2)

    testtype = None

    for o, a in opts:
        if o in ("-t", "--testtype"):
            testtype = a
        else:
            assert False, "unhandled option"


    run_tests(testtype)




if __name__ == "__main__":
    main()


