# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.order.models import InvoiceFile
from sqlalchemy import and_

from basesoft.web.contrib.account.models import Account
from basesoft.web.contrib.notification.models import Notification
from basesoft.web.core.util.db import DatabaseSession

from pvcycle.core.models import PvcTbsServiceLevel, PvcTbsMemberServiceLevel
from pvcycle.order.models import PvcDeclarationOrder, PvcOrder
from pvcycle.order.controllers import PvcDeclarationOrderController
from pvcycle.product.models import PvcTbsProduct

sa_session = DatabaseSession().Session

def convert():
    """
    This script converts the "sender" FK of a notification from Account to Group ID.
    """

    _map = {}
    old_ids = []
    notifications = sa_session.query(Notification).all()
    for n in notifications:
        if n.sender_id is None:
            continue
        # map old id to new group id
        acc = sa_session.query(Account).filter(Account.id == n.sender_id).one()
        _map[n.sender_id] = acc.default_group.id
        # make a list of old ids
        old_ids.append(n.sender_id)
    print 'GROUP IDS ', _map

    old_ids = ','.join(str(i) for i in old_ids)
    # drop foreign key and change column name
    sa_session.execute("""
            ALTER TABLE `tblNotification`
            DROP FOREIGN KEY `tblNotification_ibfk_1`,
            CHANGE COLUMN `sender_id` `old_sender_id` INT(11) NULL DEFAULT NULL
    """)

    # add the new column and foreign key
    sa_session.execute("""
            ALTER TABLE `tblNotification`
            ADD COLUMN `sender_id` INT(11) NULL DEFAULT NULL AFTER `subject`,
            ADD FOREIGN KEY (`sender_id` )
            REFERENCES `tblGroup` (`id` )
            ON DELETE CASCADE
    """)

    update_sql = """
            UPDATE `tblNotification` SET `sender_id`= %(new_id)s WHERE `old_sender_id` IN (%(ids)s)
        """

    for old_id, new_id in _map.iteritems():
        sa_session.execute(update_sql % {
            'new_id': new_id,
            'ids': old_ids
        })

    # drop old column
    sa_session.execute("""
            ALTER TABLE `tblNotification`
            DROP COLUMN `old_sender_id`
    """)

def close_declarations():
    dc = PvcDeclarationOrderController(sa_session)

    _where = PvcDeclarationOrder.status.in_([PvcDeclarationOrder.STATUS_NEW, PvcDeclarationOrder.STATUS_PENDING])
    _where = and_(_where, PvcDeclarationOrder.type != PvcDeclarationOrder.TYPE_CREDIT)

    open_orders = sa_session.query(PvcDeclarationOrder).filter(_where).all()
    # Note: calling protected method on controller.
    closed = dc._close_declarations(open_orders)
    print [d.id for d in closed]

def grant_privileges():
    dc = PvcDeclarationOrderController(sa_session)

    def do_grant(order):
        if order.invoice is None:
            return
        groups = order.get_invoice_view_privileges()
        for group in groups:
            dc.grant_privilege_to(order.invoice, [InvoiceFile.PRIV_VIEW], license=True, target_group=group, check_own_privs=False)

    for o in sa_session.query(PvcOrder).all():
        do_grant(o)

    for o in sa_session.query(PvcDeclarationOrder).all():
        do_grant(o)

def german_sl_change():
    GERMAN_TBS_ID = 22
    print 'Moving German SL to PV Module.'
    service_levels = sa_session.query(PvcTbsServiceLevel).filter(PvcTbsServiceLevel.tbs_id == GERMAN_TBS_ID).all()
    for sl in service_levels:
        print 'Updating service level %s' % sl
        msls = sa_session.query(PvcTbsMemberServiceLevel).filter(PvcTbsMemberServiceLevel.service_level_id == sl.id).all()
        for msl in msls:
            print ' - TBS Member %s' % msl.tbs_member
            msl.pv_service_level = sl
            msl.service_level = None
            sa_session.add(msl)

        sl.product_type = PvcTbsProduct.TYPE_PV_RELATED
        sa_session.add(sl)


if __name__ == "__main__":
    sa_session.begin()
    try:
        convert()
        close_declarations()
        grant_privileges()
        german_sl_change()
        sa_session.commit()
    except Exception, e:
        sa_session.rollback()
        raise e