"""
Common settings script for debugging
"""

from copy import copy
from basesoft.settings.settings import *

DEBUG = True
TEMPLATE_DEBUG = True
YUI_DEBUG_FILES = True

# no need for caching for the dev sites
CACHE_BACKEND = 'dummy:///'

# this disables django catching exceptions but instead they just propagate up so we can see a plain text traceback
DEBUG_PROPAGATE_EXCEPTIONS = True

# default connection string points to localhost
DB_CONNSTR = 'mysql://root:f00b4r@127.0.0.1:3306/iqportal'

# Allow any remote addr to be in admin ips
MIDDLEWARE_CLASSES.append('basesoft.web.middleware.dev.RemoteAddrToInternalIp')

INSTALLED_APPS.extend(MISC_APPS)
INSTALLED_APPS.extend(DEMO_APPS)

BASE_APPS = BASE_APPS + UAT_APPS

# append test to installed apps if exists
def add_test_apps(INSTALLED_APPS):
    for app in copy(INSTALLED_APPS):
        try:
            app = '.'.join(app.split('.')[:-1]) + '.test'
            module = __import__(app, {}, {}, [''])
            if app not in INSTALLED_APPS:
                INSTALLED_APPS.append(app)
        except ImportError:
            pass
add_test_apps(INSTALLED_APPS)


SERVE_STATIC_MEDIA = True

# Selenium configuration
SELENIUM = {}
SELENIUM['browserStartCommand'] = '*firefox'        # point this to the browser you want to run *firefox or *iexplore
SELENIUM['browserURL']  = 'http://127.0.0.1:8000/'  # the url to where selenium has to connect to (where django is running)
SELENIUM['host'] = '127.0.0.1'                      # the hostname where the selenium server is running
SELENIUM['port'] = '4444'                           # tge port where the selenium server is running
SELENIUM['closeBrowser'] = True                     # if true closes browser after failed test
SELENIUM['timeout'] = 10 * 1000                     # default timeout for selenium tests
SELENIUM['screenshot_path'] = '/tmp/'               # path on the selenium server to store screenshots when tests fail
SELENIUM['assertClean'] = False                     # if true the testrunner will set repeat=2 and assert new_listeners = 0
SELENIUM['maximize_window'] = True
SELENIUM['start_django'] = True
SELENIUM['start_scheduler'] = True

SELENIUM_DJANGO_ADDR = '127.0.0.1:8000'             # ip/port which is passed to 'manage.py runserver' to start a django server.
SELENIUM_PYTHONPATH = None


import django
SELENIUM_DJANGO_ADMIN = django.__path__[0] + '/bin/django-admin.py'           # program to run to get the django server started

# used for selenium testcases if IQube is not installed as a python package.
IQUBE_SRC_LOCATION = ''

IQUBE_TEST_IQREPORT_PROJECT = {
    'project_name': 'test_iqreport_compose',
    'connstr': 'sqlite:////tmp/iqportal_test_webshop.db',
    'server': {
        'host': '/tmp',
        'port': None,
        'database_name': 'iqportal_test_webshop.db',
        'user_name': 'test',
        'password': 'test',
        'protocol': 'sqlite'
    }
}

LOGGER_STDOUT_LEVEL = logging.DEBUG
LOGGER_SYSLOG_LEVEL = logging.DEBUG

MONDRIAN_XMLA_CACHE_ENABLED = False
GRAPH_CACHE_ENABLED = False

TRIAL_APPS = None

VIRTUAL_DATE = True

USE_MOCK_FILES = True

SITE_NAME = 'Development site'
SHOW_SITE_NAME = True