# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

class SQLAlchemyTableWrapper:
    """
    This is a wrapper object for the SQLAlchemy Table object so that we can
    JSon serialize it.
    """
    def __init__(self, table):
        """
        Check to see if the passed object is really an sqlalchemy.Table object
        """
        import sqlalchemy
        assert isinstance(table, sqlalchemy.Table)
        self.table = table

    def toJson(self):
        """
        Convert the table to a JSon serializable dictionary
        """
        cols = []
        for col in self.table.columns:
            fkeys = [self._get_foreign_key(x) for x in col.foreign_keys
                     if x is not None]
            cols.append({'name': col.name,
                         'type': self._get_col_type(col.type),
                         'foreign_key': fkeys,
                         'primary_key': col.primary_key})
        return cols

    def _get_foreign_key(self, foreign_key):
        """
        Get the foreign key definition for a column
        """
        try:
            key = str(foreign_key._colspec).split('.')
        except:
            raise UnsupportedDataTableKeyDefinition(
                                'Unsupported Foreign Key Column')
        return {'table': key[0], 'column': key[1]}

    def _get_col_type(self, col_type):
        """
        Get the column type, defaults to String if it cannot find the column
        type in the list below.
        """
        import sqlalchemy
        types = {
                 'Binary': sqlalchemy.Binary,
                 'Boolean': sqlalchemy.Boolean,
                 'Date': sqlalchemy.Date,
                 'DateTime': sqlalchemy.DateTime,
                 'Float': sqlalchemy.Float,
                 'Integer': sqlalchemy.Integer,
                 'Interval': sqlalchemy.Interval,
                 'Numeric': sqlalchemy.Numeric,
                 'PickleType': sqlalchemy.PickleType,
                 'SmallInteger': sqlalchemy.SmallInteger,
                 'String': sqlalchemy.String,
                 'Time': sqlalchemy.Time,
                 'Unicode': sqlalchemy.Unicode
                }
        for name, sqlalchemy_type in types.iteritems():
            if isinstance(col_type, sqlalchemy_type):
                return name
        return 'String'