# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

def column_label(column_index):
    """
    Return the column label based on the given index.
    Eg. 0 -> A, 1 -> B, 2 -> C, ..., 25 -> Z, 26 -> AA, 27 -> AB, ..., 701 -> ZZ

    @param column_index: integer, [0, 701] (ZZ is the biggest column the excel library works with)
    @return: string - 'A', 'B', ..., 'Z', 'AA', 'AB', ..., 'ZZ'
    """
    assert type(column_index) == int
    assert column_index >= 0 and column_index < 702

    # we use the chr function to get the letters from the ASCII table based on the index
    if column_index < 26:
        return chr(65 + column_index)
    else:
        return chr(column_index / 26 + 64) + chr(column_index % 26 + 65)

def row_label(row_index):
    """
    Return the row label based on the given index
    @param row_index: integer, [0, n)
    @return string: '1', '2', ...
    """
    assert type(row_index) == int
    assert row_index >= 0
    return "%s" % (row_index + 1)

def excel_coord(row, col, symbol=''):
    """
    Return a coordinate like A1, B15, etc. Almost like xlrd.cellname(row, col), but with the possibility to specify the
    symbol.

    @param row: integer - the index of the row [0, n)
    @param col: integer - the index of the column [0, 701]
    @param symbol: string - the symbol that needs to be added in the coordinate for (eg. '$' -> B$15)
    @return: string - the excel coordinate (eg. A10, B9, AB3)
    """
    return "%s%s%s" % (column_label(col), symbol, row_label(row))

