# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
General purpose functions that can be used for list processing.
"""

def remove_duplicates(seq, idfun=None):
    """
    Remove the duplicates from a list while preserving the order of the elements

    @param seq: the list from where we remove the duplicates
    @return: a new list that holds all the unique elements from seq while keeping the order
    """

    if idfun is None:
        def idfun(x): return x
    seen = {}
    result = []
    for item in seq:
        marker = idfun(item)
        if marker in seen: continue
        seen[marker] = 1
        result.append(item)
    return result


def list_diff(a, b):
    return set(a) - set(b)


def assert_list_equal(a, b):

    diff1 = list_diff(a, b)
    diff2 = list_diff(b, a)

    assert len(diff1) == 0, 'diff 1: %s' % diff1
    assert len(diff2) == 0, 'diff 2: %s' % diff2

