# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.

from decimal import Decimal, InvalidOperation

import basesoft.web.contrib.i18n as i18n

from basesoft.web.core.exceptions import PublicUserException

class DecimalConversionException(PublicUserException):
    pass

def get_decimal(value, account):

    assert type(value) == str

    try:
        rv = Decimal(value)
        return rv
    except InvalidOperation:
        pass

    import locale
    locale.setlocale(locale.LC_NUMERIC, str(account.get_config('basesoft.web.contrib.locale')))

    ds = account.get_config('basesoft.web.contrib.i18n.number.decimal.separator')
    ts = account.get_config('basesoft.web.contrib.i18n.number.thousands.separator')

    if ds == i18n.THOUSANDS_SEPARATOR_LOCALE and ts == i18n.DECIMAL_SEPARATOR_LOCALE:
        return Decimal(str(locale.atof(value)))

    if ts == i18n.THOUSANDS_SEPARATOR_LOCALE:
        ts = locale.localeconv()['thousand_sep']
    else:
        ts = i18n.THOUSAND_SEPARATOR_CHAR[ts-1]

    if ds == i18n.DECIMAL_SEPARATOR_LOCALE:
        ds = locale.localeconv()['decimal_point']
    else:
        ds = i18n.DECIMAL_SEPARATOR_CHAR[ds-1]

    groups = value.split(ds)
    if len(groups) > 2:
        raise DecimalConversionException('More than one decimal separator (%s) in value' % ds)

    def check_thousands(g):
        last = g[0]
        if len(last) > 3:
            return False
        if len(g) == 1:
            return True
        return len([t for t in g[1:] if len(t) != 3]) == 0

    if len(groups) == 2:
        if len(groups[1].split(ts)) > 1:
            raise DecimalConversionException('Thousand separator (%s) found after decimal separator (%s) in value' % (ts, ds))

        if not check_thousands(groups[0].split(ts)):
            raise DecimalConversionException('Incorrect group sizes found for thousand separators.')

        return Decimal(groups[0].replace(ts, '')+'.'+groups[1])

    if not check_thousands(groups[0].split(ts)):
        raise DecimalConversionException('Incorrect group sizes found for thousand separators.')

    return Decimal(groups[0].replace(ts, ''))
