# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _
from basesoft.web.core.config import DefaultConfig


class AccountConfig(DefaultConfig):
    permissions = None
    label = _('Account settings')
    is_expanded = False

    def configs(self):
        return [{
            'name': 'basesoft.web.core.media.themes',
            'label': _('Display theme'),
            'options': {
                'optionsRpcUrl': 'basesoft.web.contrib.license.ThemesRPCHandler',
                'optionsRpcMethod': 'get_available_themes',
                'emptyFirstOption': True,
                'disableWhenSingleOption': True,
            },
            'help': _('Picking a different theme will change the look and feel of the application.')
        }, {
            'name': 'basesoft.web.core.growl.position',
            'label': _('Information message position'),
            'options': {
                'options': [{
                    'value': 'tl', 'label': _('Top-left corner')
                }, {
                    'value': 'tr', 'label': _('Top-right corner')
                }, {
                    'value': 'bl', 'label': _('Bottom-left corner')
                }, {
                    'value': 'br', 'label': _('Bottom-right corner')
                }]
            }
        }, {
            'name': 'basesoft.web.core.effects.enabled',
            'label': _('Enable visual effects throughout the site'),
            'help': _('Visual effects include fade-in, fade-outs and animations, enabling this can make the application feel slower.')
        }, {
            'name': 'basesoft.web.core.context.sensitive.help',
            'label': _('Show context sensitive help'),
            'help': _('Show these kind of pointers in the interfaces.')
        }, {
            'name': 'basesoft.account.show.form.behavior.errors.inline',
            'label': _('Show input errors on forms inline'),
            'help': _('In case deselected, all input errors will be group into a single message.')
        }]


class NavigationConfig(DefaultConfig):
    permissions = None
    label = _('Navigation & panel settings')
    is_expanded = False

    def configs(self):
        return [{
            'name': 'basesoft.web.core.menucolors.enabled',
            'label': _('Enable colors in palettes'),
        }, {
            'name': 'basesoft.account.use.iconbar',
            'label': _('Use the icon bar to navigate the site'),
            'help': _('If you prefer a normal menu instead of the bottom icon bar, deselect this option')
        }, {
            'name': 'basesoft.account.top.menu.show.vertical.submenu',
            'label': _('Show a vertical sub menu for site navigation'),
            'help': _('When using the top menu, render a vertical list')
        }, {
            'name': 'basesoft.web.core.auto.update.overviews.on.new.data',
            'label': _('Automatically update overviews when new data is available.'),
            'help': _('When disabled a button will appear on overviews when new data becomes available to manually update.'),
        }, {
            'name' : 'basesoft.web.core.store.panel.size.locally',
            'label': _('Remember the custom size of panels on your local machine.'),
            'help': _('When disabled, reopening previously resized panels will acquire their original size again.'),
        }]

configs = (AccountConfig(), NavigationConfig(), )
