# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.mailing.controllers import MailBaseController, AbstractMailStateController
from basesoft.web.contrib.account.models import LostPasswordMail, AccountActivationMail
from basesoft.web.contrib.license.controllers import LicenseController

class AbstractAccountMailStateController(AbstractMailStateController):

    # permissions needed for using this kind of mail
    CREATE_ACCESS_PERMISSIONS   = ['create_account_mail']
    VIEW_ACCESS_PERMISSIONS     = ['view_account_mail']
    UPDATE_ACCESS_PERMISSIONS   = ['update_account_mail']
    DELETE_ACCESS_PERMISSIONS   = ['delete_account_mail']
    SCHEDULE_ACCESS_PERMISSIONS = []

    def _save_plugin(self, mail, state):
        if not state.has_key('recipients') and not state.has_key('world_access'):
            return

        if state.has_key('recipients'):
            if state['recipients'] is None:
                return
            lc = LicenseController(self.sa_session, account_id=self.get_account().id)

            for g in mail.groups:
                if g.group == self.get_default_group():
                    continue
                lc.revoke_privilege_from(mail, mail.PRIV_USE_FOR_SENDING, group=False, license=True, domain=True, check_own_privs=False, target_group=g.group)

            recipients = state['recipients']
            for r in recipients:
                if r[:8] == 'license_':
                    license = lc.get_licenses(r[8:]).pop()
                    self.grant_privilege_to(mail, mail.PRIV_USE_FOR_SENDING, group=False, license=True, check_own_privs=False, target_group=license.default_group)
                    continue
                if r[:7] == 'domain_':
                    domain = lc.get_domains(r[7:]).pop()
                    self.grant_privilege_to(mail, mail.PRIV_USE_FOR_SENDING, group=False, domain=True, check_own_privs=False, target_group=domain.default_group)
                    continue

        if state.has_key('world_access'):
            # TODO: validate reseller
            pass
            #self.grant_privilege_to(mail, mail.PRIV_USE_FOR_SENDING, group=False, check_own_privs=False, world=True)

    def _load_plugin(self, mail, state):
        recipients = []
        for g in mail.groups:
            if g.privilege!=mail.PRIV_USE_FOR_SENDING:
                continue
            if g.license_has_privilege:
                recipients.append('license_%s' % g.group.license.id)
            if g.domain_has_privilege:
                recipients.append('domain_%s' % g.group.license.domain.id)
                continue
        state['mail']['recipients'] = recipients

class LostPasswordMailStateController(AbstractAccountMailStateController):
    _instance_class = LostPasswordMail

class AccountActivationMailStateController(AbstractAccountMailStateController):
    _instance_class = AccountActivationMail

class MailController(MailBaseController):
    """
    Class that's on top of the MailBaseController. This is used for
    handling AccounttMail objects.
    """
    # mapping between object type and the controller
    _state_controllers = {
        'LostPasswordMail': LostPasswordMailStateController,
        'AccountActivationMail': AccountActivationMailStateController
    }
