# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
Views which allow users to create and activate accounts.
TODO: this file needs cleaning
"""
from basesoft.util.ordered_dict import OrderedDict
from django.utils.translation import ugettext as _
from sqlalchemy import and_, select, case
from django.conf import settings
from twisted.python.reflect import namedAny

from basesoft.web.core.util.controllers import AccountConfigController, get_models_of_type, LicenseConfigController, AbstractController
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.core.exceptions import PublicUserException,\
    PublicSecurityException

from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.contrib.license.models import Domain

from basesoft.web.contrib.account.controllers import AccountController, UniqueEmailAddress
from basesoft.web.contrib.session.controllers import SessionController
from basesoft.web.contrib.notification.controllers import NotificationController

from basesoft.web.core.input.validation import plugins, group_plugins
from basesoft.web.contrib.account.models import Account, AccountActivation
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.license.rpc import get_configs
from basesoft.web.contrib.branding.controllers import SiteController,\
    LicenseThemeController

LOCALE_PATH = 'basesoft.web.contrib.account'

__configs_defined = False
configs = ()

def rpc(request):

    dispatcher = JsonRpcDispatcher()

    sa_session = request.Session()
    controller = AccountController(sa_session, request=request)

    global __configs_defined
    global configs
    if not __configs_defined:
        for app in settings.INSTALLED_APPS:
            try:
                sa_module = namedAny(app+'.config')
            except AttributeError:
                continue
            if not hasattr(sa_module, 'configs'):
                continue
            configs = configs + sa_module.configs
        __configs_defined = True

    class Handler:

        def save_account_person_info(self, activation_key, person_state):
            if controller.can_active(activation_key) is True:
                raise PublicUserException('Account details already provided')

            controller.save_person_info(activation_key, person_state)
            return True

        def activate(self, activation_key):
            activation_key = activation_key.lower() # Normalize before trying anything with it.

            # check if all requirements are met.
            rv = controller.can_active(activation_key)
            if rv is True:
                controller.activate_account(activation_key)
                return True

            return rv

        def get_configs(self):
            pc = AbstractController(sa_session, account_id=request.session['user_id'], request=request)

            # we load the account configs and check the permissions.
            # FIXME OrderedDict
            ac = [c.get_config(sa_session) for c in configs if c.has_permissions(pc)]
            lc = []

            if not pc.has_permissions('manage_license_configs'): # none of the attached licenses has this permission
                return {'account': ac, 'license': lc}

            cfgs = get_configs()

            # we load the license configs and check the permissions per license.
            licenses = []
            for group in pc.get_active_groups():
                if group.license not in licenses and pc.has_permissions('manage_license_configs', license_id=group.license.id):

                    if not cfgs.has_key(group.license.polymorphic_type):
                        continue

                    conf = cfgs[group.license.polymorphic_type]
                    licenses.append(group.license)
                    lcc = LicenseConfigController(sa_session, scope=group.license)
                    rv = []
                    for c in conf:
                        # the license needs to have the actual permission, not just any license.
                        if not c.has_permissions(pc, license_id=group.license.id):
                            continue
                        c.set_controller(pc)
                        rv.append(c.get_config(sa_session))
                    lc.append({
                        'license_id': group.license.id,
                        'license_name': group.license.name,
                        'configs': rv,
                        'values': lcc.get_configs(),
                        'polymorphic_type': group.license.polymorphic_type,
                    })

            return {'account': ac, 'licenses': lc}

    dispatcher.register_instance(Handler())

    return dispatcher.handle_request(request)

def login_rpc(request):
    """
    Login view for rpc account validation

    In case of login the 'submit' method called from rpc will only check if the submitted account is valid.
    We need another method that will store the account in the session, update last login date/time for the account
     and return the redirect url.
    This method will also be called from rpc.
    """
    account_id = None
    if request.session.has_key('user_id'):
        account_id = request.session['user_id']

    controller = AccountController(request.Session(), account_id=account_id, request=request)
    session_controller = SessionController(request.Session())
    notification_controller = NotificationController(request.Session(), request=request)

    def _set_session_data(account):
        """
        set the session data: user_id, username and group_id for the supplied account
        """
        request.session['user_id'] = account.id
        request.session['username'] = account.name

        ac = AccountController(request.Session(), account_id = account, request=request)
        if not ac.config.get('basesoft.web.contrib.locale'):
            ac.config.set('basesoft.web.contrib.locale', request.LANGUAGE_CODE)
        request.session['config'] = ac.config.get_configs()

        ltc = LicenseThemeController(request.Session(), account_id=account.id, request=request)
        current_theme = ltc.get_current_theme()
        request.session['theme_module_name'] = current_theme.module_name

        # get all the groups that include the current account
        account_groups = controller.get_groups_for_account(account)

        #if account has only one group, save that one
        if (len(account_groups) == 1):
            request.session['group_id'] = account_groups[0].group.id

    def login(username, password=None, keep_logged_in=False):
        """
        rpc call to login
        @return the username and id of the user that logged in
        """
        if isinstance(username, dict):
            controller.logger.warning("login: deprecated passing of arguments")
            password = username['password']
            username = username['username']

        account = controller.validate_account(name=username, password=password)

        if not account:
            raise PublicSecurityException(_("Incorrect Username or Password"))

        domain = account.default_group.license.domain
        # here we check to see if the domain the user is part of is actually active
        domain.is_active()

        # check if we have a user that needs some extra checks; eg: domain might have the setting for single login,
        # so if that's the case, we check if the user is already logged on another machine and deny him access.
        # if the domain doesn't have this setting proceed with the login as usual
        if domain.has_setting(Domain.SETTING_SINGLE_LOGIN):
            if session_controller.check_active(account.id):
                # user's logged in somewhere else
                notification_controller.create_system_notification(_("Duplicate login attempt"),
                    _("User %s tried to log in while active on a different machine. Login was denied") % account.name,
                    account.default_group.license.domain.default_license.default_group)
                controller.logger.warning("User %s tried to log in while active on a different machine; login denied" % account.name)
                raise PublicSecurityException(_("You are unable to log in at the moment"))

        request.session['keep_logged_in'] = keep_logged_in

        # set this in the session so we can update the logout date from the session table and set it back to null if
        # it's a recycled session, otherwise the login manager will show the user as logged out
        request.session['logged_out'] = None

        original_theme_name = request.session.get('theme_module_name', None)
        _set_session_data(account)
        #if original_theme_name != request.session['theme_module_name']:
        #    request.session['theme_mismatch'] = True

        # normal login removes any traces of previous impersonators (for the timeouts for example)
        if request.session.has_key('impersonator_username'):
            del request.session['impersonator_username']
        if request.session.has_key('impersonator_keep_logged_in'):
            del request.session['impersonator_keep_logged_in']

        controller.update_last_login(account)
        controller.logger.update(request=request)
        controller.logger.info("logged in")

        return {
            'id': account.id,
            'username': account.name
        }

    def login_as(username):
        """
        Wrapper for the _login_as method, to make sure the proper params are used
        Since this method can be called from RPC, it must always check the ownership
        """

        keep_logged_in = request.session.has_key('keep_logged_in') and request.session['keep_logged_in']
        return _login_as(username, store_previous=True, check_ownership=True, keep_logged_in=keep_logged_in)

    def _login_as(username, store_previous=True, check_ownership=True, keep_logged_in=False):
        """
        Log in as a different user (that you can see in the Account manager)
        @param username: the username of the account we're switching to
        @param store_previous: Boolean flag that indicates if we should store the info in the session about who switched
            to username (False only when we return from a switched account to the super account who made the switch)
        @param check_ownership: Boolean flag that indicates
        @param keep_logged_in: a flag that indicates if the username should be kept logged in
        """
        current_username = None
        current_keep_logged_in = None
        if request.session.has_key('username'):
            current_username = request.session['username']
        if request.session.has_key('keep_logged_in'):
            current_keep_logged_in = request.session['keep_logged_in']

        controller.logout(request) # log out and clear the session
        # check if current_username is allowed to log in as username and return the validated account
        account = controller.login_as(request, current_username, username, check_ownership)

        if not account:
            controller.logger.warning("user %s tried to log in as user %s and was not allowed." % (current_username, username))
            raise PublicSecurityException(_("Cannot switch to this username"))

        request.session['keep_logged_in'] = keep_logged_in

        # set this in the session so we can update the logout date from the session table and set it back to null if
        # it's a recycled session, otherwise the login manager will show the user as logged out
        request.session['logged_out'] = None

        original_theme_name = request.session['theme_module_name']
        _set_session_data(account)
        #if original_theme_name != request.session['theme_module_name']:
        #    request.session['theme_mismatch'] = True

        if current_username is not None and store_previous:
            request.session['impersonator_username'] = current_username
            if current_keep_logged_in is not None:
                request.session['impersonator_keep_logged_in'] = current_keep_logged_in
        else:
            if request.session.has_key('impersonator_username'):
                del request.session['impersonator_username']
            if request.session.has_key('impersonator_keep_logged_in'):
                del request.session['impersonator_keep_logged_in']

        controller.update_last_login(account)
        controller.logger.update(request=request)

        if check_ownership:
            controller.logger.info("user %s impersonated user %s" % (current_username, account.name))
        else:
            controller.logger.info("logged back as %s after impersonating %s" % (username, current_username))

        return {
            'id': account.id,
            'username': account.name
        }

    def logout():
        # check if user came from some admin that logged in for helpdesk purposes
        impersonator_username = None
        if request.session.has_key('impersonator_username'):
            impersonator_username = request.session['impersonator_username']

        keep_logged_in = False
        if request.session.has_key('impersonator_keep_logged_in'):
            keep_logged_in = request.session['impersonator_keep_logged_in']


        if impersonator_username is not None:
            # need to log back as the original user
            _login_as(impersonator_username, store_previous=False, check_ownership=False, keep_logged_in=keep_logged_in)
        else:
            controller.logout(request)

    def is_logged_in():
        """
        This checks to see whether or not the current user is a logged in user
        or not.
        """
        if request.session.has_key('user_id'):
            return True
        raise PublicUserException('User is not logged in')

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(is_logged_in,'is_logged_in')
    dispatcher.register_function(logout,'logout')
    dispatcher.register_function(login,'login')
    dispatcher.register_function(login_as,'login_as')
    return dispatcher.handle_request(request)


def change_password_rpc(request):
    """
    View for rpc form validation and submission requests for for forgot password form
    """
    sa_session = request.Session()
    controller = AccountController(sa_session, request=request)

    class ChangePasswordHandler:
        """
        Handles forgot password form submission and validation
        """
        def submit(self, values, activation_key):
            """
            Handle the values submitted from the forgot password dialog.
            """
            if values['password1'] != values["password2"]:
                raise PublicUserException(_(u'Passwords do not match'))
            if values['password1'] == values["username"]:
                raise PublicUserException(_(u'Password cannot be the same as username'))
            controller.activate_change_password(
                activation_key=activation_key,
                name=values['username'],
                new_password=values['password1']
            )

        def validate(self, *args, **kwargs):
            """
            Used for forgot password form validation
            """
            form_handler = FormValidator({
                'username': [(plugins.Required, {}), (plugins.Username, {'min': 5, 'max': 40})],
                'password1': [(plugins.Required, {}), (plugins.StrongPassword, {})],
                'password2': [(plugins.Required, {}), (plugins.StrongPassword, {})],
            })
            return form_handler.validate(*args, **kwargs)

    dispatcher = JsonRpcDispatcher()
    handler = ChangePasswordHandler()
    dispatcher.register_instance(handler)
    return dispatcher.handle_request(request)

@login_required
def manager_rpc(request):
    """
    """
    sa_session = request.Session()
    user_id = request.session['user_id']

    controller = AccountController(sa_session, account_id=user_id, request=request)

    def get_accounts():
        """
        """
        return [{'value': a.id, 'label': a.name} for a in controller.get_accounts()]

    def get_account_types():
        rv = []
        for m in get_models_of_type(Account, False, None, True):
            # only add accounts of type we have permissions for.
            if not controller.has_permissions(*m['model'].permissions):
                continue
            rv.append({
                'value': m['value'],
                'label': m['label'],
                'js_module': m['model'].JS_MODULE
            })
        return rv

    def get_account_groups(account_id):
        account = controller.get_accounts(account_id).pop()
        return {'groups': [g.group.id for g in account.groups], 'default_group': account.default_group.id}

    def get_available_groups_for_account(selected=None, filter=''):

        lc = LicenseController(sa_session, account_id=user_id, request=request)
        groups = {}
        for g in lc.get_groups():

            if selected is None or g.id not in selected:
                if filter.lower() not in g.license.name.lower():
                    continue
            if not groups.has_key(g.license.name):
                groups[g.license.name] = []
            groups[g.license.name].append({'label': g.name, 'value': g.id})
        return OrderedDict(sorted(groups.items()))

    def save_groups_for_account(account_id, groups, default_group):
        controller.save_groups_for_account(account_id, groups, default_group)
        return True

    def send_activation_mail(account_ids):
        """
        Send an activation mail in case there is none send yet.
        """
        if not isinstance(account_ids, list):
            account_ids = [account_ids]
        accounts = controller.get_accounts(account_ids)
        sent = False
        for account in accounts:
            aa = controller.get_account_activation(account, AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT_FROM_LICENSE)
            if aa is None:
                controller.save_account_activation(account, AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT_FROM_LICENSE)
                sent = True
        return sent # if it was sent to at least one of the accounts

    def reset_activation(account_ids):
        if not isinstance(account_ids, list):
            account_ids = [account_ids]

        # check for one account, this makes feedback better in case of error
        if len(account_ids) != 1:
            raise PublicUserException(_('Please select only one account'))
        account = controller.get_accounts(account_ids)[0]

        # do the reset
        controller.reset_account_activation(account)


    def update_field(field, value, group_id, account_id):
        """
        """
        account = controller.get_accounts(account_id)[0]
        group_account = controller.get_groups_for_account(account, group_id)[0]

        is_default = group_account.is_default
        is_active = group_account.active

        try:
            value = int(value)
        except TypeError:
            value = 0

        if(field=='active'):
            is_active = (value != 0)
        elif(field=='is_default'):
            is_default = (value != 0)

        controller.save_or_update_group_account(group_account.group, account,
                                                is_default=is_default,
                                                is_active=is_active)
        return True

    def remove_account(account_id):
        return controller.remove_account(account_id)

    def save_account(account_id, state):
        """
        """
        # TODO: pass in the right license controller (reseller vs normal)
        controller.save_account_from_state(state, account_id=account_id)
        return True

    def get_account_state(id):
        """
        """
        account = controller.get_accounts(id).pop()

        return {
            'state': controller.get_account_state(account),
            'type': account.__class__.__name__,
            'type_label': account.NAME_I18N,
            'type_jsmodule': account.JS_MODULE
        }

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(get_accounts,'get_accounts')
    dispatcher.register_function(update_field,'update_field')
    dispatcher.register_function(save_account,'save_account')
    dispatcher.register_function(get_account_state,'get_account_state')
    dispatcher.register_function(get_account_types,'get_account_types')
    dispatcher.register_function(send_activation_mail,'send_activation_mail')
    dispatcher.register_function(reset_activation,'reset_activation')
    dispatcher.register_function(get_account_groups,'get_account_groups')
    dispatcher.register_function(get_available_groups_for_account,'get_available_groups_for_account')
    dispatcher.register_function(save_groups_for_account,'save_groups_for_account')
    dispatcher.register_function(remove_account,'remove_account')

    return dispatcher.handle_request(request)

@login_required
def config_manager_rpc(request):
    """
    """
    sa_session = request.Session()
    user_id = request.session['user_id']

    def update_config(account_config, account_id=None):
        """
        if no account_id was supplied, then we'll update the configs for the current user
        """

        if account_id is None:
            ac_controller = AccountConfigController(sa_session, account_id=user_id)
        else:
            ac_controller = AccountConfigController(sa_session, account_id=account_id, scope_account_id=user_id)
        ac_controller.set_configs(account_config)
        configs = ac_controller.get_configs()
        request.session['config'] = configs
        return configs

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(update_config,'update_config')
    return dispatcher.handle_request(request)


@login_required
def account_validation_rpc(request):
    form_handler = FormValidator({
        'account_name': [
            (plugins.Required, {}),
            (plugins.Username, {'min': 5, 'max': 40}),
            (plugins.UniqueAccountName, {'db_session': request.Session()})
        ],
        'license': [(plugins.Required, {})],
        'groups': [(plugins.Required, {})],
        'default_group': [(plugins.Required, {})]
    })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

def registration_form_rpc(request):
    """
    """
    sa_session = request.Session()
    controller = AccountController(sa_session, request=request)
    sc = SiteController(sa_session, request=request)

    class RegistrationHandler:
        """
        Handles registration form submission and validation
        """
        def submit(self, values):
            """
            Handle the values submitted from the registration dialog.
            """
            # validate the data
            # check that the username is unique

            if not controller.is_unique_account_name(values['username']):
                raise PublicUserException(_(u'This username is already taken. Please choose another.'))

            # check that the username and password are different
            if values['username'] == values['password']:
                raise PublicUserException(_(u'Password cannot be the same as username'))

            if values['password'] != values['password2']:
                raise PublicUserException(_(u'Passwords do not match'))

            site = sc.get_current_site()
            if site is not None and site.domain is not None:
                if site.domain.has_setting(Domain.SETTING_REQUIRE_EULA_ON_REGISTRATION) and not values.has_key('eula'):
                    raise PublicUserException(_(u'You have to accept the EULA in order to register'))


            lc = LicenseController(sa_session, request=request)
            site_licensee = lc.get_license_for_request(request)

            controller.create_inactive_account(name=values['username'],
                                               password=values['password'],
                                               email=values['email'],
                                               reseller=site_licensee)
        def validate(self, *args, **kwargs):
            """
            Used for registration form validation
            """
            validate = {
                'username': [(plugins.Required, {}), (plugins.Username, {'min': 5, 'max': 40})],
                'email': [(plugins.Required, {}), (plugins.Email, {}), (UniqueEmailAddress, {'db_session': sa_session})],
                'password': [(plugins.Required, {}), (plugins.StrongPassword, {})],
                'password2': [(plugins.Required, {}), (plugins.StrongPassword, {})],
            }

            site = sc.get_current_site()
            if site is not None and site.domain is not None:
                if site.domain.has_setting(Domain.SETTING_REQUIRE_EULA_ON_REGISTRATION):
                    validate['eula'] = [(plugins.RequiredEULA, {})]

            form_handler = FormValidator(validate)
            return form_handler.validate(*args, **kwargs)

    dispatcher = JsonRpcDispatcher()
    handler = RegistrationHandler()
    dispatcher.register_instance(handler)
    return dispatcher.handle_request(request)

def forgot_password_validation_rpc(request):
    form_handler = FormValidator(
        group_plugins = [(group_plugins.Required, {
            'email': {'username': False},
            'username': {'email': False}
        })]
    )

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

def forgot_password_form_rpc(request):
    """
    """
    sa_session = request.Session()
    controller = AccountController(sa_session, request=request)

    class ForgotPasswordHandler:
        """
        Handles forgot password form submission and validation
        """
        def submit(self, values):
            """
            Handle the values submitted from the forgot password dialog.
            """
            controller.forgot_credentials(account_name=values['username'], email=values['email'])

    dispatcher = JsonRpcDispatcher()
    handler = ForgotPasswordHandler()
    dispatcher.register_instance(handler)
    return dispatcher.handle_request(request)
