# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext as _
from sqlalchemy.sql.expression import and_, or_

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryModel, CustomQueryField
from basesoft.web.contrib.announce.models import Announcement, AnnouncementAccount


class AnnounceCustomQuery(CustomQuery):
    """
    custom query for managing (editing) announcements
    """

    permissions = ['manage_announcements']

    def tables(self):

        return [
            CustomQueryModel(Announcement),
        ]

    def fields(self):
        return [
            CustomQueryField(Announcement.id, _('ID'), key='announcement_id'),
            CustomQueryField(Announcement.label, _('Label'), key='label'),
            CustomQueryField(Announcement.publish_date, _('Publish Date'), key='publish_date'),
            CustomQueryField(Announcement.expire_date, _('Expire Date'), key='expire_date'),
            CustomQueryField(Announcement.must_read, _('Must Read'), key='must_read'),
            CustomQueryField(Announcement.id, _('Translate'), key='translate'),

        ]

class AnnounceRecipientCustomQuery(CustomQuery):
    """
    custom query for reading recieved announcements
    """

    permissions = ['receive_announcements']

    def tables(self):

        return [
            CustomQueryModel(Announcement, priv=Announcement.PRIV_RECIPIENT),
            CustomQueryModel(AnnouncementAccount, use_outerjoin=True),
        ]

    def fields(self):
        return [
            CustomQueryField(Announcement.id, _('ID'), key='announcement_id'),
            CustomQueryField(Announcement.label, _('Label'), key='label'),
            CustomQueryField(Announcement.publish_date, _('Publish Date'), key='publish_date'),
            CustomQueryField(Announcement.expire_date, _('Expire Date'), key='expire_date'),
            CustomQueryField(AnnouncementAccount.account_id, _('Has Read'), key='has_read'),

        ]

    def set_where(self):

        # filter out announcents published after this user was created
        # and not yet expired
        self.where = and_(
            Announcement.status == Announcement.STATUS_PUBLISHED,
            Announcement.publish_date >= self.get_account().created,
            or_(Announcement.expire_date == None, self.now() < Announcement.expire_date),
        )




custom_queries = (AnnounceCustomQuery, AnnounceRecipientCustomQuery, )

