# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class AnnounceManagerIcon(IconBarIcon):

    id = 'icon-announce-manager'
    title = _('Announcement manager')
    icon_index = 75
    component = 'iq_announce_manager'
    group = 'announce'
    group_title = _('Announcement')
    colorname = 'iconbar-crimson'

    char_main = u'\uE819' # Loadspeaker icon
    char_sub = u'\uE806' # Man icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_announcements')
        controller.assert_any_permissions('view_domain', 'view_license', 'view_all_domains')

class AnnounceRecipientIcon(IconBarIcon):

    id = 'icon-announce-recipient-manager'
    title = _('Announcement overview')
    icon_index = 75
    component = 'iq_announce_recipient_manager'
    group = 'announce'
    group_title = _('Announcement')
    colorname = 'iconbar-crimson'

    char_main = u'\uE819' # Loadspeaker icon
    char_sub = u'\uE82B' # List icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('receive_announcements')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """
    return [AnnounceManagerIcon(), AnnounceRecipientIcon(), ]
