# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.announce.controllers import AnnounceController
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.contrib.wiki.rpc import AbstractWikiHandler

class AnnounceHandler(AbstractWikiHandler):
    """
    AnnounceHandler extends AbstractWikiHandler which provides the wiki rpc calls
    """

    @login_required
    def __init__(self, request, **kwargs):
        self.request = request
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self._controller = AnnounceController(self.sa_session, account_id = self.account_id, request=request)

    def validate_announcement(self, args):

        form_handler = FormValidator({
            'label': [(plugins.Required, {})],
            'publish_date': [(plugins.Required, {})],
        })
        rv = form_handler.validate(args)

        return rv

    def save_announcement(self, announcement_id, state):
        self._controller.save_announcement(announcement_id, state)

    def load_announcement(self, announcement_id):
        return self._controller.load_announcement(announcement_id)

    def get_unread_announcements(self, must_read=False):
        return [x.id for x in self._controller.query_unread_announcements(must_read)]

    def mask_as_read(self, announcement_id):
        self._controller.mark_as_read(announcement_id)

    def load_wiki_with_fallback(self, announcement_id):
        rv = super(AnnounceHandler, self).load_wiki_with_fallback(announcement_id)
        rv['logo'] = self._controller.get_logo(announcement_id)
        return rv

handlers = (AnnounceHandler, )
