# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel
from basesoft.web.contrib.branding.models import BrandingHeaderFooter, Site, Theme, LicenseTheme
from basesoft.web.contrib.license.models import Domain, License

class HeaderFooterCustomQuery(CustomQuery):
    def tables(self):
        return (
            CustomQueryModel(BrandingHeaderFooter),
        )

    def fields(self):
        return (
            CustomQueryField(BrandingHeaderFooter.id, _('ID'), key="id"),
            CustomQueryField(BrandingHeaderFooter.name, _('Name'), key="name", is_filter=True),
            CustomQueryField(BrandingHeaderFooter.created, _('Created')),
            CustomQueryField(BrandingHeaderFooter.last_updated, _('Last updated')),
        )

class SiteCustomQuery(CustomQuery):
    permissions = ['site_manager']
    def tables(self):
        return (
            CustomQueryModel(Site),
            CustomQueryModel(Theme),
            CustomQueryModel(Domain, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Site.host_match, _('Host match'), key='host_match', is_filter=True),
            CustomQueryField(Site.host_match, _('Details'), key='details'),
            CustomQueryField(Site.order, _('Order'), key='order_id'),
            CustomQueryField(Domain.name, _('Domain'), key='domain', is_filter=True),
            CustomQueryField(Theme.name, _('Theme'), key='theme'),
            CustomQueryField(Site.home_app, _('Home app'), key='home_app', is_filter=True),
            CustomQueryField(Site.analytics_id, _('Google UA'), key='analytics_id', is_filter=True),
        )

class LicenseThemeCustomQuery(CustomQuery):
    permissions = ['site_manager', 'view_all_domains']
    def tables(self):
        return (
            CustomQueryModel(Theme),
            CustomQueryModel(LicenseTheme, use_outerjoin=True),
            CustomQueryModel(License, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Theme.name, _('Theme'), key='theme', is_filter=True, is_group_by=True),
            CustomQueryField(Theme.id, 'Theme ID', key='theme_id'),
            CustomQueryField(License.name, _('License'), key='license', is_filter=True, is_group_by=True),
            CustomQueryField(License.id, 'License ID', key='license_id'),
        )

    group_by = (Theme.id, LicenseTheme.license_id, )

custom_queries = (HeaderFooterCustomQuery, SiteCustomQuery, LicenseThemeCustomQuery, )
