# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class HeaderFooterManagerIcon(IconBarIcon):
    """
    Class responsible for handling the header footer manager icon
    """

    id = 'icon-header-footer-manager'
    title = _('Header and Footer manager')
    icon_index = 75
    component = 'iq_header_footer_manager'
    group = 'branding'
    colorname = 'iconbar-buttercup'

    char_main = u'\uE81d' # Edit icon
    char_sub = u'\uE811' # None

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('branding_view_header_footer')

class SiteManagerIcon(IconBarIcon):

    id = 'icon-site-manager'
    title = _('Site manager')
    icon_index = 70
    component = 'iq_site_manager'
    group = 'branding'
    colorname = 'iconbar-buttercup'

    char_main = u'\uE81d' # Edit icon
    char_sub = u'\uE806' # Man icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('site_manager')

class ThemeManagerIcon(IconBarIcon):

    id = 'icon-theme-manager'
    title = _('Theme manager')
    icon_index = 71
    component = 'iq_theme_manager'
    group = 'branding'
    colorname = 'iconbar-buttercup'

    char_main = u'\uE81d' # Edit icon
    char_sub = u'\uE806' # Man icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('site_manager', 'view_all_domains')

def get_icons():
    """
    return the list of icons that the branding app adds to the iconbar
    """

    return [HeaderFooterManagerIcon(), SiteManagerIcon(), ThemeManagerIcon()]
