'''
Created on 25 Jan 2013

@author: martijn
'''
import datetime

from django.utils.translation import ugettext as _

from basesoft.web.core.exceptions import SystemException, PublicUserException

class ContactLineImporter(object):

    separator = None
    quotechar = None
    encoding = None

    def __init__(self, file_path):
        self._file_path = file_path

        # file content as array with file rows
        self._file_rows = []

    def set_file_lines(self, *args, **kwargs):
        raise Exception('abstract')

    def get_rows(self):
        """
        Return the file lines
        """
        return self._file_rows



class CsvImporter(ContactLineImporter):

    def set_file_lines(self, separator=None, quotechar=None):
        """
        sets the file rows
        @return: {Array} with file rows
        """

        from basesoft.web.contrib.contact.controllers import ImportContactController

        if separator is None:
            self.__autodetermine_separator()
        else:
            for s in ImportContactController.SEPARATOR_OPTIONS:
                if separator == s["value"]:
                    self.separator = s["value"]
                    break
            if self.separator is None:
                raise SystemException('Invalid separator %s' % separator)
        if quotechar is None:
            # TODO: autodetermine quotechar
            self.quotechar = ImportContactController.QUOTECHAR_OPTIONS[0]
        else:
            self.quotechar = quotechar
        import csv

        rows = csv.reader(open(self._file_path, 'rU'), delimiter=self.separator, quotechar=self.quotechar)

        encodings = ['utf-8', 'iso-8859-1', 'iso-8859-2', 'ascii']
        chardet_used = False
        ret = []
        try:
            for r in rows:
                if len(r) > 0:
                    row = []
                    for v in r:
                        if self.encoding is not None:
                            try:
                                v = unicode(v, self.encoding)
                            except UnicodeDecodeError:
                                self.encoding = None
                        if self.encoding is None:
                            for enc in encodings:
                                try:
                                    v = unicode(v, enc)
                                    self.encoding = enc
                                    break
                                except UnicodeDecodeError:
                                    pass
                        if self.encoding is None and not chardet_used:
                            self.encoding = self.__detect_encoding()
                            chardet_used = True
                            try:
                                v = unicode(v, self.encoding)
                            except UnicodeDecodeError:
                                self.encoding = None
                        if self.encoding is None:
                            raise PublicUserException(_('Invalid file encoding'))
                        row.append(v.strip())
                    # check if the row is not empty
                    for i in row:
                        if i is not None and i != '':
                            ret.append(row)
                            break
        except csv.Error, e:
            print 'file %s, line %d: %s' % (self._file_path, rows.line_num, e)
            # there was something wrong with the file, so the user needs to know
            raise PublicUserException(_('File error') + ':' + e.message)

        self._file_rows = ret

    def __detect_encoding(self):
        from chardet.universaldetector import UniversalDetector
        detector = UniversalDetector()
        infile = open(self._file_path, "r")
        i = 0
        for line in infile:
            detector.feed(line)
            if detector.done: break
            # only check for the first 1000 rows otherwise it could timeout
            if i == 1000: break
            i = i + 1
        detector.close()
        return detector.result['encoding']

    def __autodetermine_separator(self):
        """
        Method to autodetermine the columns separator in a csv file
        """
        from basesoft.web.contrib.contact.controllers import ImportContactController

        separators = [s["value"] for s in ImportContactController.SEPARATOR_OPTIONS]

        infile = open(self._file_path, "r")
        ret = []
        for line in infile:
            #remove line endings
            line = line.rstrip('\n')
            line = line.rstrip('\r')
            ret.append(line)
            if len(ret) == 3: # test for the first 3 rows(should be enough)
                break

        sep_count = {}
        if len(ret) == 3:
            for separator in separators:
                v1 = ret[0].count(separator)
                v2 = ret[1].count(separator)
                v3 = ret[2].count(separator)
                if v1==0 or v2==0 or v3==0:
                    continue
                if v1==v2 and v2==v3:
                    sep_count[separator] = v1

        # just pick the first one if nothing matched
        if len(sep_count)==0:
            self.separator = separators[0]
            return
        max = 0
        for key in sep_count:
            if sep_count[key] > max:
                max = sep_count[key]
                sep = key
        self.separator = sep
        return


class XlsxImporter(ContactLineImporter):
    """
    Importer for XLSX
    """

    def set_file_lines(self, *args, **kwargs):

        for row in self._read_row(self._file_path):
            self._file_rows.append(row)


    def _read_row(self, file_path):

        from openpyxl.reader.excel import load_workbook
        
        wb = load_workbook(filename = file_path, use_iterators = True)
        ws = wb.get_active_sheet()

        for row in ws.iter_rows():
            values = []
            for cell in row:

                value = cell.internal_value
                if  value is None:
                    value = ''

                values.append(value)

            yield values




class XlsImporter(ContactLineImporter):
    """
    Importer for XLSX
    """

    def set_file_lines(self, *args, **kwargs):

        for row in self._read_row(self._file_path):
            self._file_rows.append(row)

    def _read_row(self, file_path):

        # xls file is in old format
        # http://scienceoss.com/readexcelfilesfrompython/

        import xlrd
        from xlrd.sheet import Cell, ctype_text
        from xlrd.xldate import xldate_as_tuple

        wb = xlrd.open_workbook(file_path)
        sh = wb.sheet_by_index(0)

        for rownum in range(sh.nrows):

            values = sh.row_values(rownum)
            newrow = []

            for colnum in range(len(values)):
                c = sh.row(rownum)[colnum]

                if ctype_text[c.ctype] == 'xldate':
                    y, m, d, h, mn, s  = xldate_as_tuple(c.value, wb.datemode)
                    dt = datetime.datetime(y, m, d, h, mn, s)
                    newrow.append(dt)

                else:
                    newrow.append(c.value)

            yield newrow


