# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

"""
Task for Importing contacts from large files saved in ImportFile
"""

from sqlalchemy import and_
from basesoft.scheduler.task import ScheduleTask
from basesoft.web.contrib.contact.controllers import BackgroundImportContactController, SimpleContactController
from basesoft.web.contrib.file.models import ImportFile, ImportFileType
from basesoft.web.contrib.file.controllers import ImportFileController

from basesoft import logging
logger = logging.getLogger('basesoft.scheduler')

class ImportContactsTask(ScheduleTask):

    INTERVAL = 57

    def execute(self):
        """
        Imports contacts from large files saved in ImportFile
        """
        results = self.sa_session.query(ImportFile).join(ImportFileType).filter(and_(
                    ImportFileType.name=='basesoft.web.contrib.contact',
                    ImportFile.status==ImportFile.STATUS_NEW,
                    ImportFile.deleted==False,
                    ImportFile.archived==False)).all()

        if len(results) == 0:
            return

        bc = BackgroundImportContactController(self.sa_session)
        impc = ImportFileController(self.sa_session)

        # since multiple files could be already in the query result, update the status for all of them now
        # (because saving contacts could take a while) so they won't be selected next time
        for file in results:
            impc.set_data(file, ImportFile.STATUS_STARTED)

        for file in results:
            try:
                contacts, validation_file = bc.import_contact_file(file, loop_validator=self.loop_validator)
                impc.set_data(file, ImportFile.STATUS_SUCCESS, len(contacts), validation_file)
            except Exception, e:
                impc.set_data(file, ImportFile.STATUS_FAILURE, e.message)
                raise

class AutomatedImportTask(ScheduleTask):
    INTERVAL = 600

    def execute(self):
        bc = BackgroundImportContactController(self.sa_session)
        logger.info('Scanning for background imports')
        bc.scan_for_files()

class TempCompanyGarbageCollectorTask(ScheduleTask):
    INTERVAL = 86400 # once a day

    def execute(self):
        scc = SimpleContactController(self.sa_session)
        scc.cleanup_temporary_companies()

# publish task so the scheduler can pick them up
tasks = (ImportContactsTask, AutomatedImportTask, TempCompanyGarbageCollectorTask, )



