# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.file.controllers import load_import_file_type_map
from sqlalchemy.orm import aliased
from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel

from basesoft.web.contrib.file.models import File, FileCategory, ImportFile, ImportFileType

class FileCustomQuery(CustomQuery):
    def tables(self):
        return (
            CustomQueryModel(File),
            CustomQueryModel(FileCategory, onclause=File.category_id==FileCategory.id),
        )

    def fields(self):
        return (
            CustomQueryField(File.id, _('ID'), key='id'),
            CustomQueryField(File.file_name, _('Name'), key='file_name'),
            CustomQueryField(File.file_hash, _('Hash'), key='file_hash'),
            CustomQueryField(File.polymorphic_type, _('Type')),
            CustomQueryField(FileCategory.id, _('Category'), in_select=False, is_filter=True, filter_options=FileCategory.label),
            CustomQueryField(FileCategory.label, _('Category'), is_group_by=True),
            CustomQueryField(File.created, _('Created'), is_sorted=True, sorted_asc=False),
        )

ValidationFile = aliased(File)

class AbstractFileImportCustomQuery(CustomQuery):
    _file_type_mapping = None

    def tables(self):
        return (
            CustomQueryModel(ImportFile),
            CustomQueryModel(ImportFileType),
            CustomQueryModel(ValidationFile, onclause=ImportFile.failure_file_id == ValidationFile.id, priv=None, use_outerjoin=True),
        )

    def fields(self):
        if self._file_type_mapping is None:
            self._file_type_mapping = load_import_file_type_map()

        fo = []
        for ft, c in self._file_type_mapping.iteritems():
            fo.append({
                'value': ft,
                'label': c.IMPORT_FILE_TYPE_I18N
            })

        fo = sorted(fo, key=lambda k: k['label'])

        return (
            CustomQueryField(ImportFile.id, _('ID'), key='id'),
            CustomQueryField(ImportFile.file_name, _('File name'), is_filter=True, key='file_name'),
            CustomQueryField(ImportFile.file_hash, _('File hash'), key='file_hash'),
            CustomQueryField(ImportFileType.name, _('File type'), key='file_type', is_filter=True, filter_options=fo),
            CustomQueryField(ImportFile.status, _('Status'), is_filter=True, key='status'),
            CustomQueryField(ImportFile.result, _('Comment'), key='result'),
            CustomQueryField(ImportFile.id, _('Settings'), key='settings'),
            CustomQueryField(ValidationFile.file_name, _('Validation file'), is_filter=True, key='validation_file'),
            CustomQueryField(ValidationFile.file_hash, _('Validation file hash'), key='validation_file_hash'),
            CustomQueryField(ValidationFile.file_hash, _('View validation file'), key='validation_file_inline'),
            CustomQueryField(ImportFile.created, _('Created'), key='created'),
            CustomQueryField(ImportFile.last_updated, _('Last updated'), key='last_updated'),
        )

    def set_value_processors(self):

        if self._file_type_mapping is None:
            self._file_type_mapping = load_import_file_type_map()

        def proc(row, key):
            value = getattr(row, key)
            if not self._file_type_mapping.has_key(value):
                return '-'
            return self._file_type_mapping[value].IMPORT_FILE_TYPE_I18N

        self._value_processors['file_type'] = proc

    def set_where(self):
        self.where = ImportFileType.name==self.IMPORT_FILE_TYPE

    order_by = (ImportFile.id.desc(), )

custom_queries = (FileCustomQuery, )
