# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.notification.models import Notification

from django.utils.translation import ugettext_lazy as _
from basesoft.web.core.models import PrivilegeObject

class FileTmp(object):
    """
    Model representing an "unprocessed" file.
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TmpFile hash="%s">' % self.file_hash

class File(PrivilegeObject):
    """
    Model representing a file
    @DynamicAttrs
    """

    NAME_I18N = _('File')

    MIMETYPES = None

    mark_as_deleted_permission = 'delete_file'
    mark_as_archived_permission = 'update_file'

    _name_property = 'file_name'

    def get_group_object(self):
        return (FileGroup, 'file')

    def cleanup_garbage(self):
        # TODO: remove file from the OS. - #3182
        pass

    def __repr__(self):
        return '<File id=%s>' % self.id


class FileGroup(object):
    """
    Model representing the link between files and groups. Used for permissions.
    @DynamicAttrs
    """

class FileCategory(PrivilegeObject):
    """
    Categorization of Files. Group based object for privileges checking.
    @DynamicAttrs
    """
    _name_property = 'label'

    def get_group_object(self):
        return (FileCategoryGroup, 'category')

    def __repr__(self):
        return '<FileCategory id=%s>' % self.id

class FileCategoryGroup(object):
    """
    Model representing the link between file categories and groups.
    @DynamicAttrs
    """

class ImageFile(File):
    """
    @DynamicAttrs
    """
    NAME_I18N = _('Image file')

    MIMETYPES = ['image/jpeg', 'image/png', 'image/gif']

class ExcelFile(File):
    """
    @DynamicAttrs
    """
    NAME_I18N = _('Excel file')

    MIMETYPES = ['application/excel', 'application/vnd.ms-excel', 'application/x-excel', 'application/x-msexcel']

class ImportFileType(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<ImportFileType id=%s>' % self.id

class ImportFile(File):
    """
    Model representing a file for importing in the background
    @DynamicAttrs
    """

    NAME_I18N = _('ImportFile')

    # statuses for the import for the contacts in the file
    STATUS_NEW = 0
    STATUS_STARTED = 1
    STATUS_SUCCESS = 2
    STATUS_FAILURE = 3
    STATUS_DO_NOT_PROCESS = 4

    def __repr__(self):
        return '<ImportFile id=%s>' % self.id

class FileImportNotification(Notification):
    name_i18n = _('File import')
