# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
import os
from basesoft.web.core.exceptions import PublicSecurityException
from django.conf import settings

from basesoft.web.contrib.file.controllers import ProcessFileController, CSVFileController, ImportFileController


class FileRpcHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.user_id = request.session['user_id']
        self.request = request
        self.controller = ProcessFileController(self.sa_session, account_id=self.user_id, request=request)

    def get_csv_lines(self, hash):
        f = self.controller.get_file_by_hash(hash)
        if f is None:
            raise PublicSecurityException('No access to the requested file.')
        path = settings.FILE_REPOSITORY
        file_path = os.path.join(path, f.file_hash)
        c = CSVFileController(file_path)
        return {'file_name': f.file_name, 'lines': c.get_file_rows()}

    def get_import_file_setting(self, file_id):
        ifc = ImportFileController(self.sa_session, account_id=self.user_id, request=self.request)
        return ifc.get_import_file_setting(self.controller.get_files(file_id).pop())

handlers = (FileRpcHandler, )
