# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the method to remove any old unused(unprocessed) uploaded file.
It starts every hour and removes all unused uploaded files(in tblFileTmp) that are
older than 24 hours.
"""

import commands
import datetime

from django.conf import settings

from basesoft.web.contrib.file.controllers import UploadFileController, ImportFileController
from basesoft.web.contrib.file.models import FileTmp, ImportFile, ImportFileType
from basesoft.scheduler.task import ScheduleTask

from basesoft import logging
from sqlalchemy.sql.expression import and_

from basesoft.web.core.exceptions import PublicSystemException
from django.utils import simplejson

logger = logging.getLogger('basesoft.scheduler')

class RemoveTempFileTask(ScheduleTask):

    INTERVAL = 3600 * 4

    def execute(self):
        """
        Remove files from db (tblFileTmp) and disk that are older than 24 hours.
        """

        # delete all files that are older than 24h
        # http://www.howtogeek.com/howto/ubuntu/delete-files-older-than-x-days-on-linux/
        cmd = "/usr/bin/find %s -maxdepth 1 -mtime +1 -exec rm {} \;" % settings.TMP_FILE_REPOSITORY
        status, output = commands.getstatusoutput(cmd)
        if status != 0:
            raise Exception('error while deleting files: %s' % output)

        # get all files older than 24 hours
        files = self.sa_session.query(FileTmp).filter(FileTmp.created<=datetime.datetime.now()-datetime.timedelta(days=1)).all()
        if len(files):
            logger.info('RemoveUploadedTempFileTask: removing %s uploaded temp files' % len(files))

            # remove them using the controller method
            controller = UploadFileController(self.sa_session, None)
            controller.delete_temp_files(files)

        return None


class AbstractImportFileTask(ScheduleTask):
    importer = None
    controller = None

    def execute(self):

        # get the saved file
        files = self.get_controller_and_files()
        if len(files) == 0:
            return

        c = ImportFileController(self.sa_session)
        for f in files:
            c.set_data(f, ImportFile.STATUS_STARTED)

        for f in files:
            settings = simplejson.loads(f.settings)
            account_id = None if not settings.has_key('account_id') else settings['account_id']
            controller = self.controller(self.sa_session, account_id=account_id)
            importer = self.importer.get_importer(f)(self.sa_session)
            try:
                rv, validation_file = importer.background_import_file(f, controller, False, **settings)
                cb = importer.get_notification_callback()
                text = importer.get_notification_text()
                subject = importer.get_notification_subject()
                notify = importer.get_notification_update_notify()
                importer.set_data(f, ImportFile.STATUS_SUCCESS, len(rv), validation_file, cb, subject=subject, text=text, notify=notify)
            except Exception, e:
                cb = importer.get_failure_notification_callback()
                importer.set_data(f, ImportFile.STATUS_FAILURE, 'Something went wrong, our engineers have been notified of this problem and will work to resolve it.', callback=cb)
                raise

        return None

    def get_controller_and_files(self):
        files = self.sa_session.query(ImportFile).join(ImportFileType).filter(and_(
                    ImportFileType.name == self.importer.IMPORT_FILE_TYPE,
                    ImportFile.status == ImportFile.STATUS_NEW,
                    ImportFile.deleted == False,
                    ImportFile.archived == False)).all()
        return files

# publish task so the scheduler can pick them up
tasks = (RemoveTempFileTask,)