# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _


"""
Internationalisation stuff. This includes locales, countries and regions.
"""

# settings for decimal separator
DECIMAL_SEPARATOR_LOCALE = 1  # use the default separator set by the locale.
DECIMAL_SEPARATOR_DOT = 2  # use a . as decimal separator
DECIMAL_SEPARATOR_COMMA = 3  # use a , as decimal separator

DECIMAL_SEPARATOR_MAP = [
    {'value': DECIMAL_SEPARATOR_LOCALE, 'label': _('Use default')},
    {'value': DECIMAL_SEPARATOR_DOT, 'label': '.'},
    {'value': DECIMAL_SEPARATOR_COMMA, 'label': ','},
]

DECIMAL_SEPARATOR_CHAR = [
    None,
    '.',
    ',',
]

# settings for thousands separator
THOUSANDS_SEPARATOR_LOCALE = 1  # use the default separator set by the locale.
THOUSANDS_SEPARATOR_DOT = 2  # use a . as thousands separator
THOUSANDS_SEPARATOR_COMMA = 3  # use a , as thousands separator
THOUSANDS_SEPARATOR_SPACE = 4  # use a space as thousands separator
THOUSANDS_SEPARATOR_APOSTROPHE = 5  # use a ' as thousands separator (Switzerland)
THOUSANDS_SEPARATOR_NONE = 6  # don't separate groups of thousands

THOUSANDS_SEPARATOR_MAP = [
    {'value': THOUSANDS_SEPARATOR_LOCALE, 'label': _('Use default')},
    {'value': THOUSANDS_SEPARATOR_DOT, 'label': '.'},
    {'value': THOUSANDS_SEPARATOR_COMMA, 'label': ','},
    {'value': THOUSANDS_SEPARATOR_SPACE, 'label': _('Space')},
    {'value': THOUSANDS_SEPARATOR_APOSTROPHE, 'label': "'"},
    {'value': THOUSANDS_SEPARATOR_NONE, 'label': _('None')},
]

THOUSAND_SEPARATOR_CHAR = [
    None,
    '.',
    ',',
    ' ',
    "'",
    ''
]

# date formatting options
DATE_FORMATTING_LOCALE = 1  # use the default date formatting set by the locale
DATE_FORMATTING_EN_GB = 2  # similar setup as en_GB
DATE_FORMATTING_NL_NL = 3  # similar setup as nl_NL
DATE_FORMATTING_DE_DE = 4  # similar setup as de_DE
DATE_FORMATTING_EN_US = 5  # similar setup as en_US

DATE_FORMATTING_MAP = [
    {'value': DATE_FORMATTING_LOCALE, 'label': _('Use default')},
    {'value': DATE_FORMATTING_EN_GB, 'label': 'dd/mm/yyyy'},
    {'value': DATE_FORMATTING_NL_NL, 'label': 'dd-mm-yyyy'},
    {'value': DATE_FORMATTING_DE_DE, 'label': 'dd.mm.yyyy'},
    {'value': DATE_FORMATTING_EN_US, 'label': 'mm/dd/yyyy'},
]

# TODO time formatting options
TIME_FORMATTING_LOCALE = 1  # use the default time formatting set by the locale

TIME_FORMATTING_MAP = {
    TIME_FORMATTING_LOCALE: _('Use default'),
}

from basesoft.web.core import L10n_settings as l10n
__MONTHS_LONG = l10n.default.MONTHS_LONG
MONTHS_MAP = [ {'value': i+1, 'label': __MONTHS_LONG[i]} for i in range(len(__MONTHS_LONG)) ]
