# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy, ugettext as _
from basesoft.web.core.input.validation.plugins import Validation
from basesoft.web.core.config import DefaultConfig
from basesoft.web.contrib import i18n


class SeparatorValidation(Validation):
    def check(self, params={}):
        if self._value in [None, '']:
            return
        if not params.has_key('other'):
            return

        self._value = int(self._value)

        other = params['other']

        # both set to locale all good.
        if other == 1 and self._value == 1:
            return

        if other == self._value:
            raise ValidationError(_('Decimal and thousand separators cannot be the same.'))

        # both not set to locale
        if other != 1 and self._value != 1:
            return

        raise ValidationError(_('Both decimal and thousand separators need to be set.'))

class I18nConfig(DefaultConfig):
    permissions = None
    label = ugettext_lazy('Language & formatting settings')
    is_expanded = True

    def configs(self):
        return [{
            'name': 'basesoft.web.contrib.locale',
            'label': _('Select language'),
            'options': {
                'optionsRpcUrl': '/basesoft.web.contrib.i18n/rpc/',
                'optionsRpcMethod': 'get_site_translations_locales',
                'emptyFirstOption': False
            }
        }, {
            'name': 'basesoft.web.contrib.i18n.date.format',
            'label': _('Date format'),
            'options': {
                'options': i18n.DATE_FORMATTING_MAP
            }
        }, {
            'name': 'basesoft.web.contrib.i18n.number.decimal.separator',
            'label': _('Decimal separator'),
            'help': _('The character used as separator for decimal numbers (Radix character).'),
            'options': {
                'options': i18n.DECIMAL_SEPARATOR_MAP
            }
        }, {
            'name': 'basesoft.web.contrib.i18n.number.thousands.separator',
            'label': _('Thousands separator'),
            'help': _('The character used to separate groups of thousands.'),
            'options': {
                'options': i18n.THOUSANDS_SEPARATOR_MAP
            }
        }]

    def get_form_behaviour(self, validators, values):

        dec_sep = values.get('basesoft.web.contrib.i18n.number.decimal.separator', None)
        thousand_sep = values.get('basesoft.web.contrib.i18n.number.thousands.separator', None)

        if dec_sep == '' or thousand_sep == '':
            return validators

        if dec_sep is None and thousand_sep is None:
            return validators

        if dec_sep is not None:
            dec_sep = int(dec_sep)
        if thousand_sep is not None:
            thousand_sep = int(thousand_sep)

        validators['basesoft.web.contrib.i18n.number.decimal.separator'] = [(SeparatorValidation, {'other': thousand_sep}, )]
        validators['basesoft.web.contrib.i18n.number.thousands.separator'] = [(SeparatorValidation, {'other': dec_sep}, )]

        return validators


configs = (I18nConfig(), )