# -*- coding: utf-8 -*-

from basesoft.web.contrib.i18n.models import Locale, Country, Language
from sqlalchemy.orm.exc import NoResultFound

def sync(sa_session):

    sync_locales(sa_session)

    sync_languages(sa_session)

    sync_countries(sa_session)

def sync_locales(sa_session):
    """
    syncs locales to the db
    notice: does not delete
    """

    locales = [
        {'id': 1, 'name' :'English', 'locale': 'en_GB', 'availability': 3},
        {'id': 2, 'name' :'Dutch', 'locale': 'nl_NL', 'availability': 3},
        {'id': 3, 'name' :'German', 'locale': 'de_DE', 'availability': 1},
        {'id': 4, 'name' :'French', 'locale': 'fr_FR', 'availability': 1},
        {'id': 5, 'name' :'French (Belgium)', 'locale': 'fr_BE', 'availability': 1},
        {'id': 6, 'name' :'Dutch (Belgium)', 'locale': 'nl_BE', 'availability': 1},
        {'id': 7, 'name' :'English (US)', 'locale': 'en_US', 'availability': 1},
        {'id': 8, 'name' :'Chinese (PRC)', 'locale': 'zh_CN', 'availability': 1},
        {'id': 9, 'name' :'Chinese (Taiwan)', 'locale': 'zh_TW', 'availability': 1},
        {'id': 10, 'name' :'Czech (Czech Republic)', 'locale': 'cs_CZ', 'availability': 1},
        {'id': 11, 'name' :'English (Australia)', 'locale': 'en_AU', 'availability': 1},
        {'id': 12, 'name' :'English (Canada)', 'locale': 'en_CA', 'availability': 1},
        {'id': 13, 'name' :'English (New Zealand)', 'locale': 'en_NZ', 'availability': 1},
        {'id': 14, 'name' :'English (Singapor)', 'locale': 'en_SG', 'availability': 1},
        {'id': 15, 'name' :'French (Canada)', 'locale': 'fr_CA', 'availability': 1},
        {'id': 16, 'name' :'French (Switzerland)', 'locale': 'fr_CH', 'availability': 1},
        {'id': 17, 'name' :'German (Austria)', 'locale': 'de_AT', 'availability': 1},
        {'id': 18, 'name' :'German (Liechtenstein)', 'locale': 'de_LI', 'availability': 1},
        {'id': 19, 'name' :'German (Switzerland)', 'locale': 'de_CH', 'availability': 1},
        {'id': 20, 'name' :'Italian (Italy)', 'locale': 'it_IT', 'availability': 1},
        {'id': 21, 'name' :'Italian (Switzerland)', 'locale': 'it_CH', 'availability': 1},
        {'id': 22, 'name' :'Japanese (Japan)', 'locale': 'ja_JP', 'availability': 1},
        {'id': 23, 'name' :'Korean (Korea)', 'locale': 'ko_KR', 'availability': 1},
        {'id': 24, 'name' :'Polish (Poland)', 'locale': 'pl_PL', 'availability': 1},
        {'id': 25, 'name' :'Russian (Russia)', 'locale': 'ru_RU', 'availability': 1},
        {'id': 26, 'name' :'Spanish (Spain)', 'locale': 'es_ES', 'availability': 1},
        {'id': 27, 'name' :'Arabic (Egypt)', 'locale': 'ar_EG', 'availability': 1},
        {'id': 28, 'name' :'Arabic (Israel)', 'locale': 'ar_IL', 'availability': 1},
        {'id': 29, 'name' :'Bulgarian (Bulgaria)', 'locale': 'bg_BG', 'availability': 1},
        {'id': 30, 'name' :'Catalan (Spain)', 'locale': 'ca_ES', 'availability': 1},
        {'id': 31, 'name' :'Croatian (Croatia)', 'locale': 'hr_HR', 'availability': 1},
        {'id': 32, 'name' :'Danish (Denmar)', 'locale': 'da_DK', 'availability': 1},
        {'id': 33, 'name' :'English (India)', 'locale': 'en_IN', 'availability': 1},
        {'id': 34, 'name' :'English (Ireland)', 'locale': 'en_IE', 'availability': 1},
        {'id': 35, 'name' :'English (Zimbabwe)', 'locale': 'en_ZA', 'availability': 1},
        {'id': 36, 'name' :'Finnish (Finland)', 'locale': 'fi_FI', 'availability': 1},
        {'id': 37, 'name' :'Greek (Greece)', 'locale': 'el_GR', 'availability': 1},
        {'id': 38, 'name' :'Hebrew (Israel)', 'locale': 'iw_IL', 'availability': 1},
        {'id': 39, 'name' :'Hindi (India)', 'locale': 'hi_IN', 'availability': 1},
        {'id': 40, 'name' :'Hungarian (Hungary)', 'locale': 'hu_HU', 'availability': 1},
        {'id': 41, 'name' :'Indonesian (Indonesia)', 'locale': 'in_ID', 'availability': 1},
        {'id': 42, 'name' :'Latvian (Latvia)', 'locale': 'lv_LV', 'availability': 1},
        {'id': 43, 'name' :'Lithuanian (Lithuania)', 'locale': 'lt_LT', 'availability': 1},
        {'id': 44, 'name' :'Norwegian-Bokmol (Norwa)', 'locale': 'nb_NO', 'availability': 1},
        {'id': 45, 'name' :'Portuguese (Brazil)', 'locale': 'pt_BR', 'availability': 1},
        {'id': 46, 'name' :'Portuguese (Portugal)', 'locale': 'pt_PT', 'availability': 1},
        {'id': 47, 'name' :'Romanian (Romania)', 'locale': 'ro_RO', 'availability': 1},
        {'id': 48, 'name' :'Serbian (Serbia)', 'locale': 'sr_RS', 'availability': 1},
        {'id': 49, 'name' :'Slovak (Slovakia)', 'locale': 'sk_SK', 'availability': 1},
        {'id': 50, 'name' :'Slovenian (Slovenia)', 'locale': 'sl_SI', 'availability': 1},
        {'id': 51, 'name' :'Spanish (US)', 'locale': 'es_US', 'availability': 1},
        {'id': 52, 'name' :'Swedish (Sweden)', 'locale': 'sv_SE', 'availability': 1},
        {'id': 53, 'name' :'Tagalog (Philippines)', 'locale': 'tl_PH', 'availability': 1},
        {'id': 54, 'name' :'Thai (Thailand)', 'locale': 'th_TH', 'availability': 1},
        {'id': 55, 'name' :'Turkish (Turkey)', 'locale': 'tr_TR', 'availability': 1},
        {'id': 56, 'name' :'Ukrainian (Ukraine)', 'locale': 'uk_UA', 'availability': 1},
        {'id': 57, 'name' :'Vietnamese (Vietnam)', 'locale': 'vi_VN', 'availability': 1},
    ]

    for l_obj in locales:

        try:
            # id is leading
            l = sa_session.query(Locale).filter(Locale.id == l_obj['id']).one()

            # error if the locale mismatches. manual db adjustments should be made
            if l.locale != l_obj['locale']:
                raise Exception('locale mismatch: %s vs %s' % (l.id, l_obj) )

        except NoResultFound:

            print 'Creating new locale: %s' % l_obj['name']

            # make a new locale
            l = Locale(l_obj['name'], l_obj['locale'])
            l.id = l_obj['id']

        l.name = l_obj['name']
        l.availability = l_obj['availability']

        sa_session.add(l)
    sa_session.flush()

def sync_countries(sa_session):
    """
    syncs countries to the db
    notice: does not delete
    """

    countries = [
        {'id':'1','name':'Afghanistan','iso_3166_a2':'AF','iso_3166_a3':'AFG'},
        {'id':'2','name':'Åland Islands','iso_3166_a2':'AX','iso_3166_a3':'ALA'},
        {'id':'3','name':'Albania','iso_3166_a2':'AL','iso_3166_a3':'ALB'},
        {'id':'4','name':'Algeria','iso_3166_a2':'DZ','iso_3166_a3':'DZA'},
        {'id':'5','name':'American Samoa','iso_3166_a2':'AS','iso_3166_a3':'ASM'},
        {'id':'6','name':'Andorra','iso_3166_a2':'AD','iso_3166_a3':'AND'},
        {'id':'7','name':'Angola','iso_3166_a2':'AO','iso_3166_a3':'AGO'},
        {'id':'8','name':'Anguilla','iso_3166_a2':'AI','iso_3166_a3':'AIA'},
        {'id':'9','name':'Antarctica','iso_3166_a2':'AQ','iso_3166_a3':'ATA'},
        {'id':'10','name':'Antigua and Barbuda','iso_3166_a2':'AG','iso_3166_a3':'ATG'},
        {'id':'11','name':'Argentina','iso_3166_a2':'AR','iso_3166_a3':'ARG'},
        {'id':'12','name':'Armenia','iso_3166_a2':'AM','iso_3166_a3':'ARM'},
        {'id':'13','name':'Aruba','iso_3166_a2':'AW','iso_3166_a3':'ABW'},
        {'id':'14','name':'Australia','iso_3166_a2':'AU','iso_3166_a3':'AUS'},
        {'id':'15','name':'Austria','iso_3166_a2':'AT','iso_3166_a3':'AUT'},
        {'id':'16','name':'Azerbaijan','iso_3166_a2':'AZ','iso_3166_a3':'AZE'},
        {'id':'17','name':'Bahamas','iso_3166_a2':'BS','iso_3166_a3':'BHS'},
        {'id':'18','name':'Bahrain','iso_3166_a2':'BH','iso_3166_a3':'BHR'},
        {'id':'19','name':'Bangladesh','iso_3166_a2':'BD','iso_3166_a3':'BGD'},
        {'id':'20','name':'Barbados','iso_3166_a2':'BB','iso_3166_a3':'BRB'},
        {'id':'21','name':'Belarus','iso_3166_a2':'BY','iso_3166_a3':'BLR'},
        {'id':'22','name':'Belgium','iso_3166_a2':'BE','iso_3166_a3':'BEL'},
        {'id':'23','name':'Belize','iso_3166_a2':'BZ','iso_3166_a3':'BLZ'},
        {'id':'24','name':'Benin','iso_3166_a2':'BJ','iso_3166_a3':'BEN'},
        {'id':'25','name':'Bermuda','iso_3166_a2':'BM','iso_3166_a3':'BMU'},
        {'id':'26','name':'Bhutan','iso_3166_a2':'BT','iso_3166_a3':'BTN'},
        {'id':'27','name':'Bolivia','iso_3166_a2':'BO','iso_3166_a3':'BOL'},
        {'id':'28','name':'Bosnia and Herzegovina','iso_3166_a2':'BA','iso_3166_a3':'BIH'},
        {'id':'29','name':'Botswana','iso_3166_a2':'BW','iso_3166_a3':'BWA'},
        {'id':'30','name':'Bouvet Island','iso_3166_a2':'BV','iso_3166_a3':'BVT'},
        {'id':'31','name':'Brazil','iso_3166_a2':'BR','iso_3166_a3':'BRA'},
        {'id':'32','name':'British Indian Ocean Territory','iso_3166_a2':'IO','iso_3166_a3':'IOT'},
        {'id':'33','name':'Brunei Darussalam','iso_3166_a2':'BN','iso_3166_a3':'BRN'},
        {'id':'34','name':'Bulgaria','iso_3166_a2':'BG','iso_3166_a3':'BGR'},
        {'id':'35','name':'Burkina Faso','iso_3166_a2':'BF','iso_3166_a3':'BFA'},
        {'id':'36','name':'Burundi','iso_3166_a2':'BI','iso_3166_a3':'BDI'},
        {'id':'37','name':'Cambodia','iso_3166_a2':'KH','iso_3166_a3':'KHM'},
        {'id':'38','name':'Cameroon','iso_3166_a2':'CM','iso_3166_a3':'CMR'},
        {'id':'39','name':'Canada','iso_3166_a2':'CA','iso_3166_a3':'CAN'},
        {'id':'40','name':'Cape Verde','iso_3166_a2':'CV','iso_3166_a3':'CPV'},
        {'id':'41','name':'Cayman Islands','iso_3166_a2':'KY','iso_3166_a3':'CYM'},
        {'id':'42','name':'Central African Republic','iso_3166_a2':'CF','iso_3166_a3':'CAF'},
        {'id':'43','name':'Chad','iso_3166_a2':'TD','iso_3166_a3':'TCD'},
        {'id':'44','name':'Chile','iso_3166_a2':'CL','iso_3166_a3':'CHL'},
        {'id':'45','name':'China','iso_3166_a2':'CN','iso_3166_a3':'CHN'},
        {'id':'46','name':'Christmas Island','iso_3166_a2':'CX','iso_3166_a3':'CXR'},
        {'id':'47','name':'Cocos (Keeling) Islands','iso_3166_a2':'CC','iso_3166_a3':'CCK'},
        {'id':'48','name':'Colombia','iso_3166_a2':'CO','iso_3166_a3':'COL'},
        {'id':'49','name':'Comoros','iso_3166_a2':'KM','iso_3166_a3':'COM'},
        {'id':'50','name':'Congo','iso_3166_a2':'CG','iso_3166_a3':'COG'},
        {'id':'51','name':'Democratic Republic of the Congo','iso_3166_a2':'CD','iso_3166_a3':'COD'},
        {'id':'52','name':'Cook Islands','iso_3166_a2':'CK','iso_3166_a3':'COK'},
        {'id':'53','name':'Costa Rica','iso_3166_a2':'CR','iso_3166_a3':'CRI'},
        {'id':'54','name':'Côte d\'Ivoire','iso_3166_a2':'CI','iso_3166_a3':'CIV'},
        {'id':'55','name':'Croatia','iso_3166_a2':'HR','iso_3166_a3':'HRV'},
        {'id':'56','name':'Cuba','iso_3166_a2':'CU','iso_3166_a3':'CUB'},
        {'id':'57','name':'Cyprus','iso_3166_a2':'CY','iso_3166_a3':'CYP'},
        {'id':'58','name':'Czech Republic','iso_3166_a2':'CZ','iso_3166_a3':'CZE'},
        {'id':'59','name':'Denmark','iso_3166_a2':'DK','iso_3166_a3':'DNK'},
        {'id':'60','name':'Djibouti','iso_3166_a2':'DJ','iso_3166_a3':'DJI'},
        {'id':'61','name':'Dominica','iso_3166_a2':'DM','iso_3166_a3':'DMA'},
        {'id':'62','name':'Dominican Republic','iso_3166_a2':'DO','iso_3166_a3':'DOM'},
        {'id':'63','name':'Ecuador','iso_3166_a2':'EC','iso_3166_a3':'ECU'},
        {'id':'64','name':'Egypt','iso_3166_a2':'EG','iso_3166_a3':'EGY'},
        {'id':'65','name':'El Salvador','iso_3166_a2':'SV','iso_3166_a3':'SLV'},
        {'id':'66','name':'Equatorial Guinea','iso_3166_a2':'GQ','iso_3166_a3':'GNQ'},
        {'id':'67','name':'Eritrea','iso_3166_a2':'ER','iso_3166_a3':'ERI'},
        {'id':'68','name':'Estonia','iso_3166_a2':'EE','iso_3166_a3':'EST'},
        {'id':'69','name':'Ethiopia','iso_3166_a2':'ET','iso_3166_a3':'ETH'},
        {'id':'70','name':'Falkland Islands (Malvinas)','iso_3166_a2':'FK','iso_3166_a3':'FLK'},
        {'id':'71','name':'Faeroe Islands','iso_3166_a2':'FO','iso_3166_a3':'FRO'},
        {'id':'72','name':'Fiji','iso_3166_a2':'FJ','iso_3166_a3':'FJI'},
        {'id':'73','name':'Finland','iso_3166_a2':'FI','iso_3166_a3':'FIN'},
        {'id':'74','name':'France','iso_3166_a2':'FR','iso_3166_a3':'FRA'},
        {'id':'75','name':'French Guiana','iso_3166_a2':'GF','iso_3166_a3':'GUF'},
        {'id':'76','name':'French Polynesia','iso_3166_a2':'PF','iso_3166_a3':'PYF'},
        {'id':'77','name':'French Southern Territories','iso_3166_a2':'TF','iso_3166_a3':'ATF'},
        {'id':'78','name':'Gabon','iso_3166_a2':'GA','iso_3166_a3':'GAB'},
        {'id':'79','name':'Gambia','iso_3166_a2':'GM','iso_3166_a3':'GMB'},
        {'id':'80','name':'Georgia','iso_3166_a2':'GE','iso_3166_a3':'GEO'},
        {'id':'81','name':'Germany','iso_3166_a2':'DE','iso_3166_a3':'DEU'},
        {'id':'82','name':'Ghana','iso_3166_a2':'GH','iso_3166_a3':'GHA'},
        {'id':'83','name':'Gibraltar','iso_3166_a2':'GI','iso_3166_a3':'GIB'},
        {'id':'84','name':'Greece','iso_3166_a2':'GR','iso_3166_a3':'GRC'},
        {'id':'85','name':'Greenland','iso_3166_a2':'GL','iso_3166_a3':'GRL'},
        {'id':'86','name':'Grenada','iso_3166_a2':'GD','iso_3166_a3':'GRD'},
        {'id':'87','name':'Guadeloupe','iso_3166_a2':'GP','iso_3166_a3':'GLP'},
        {'id':'88','name':'Guam','iso_3166_a2':'GU','iso_3166_a3':'GUM'},
        {'id':'89','name':'Guatemala','iso_3166_a2':'GT','iso_3166_a3':'GTM'},
        {'id':'90','name':'Guernsey','iso_3166_a2':'GG','iso_3166_a3':'GGY'},
        {'id':'91','name':'Guinea','iso_3166_a2':'GN','iso_3166_a3':'GIN'},
        {'id':'92','name':'Guinea-Bissau','iso_3166_a2':'GW','iso_3166_a3':'GNB'},
        {'id':'93','name':'Guyana','iso_3166_a2':'GY','iso_3166_a3':'GUY'},
        {'id':'94','name':'Haiti','iso_3166_a2':'HT','iso_3166_a3':'HTI'},
        {'id':'95','name':'Heard Island and Mcdonald Islands','iso_3166_a2':'HM','iso_3166_a3':'HMD'},
        {'id':'96','name':'Holy See','iso_3166_a2':'VA','iso_3166_a3':'VAT'},
        {'id':'97','name':'Honduras','iso_3166_a2':'HN','iso_3166_a3':'HND'},
        {'id':'98','name':'Hong Kong Special Administrative Region of China','iso_3166_a2':'HK','iso_3166_a3':'HKG'},
        {'id':'99','name':'Hungary','iso_3166_a2':'HU','iso_3166_a3':'HUN'},
        {'id':'100','name':'Iceland','iso_3166_a2':'IS','iso_3166_a3':'ISL'},
        {'id':'101','name':'India','iso_3166_a2':'IN','iso_3166_a3':'IND'},
        {'id':'102','name':'Indonesia','iso_3166_a2':'ID','iso_3166_a3':'IDN'},
        {'id':'103','name':'Iran, Islamic Republic of','iso_3166_a2':'IR','iso_3166_a3':'IRN'},
        {'id':'104','name':'Iraq','iso_3166_a2':'IQ','iso_3166_a3':'IRQ'},
        {'id':'105','name':'Ireland','iso_3166_a2':'IE','iso_3166_a3':'IRL'},
        {'id':'106','name':'Isle of Man','iso_3166_a2':'IM','iso_3166_a3':'IMN'},
        {'id':'107','name':'Israel','iso_3166_a2':'IL','iso_3166_a3':'ISR'},
        {'id':'108','name':'Italy','iso_3166_a2':'IT','iso_3166_a3':'ITA'},
        {'id':'109','name':'Jamaica','iso_3166_a2':'JM','iso_3166_a3':'JAM'},
        {'id':'110','name':'Japan','iso_3166_a2':'JP','iso_3166_a3':'JPN'},
        {'id':'111','name':'Jersey','iso_3166_a2':'JE','iso_3166_a3':'JEY'},
        {'id':'112','name':'Jordan','iso_3166_a2':'JO','iso_3166_a3':'JOR'},
        {'id':'113','name':'Kazakhstan','iso_3166_a2':'KZ','iso_3166_a3':'KAZ'},
        {'id':'114','name':'Kenya','iso_3166_a2':'KE','iso_3166_a3':'KEN'},
        {'id':'115','name':'Kiribati','iso_3166_a2':'KI','iso_3166_a3':'KIR'},
        {'id':'116','name':'Democratic People\'s Republic of Korea','iso_3166_a2':'KP','iso_3166_a3':'PRK'},
        {'id':'117','name':'Republic of Korea','iso_3166_a2':'KR','iso_3166_a3':'KOR'},
        {'id':'118','name':'Kuwait','iso_3166_a2':'KW','iso_3166_a3':'KWT'},
        {'id':'119','name':'Kyrgyzstan','iso_3166_a2':'KG','iso_3166_a3':'KGZ'},
        {'id':'120','name':'Lao People\'s Democratic Republic','iso_3166_a2':'LA','iso_3166_a3':'LAO'},
        {'id':'121','name':'Latvia','iso_3166_a2':'LV','iso_3166_a3':'LVA'},
        {'id':'122','name':'Lebanon','iso_3166_a2':'LB','iso_3166_a3':'LBN'},
        {'id':'123','name':'Lesotho','iso_3166_a2':'LS','iso_3166_a3':'LSO'},
        {'id':'124','name':'Liberia','iso_3166_a2':'LR','iso_3166_a3':'LBR'},
        {'id':'125','name':'Libyan Arab Jamahiriya','iso_3166_a2':'LY','iso_3166_a3':'LBY'},
        {'id':'126','name':'Liechtenstein','iso_3166_a2':'LI','iso_3166_a3':'LIE'},
        {'id':'127','name':'Lithuania','iso_3166_a2':'LT','iso_3166_a3':'LTU'},
        {'id':'128','name':'Luxembourg','iso_3166_a2':'LU','iso_3166_a3':'LUX'},
        {'id':'129','name':'Macao Special Administrative Region of China','iso_3166_a2':'MO','iso_3166_a3':'MAC'},
        {'id':'130','name':'The former Yugoslav Republic of Macedonia','iso_3166_a2':'MK','iso_3166_a3':'MKD'},
        {'id':'131','name':'Madagascar','iso_3166_a2':'MG','iso_3166_a3':'MDG'},
        {'id':'132','name':'Malawi','iso_3166_a2':'MW','iso_3166_a3':'MWI'},
        {'id':'133','name':'Malaysia','iso_3166_a2':'MY','iso_3166_a3':'MYS'},
        {'id':'134','name':'Maldives','iso_3166_a2':'MV','iso_3166_a3':'MDV'},
        {'id':'135','name':'Mali','iso_3166_a2':'ML','iso_3166_a3':'MLI'},
        {'id':'136','name':'Malta','iso_3166_a2':'MT','iso_3166_a3':'MLT'},
        {'id':'137','name':'Marshall Islands','iso_3166_a2':'MH','iso_3166_a3':'MHL'},
        {'id':'138','name':'Martinique','iso_3166_a2':'MQ','iso_3166_a3':'MTQ'},
        {'id':'139','name':'Mauritania','iso_3166_a2':'MR','iso_3166_a3':'MRT'},
        {'id':'140','name':'Mauritius','iso_3166_a2':'MU','iso_3166_a3':'MUS'},
        {'id':'141','name':'Mayotte','iso_3166_a2':'YT','iso_3166_a3':'MYT'},
        {'id':'142','name':'Mexico','iso_3166_a2':'MX','iso_3166_a3':'MEX'},
        {'id':'143','name':'Micronesia, Federated States of','iso_3166_a2':'FM','iso_3166_a3':'FSM'},
        {'id':'144','name':'Republic of Moldova','iso_3166_a2':'MD','iso_3166_a3':'MDA'},
        {'id':'145','name':'Monaco','iso_3166_a2':'MC','iso_3166_a3':'MCO'},
        {'id':'146','name':'Mongolia','iso_3166_a2':'MN','iso_3166_a3':'MNG'},
        {'id':'147','name':'Montenegro','iso_3166_a2':'ME','iso_3166_a3':'MNE'},
        {'id':'148','name':'Montserrat','iso_3166_a2':'MS','iso_3166_a3':'MSR'},
        {'id':'149','name':'Morocco','iso_3166_a2':'MA','iso_3166_a3':'MAR'},
        {'id':'150','name':'Mozambique','iso_3166_a2':'MZ','iso_3166_a3':'MOZ'},
        {'id':'151','name':'Myanmar','iso_3166_a2':'MM','iso_3166_a3':'MMR'},
        {'id':'152','name':'Namibia','iso_3166_a2':'NA','iso_3166_a3':'NAM'},
        {'id':'153','name':'Nauru','iso_3166_a2':'NR','iso_3166_a3':'NRU'},
        {'id':'154','name':'Nepal','iso_3166_a2':'NP','iso_3166_a3':'NPL'},
        {'id':'155','name':'Netherlands','iso_3166_a2':'NL','iso_3166_a3':'NLD'},
        {'id':'156','name':'Netherlands Antilles','iso_3166_a2':'AN','iso_3166_a3':'ANT'},
        {'id':'157','name':'New Caledonia','iso_3166_a2':'NC','iso_3166_a3':'NCL'},
        {'id':'158','name':'New Zealand','iso_3166_a2':'NZ','iso_3166_a3':'NZL'},
        {'id':'159','name':'Nicaragua','iso_3166_a2':'NI','iso_3166_a3':'NIC'},
        {'id':'160','name':'Niger','iso_3166_a2':'NE','iso_3166_a3':'NER'},
        {'id':'161','name':'Nigeria','iso_3166_a2':'NG','iso_3166_a3':'NGA'},
        {'id':'162','name':'Niue','iso_3166_a2':'NU','iso_3166_a3':'NIU'},
        {'id':'163','name':'Norfolk Island','iso_3166_a2':'NF','iso_3166_a3':'NFK'},
        {'id':'164','name':'Northern Mariana Islands','iso_3166_a2':'MP','iso_3166_a3':'MNP'},
        {'id':'165','name':'Norway','iso_3166_a2':'NO','iso_3166_a3':'NOR'},
        {'id':'166','name':'Oman','iso_3166_a2':'OM','iso_3166_a3':'OMN'},
        {'id':'167','name':'Pakistan','iso_3166_a2':'PK','iso_3166_a3':'PAK'},
        {'id':'168','name':'Palau','iso_3166_a2':'PW','iso_3166_a3':'PLW'},
        {'id':'169','name':'Occupied Palestinian Territory','iso_3166_a2':'PS','iso_3166_a3':'PSE'},
        {'id':'170','name':'Panama','iso_3166_a2':'PA','iso_3166_a3':'PAN'},
        {'id':'171','name':'Papua New Guinea','iso_3166_a2':'PG','iso_3166_a3':'PNG'},
        {'id':'172','name':'Paraguay','iso_3166_a2':'PY','iso_3166_a3':'PRY'},
        {'id':'173','name':'Peru','iso_3166_a2':'PE','iso_3166_a3':'PER'},
        {'id':'174','name':'Philippines','iso_3166_a2':'PH','iso_3166_a3':'PHL'},
        {'id':'175','name':'Pitcairn','iso_3166_a2':'PN','iso_3166_a3':'PCN'},
        {'id':'176','name':'Poland','iso_3166_a2':'PL','iso_3166_a3':'POL'},
        {'id':'177','name':'Portugal','iso_3166_a2':'PT','iso_3166_a3':'PRT'},
        {'id':'178','name':'Puerto Rico','iso_3166_a2':'PR','iso_3166_a3':'PRI'},
        {'id':'179','name':'Qatar','iso_3166_a2':'QA','iso_3166_a3':'QAT'},
        {'id':'180','name':'Réunion','iso_3166_a2':'RE','iso_3166_a3':'REU'},
        {'id':'181','name':'Romania','iso_3166_a2':'RO','iso_3166_a3':'ROU'},
        {'id':'182','name':'Russian Federation','iso_3166_a2':'RU','iso_3166_a3':'RUS'},
        {'id':'183','name':'Rwanda','iso_3166_a2':'RW','iso_3166_a3':'RWA'},
        {'id':'184','name':'Saint Helena','iso_3166_a2':'SH','iso_3166_a3':'SHN'},
        {'id':'185','name':'Saint Kitts and Nevis','iso_3166_a2':'KN','iso_3166_a3':'KNA'},
        {'id':'186','name':'Saint Lucia','iso_3166_a2':'LC','iso_3166_a3':'LCA'},
        {'id':'187','name':'Saint Pierre and Miquelon','iso_3166_a2':'PM','iso_3166_a3':'SPM'},
        {'id':'188','name':'Saint Vincent and the Grenadines','iso_3166_a2':'VC','iso_3166_a3':'VCT'},
        {'id':'189','name':'Samoa','iso_3166_a2':'WS','iso_3166_a3':'WSM'},
        {'id':'190','name':'San Marino','iso_3166_a2':'SM','iso_3166_a3':'SMR'},
        {'id':'191','name':'Sao Tome and Principe','iso_3166_a2':'ST','iso_3166_a3':'STP'},
        {'id':'192','name':'Saudi Arabia','iso_3166_a2':'SA','iso_3166_a3':'SAU'},
        {'id':'193','name':'Senegal','iso_3166_a2':'SN','iso_3166_a3':'SEN'},
        {'id':'194','name':'Serbia','iso_3166_a2':'RS','iso_3166_a3':'SRB'},
        {'id':'195','name':'Seychelles','iso_3166_a2':'SC','iso_3166_a3':'SYC'},
        {'id':'196','name':'Sierra Leone','iso_3166_a2':'SL','iso_3166_a3':'SLE'},
        {'id':'197','name':'Singapore','iso_3166_a2':'SG','iso_3166_a3':'SGP'},
        {'id':'198','name':'Slovakia','iso_3166_a2':'SK','iso_3166_a3':'SVK'},
        {'id':'199','name':'Slovenia','iso_3166_a2':'SI','iso_3166_a3':'SVN'},
        {'id':'200','name':'Solomon Islands','iso_3166_a2':'SB','iso_3166_a3':'SLB'},
        {'id':'201','name':'Somalia','iso_3166_a2':'SO','iso_3166_a3':'SOM'},
        {'id':'202','name':'South Africa','iso_3166_a2':'ZA','iso_3166_a3':'ZAF'},
        {'id':'203','name':'South Georgia and the South Sandwich Islands','iso_3166_a2':'GS','iso_3166_a3':'SGS'},
        {'id':'204','name':'Spain','iso_3166_a2':'ES','iso_3166_a3':'ESP'},
        {'id':'205','name':'Sri Lanka','iso_3166_a2':'LK','iso_3166_a3':'LKA'},
        {'id':'206','name':'Sudan','iso_3166_a2':'SD','iso_3166_a3':'SDN'},
        {'id':'207','name':'Suriname','iso_3166_a2':'SR','iso_3166_a3':'SUR'},
        {'id':'208','name':'Svalbard and Jan Mayen Islands','iso_3166_a2':'SJ','iso_3166_a3':'SJM'},
        {'id':'209','name':'Swaziland','iso_3166_a2':'SZ','iso_3166_a3':'SWZ'},
        {'id':'210','name':'Sweden','iso_3166_a2':'SE','iso_3166_a3':'SWE'},
        {'id':'211','name':'Switzerland','iso_3166_a2':'CH','iso_3166_a3':'CHE'},
        {'id':'212','name':'Syrian Arab Republic','iso_3166_a2':'SY','iso_3166_a3':'SYR'},
        {'id':'213','name':'Taiwan, Province of China','iso_3166_a2':'TW','iso_3166_a3':'TWN'},
        {'id':'214','name':'Tajikistan','iso_3166_a2':'TJ','iso_3166_a3':'TJK'},
        {'id':'215','name':'United Republic of Tanzania','iso_3166_a2':'TZ','iso_3166_a3':'TZA'},
        {'id':'216','name':'Thailand','iso_3166_a2':'TH','iso_3166_a3':'THA'},
        {'id':'217','name':'Timor-Leste','iso_3166_a2':'TL','iso_3166_a3':'TLS'},
        {'id':'218','name':'Togo','iso_3166_a2':'TG','iso_3166_a3':'TGO'},
        {'id':'219','name':'Tokelau','iso_3166_a2':'TK','iso_3166_a3':'TKL'},
        {'id':'220','name':'Tonga','iso_3166_a2':'TO','iso_3166_a3':'TON'},
        {'id':'221','name':'Trinidad and Tobago','iso_3166_a2':'TT','iso_3166_a3':'TTO'},
        {'id':'222','name':'Tunisia','iso_3166_a2':'TN','iso_3166_a3':'TUN'},
        {'id':'223','name':'Turkey','iso_3166_a2':'TR','iso_3166_a3':'TUR'},
        {'id':'224','name':'Turkmenistan','iso_3166_a2':'TM','iso_3166_a3':'TKM'},
        {'id':'225','name':'Turks and Caicos Islands','iso_3166_a2':'TC','iso_3166_a3':'TCA'},
        {'id':'226','name':'Tuvalu','iso_3166_a2':'TV','iso_3166_a3':'TUV'},
        {'id':'227','name':'Uganda','iso_3166_a2':'UG','iso_3166_a3':'UGA'},
        {'id':'228','name':'Ukraine','iso_3166_a2':'UA','iso_3166_a3':'UKR'},
        {'id':'229','name':'United Arab Emirates','iso_3166_a2':'AE','iso_3166_a3':'ARE'},
        {'id':'230','name':'United Kingdom','iso_3166_a2':'GB','iso_3166_a3':'GBR'},
        {'id':'231','name':'United States of America','iso_3166_a2':'US','iso_3166_a3':'USA'},
        {'id':'232','name':'United States Minor Outlying Islands','iso_3166_a2':'UM','iso_3166_a3':'UMI'},
        {'id':'233','name':'Uruguay','iso_3166_a2':'UY','iso_3166_a3':'URY'},
        {'id':'234','name':'Uzbekistan','iso_3166_a2':'UZ','iso_3166_a3':'UZB'},
        {'id':'235','name':'Vanuatu','iso_3166_a2':'VU','iso_3166_a3':'VUT'},
        {'id':'236','name':'Venezuela (Bolivarian Republic of)','iso_3166_a2':'VE','iso_3166_a3':'VEN'},
        {'id':'237','name':'Viet Nam','iso_3166_a2':'VN','iso_3166_a3':'VNM'},
        {'id':'238','name':'British Virgin Islands','iso_3166_a2':'VG','iso_3166_a3':'VGB'},
        {'id':'239','name':'United States Virgin Islands','iso_3166_a2':'VI','iso_3166_a3':'VIR'},
        {'id':'240','name':'Wallis and Futuna Islands','iso_3166_a2':'WF','iso_3166_a3':'WLF'},
        {'id':'241','name':'Western Sahara','iso_3166_a2':'EH','iso_3166_a3':'ESH'},
        {'id':'242','name':'Yemen','iso_3166_a2':'YE','iso_3166_a3':'YEM'},
        {'id':'243','name':'Zambia','iso_3166_a2':'ZM','iso_3166_a3':'ZMB'},
        {'id':'244','name':'Zimbabwe','iso_3166_a2':'ZW','iso_3166_a3':'ZWE'},

        {'id':'245','name':'Bonaire, Sint Eustatius and Saba','iso_3166_a2':'BQ','iso_3166_a3':'BES'},
        {'id':'246','name':'Curaçao','iso_3166_a2':'CW','iso_3166_a3':'CUW'},
        {'id':'247','name':'Saint Barthélemy','iso_3166_a2':'BL','iso_3166_a3':'BLM'},
        {'id':'248','name':'Saint Martin (French part)','iso_3166_a2':'MF','iso_3166_a3':'MAF'},
        {'id':'249','name':'Sint Maarten (Dutch part)','iso_3166_a2':'SX','iso_3166_a3':'SXM'},
        {'id':'250','name':'South Sudan','iso_3166_a2':'SS','iso_3166_a3':'SSD'},
    ]

    for c_obj in countries:

        try:
            # id is leading
            c = sa_session.query(Country).filter(Country.id == c_obj['id']).one()

            # error if the iso code mismatches. manual db adjustments should be made
            if c.iso_3166_a2 != c_obj['iso_3166_a2'] or c.iso_3166_a3 != c_obj['iso_3166_a3']:
                raise Exception('iso code mismatch: %s vs %s' % (c.id, c_obj) )

        except NoResultFound:

            print 'Creating new country: %s' % c_obj['name']

            # make a new country
            c = Country(c_obj['name'], c_obj['iso_3166_a2'], c_obj['iso_3166_a3'])
            c.id = c_obj['id']

        c.name = c_obj['name']

        sa_session.add(c)
    sa_session.flush()

def sync_languages(sa_session):
    languages = [
        {"id": 1, "name": "Abkhazian","iso_639_1": "ab"},
        {"id": 2, "name": "Afar","iso_639_1": "aa"},
        {"id": 3, "name": "Afrikaans","iso_639_1": "af"},
        {"id": 4, "name": "Akan","iso_639_1": "ak"},
        {"id": 5, "name": "Albanian","iso_639_1": "sq"},
        {"id": 6, "name": "Amharic","iso_639_1": "am"},
        {"id": 7, "name": "Arabic","iso_639_1": "ar"},
        {"id": 8, "name": "Aragonese","iso_639_1": "an"},
        {"id": 9, "name": "Armenian","iso_639_1": "hy"},
        {"id": 10, "name": "Assamese","iso_639_1": "as"},
        {"id": 11, "name": "Avaric","iso_639_1": "av"},
        {"id": 12, "name": "Avestan","iso_639_1": "ae"},
        {"id": 13, "name": "Aymara","iso_639_1": "ay"},
        {"id": 14, "name": "Azerbaijani","iso_639_1": "az"},
        {"id": 15, "name": "Bambara","iso_639_1": "bm"},
        {"id": 16, "name": "Bashkir","iso_639_1": "ba"},
        {"id": 17, "name": "Basque","iso_639_1": "eu"},
        {"id": 18, "name": "Belarusian","iso_639_1": "be"},
        {"id": 19, "name": "Bengali","iso_639_1": "bn"},
        {"id": 20, "name": "Bihari languages","iso_639_1": "bh"},
        {"id": 21, "name": "Bislama","iso_639_1": "bi"},
        {"id": 22, "name": "Bokmål, Norwegian","iso_639_1": "nb"},
        {"id": 23, "name": "Bosnian","iso_639_1": "bs"},
        {"id": 24, "name": "Breton","iso_639_1": "br"},
        {"id": 25, "name": "Bulgarian","iso_639_1": "bg"},
        {"id": 26, "name": "Burmese","iso_639_1": "my"},
        {"id": 27, "name": "Catalan","iso_639_1": "ca"},
        {"id": 28, "name": "Central Khmer","iso_639_1": "km"},
        {"id": 29, "name": "Chamorro","iso_639_1": "ch"},
        {"id": 30, "name": "Chechen","iso_639_1": "ce"},
        {"id": 31, "name": "Chewa","iso_639_1": "ny"},
        {"id": 32, "name": "Chinese","iso_639_1": "zh"},
        {"id": 33, "name": "Chuang","iso_639_1": "za"},
        {"id": 34, "name": "Church Slavic","iso_639_1": "cu"},
        {"id": 35, "name": "Chuvash","iso_639_1": "cv"},
        {"id": 36, "name": "Cornish","iso_639_1": "kw"},
        {"id": 37, "name": "Corsican","iso_639_1": "co"},
        {"id": 38, "name": "Cree","iso_639_1": "cr"},
        {"id": 39, "name": "Croatian","iso_639_1": "hr"},
        {"id": 40, "name": "Czech","iso_639_1": "cs"},
        {"id": 41, "name": "Danish","iso_639_1": "da"},
        {"id": 42, "name": "Dhivehi","iso_639_1": "dv"},
        {"id": 43, "name": "Dutch","iso_639_1": "nl"},
        {"id": 44, "name": "Dzongkha","iso_639_1": "dz"},
        {"id": 45, "name": "English","iso_639_1": "en"},
        {"id": 46, "name": "Esperanto","iso_639_1": "eo"},
        {"id": 47, "name": "Estonian","iso_639_1": "et"},
        {"id": 48, "name": "Ewe","iso_639_1": "ee"},
        {"id": 49, "name": "Faroese","iso_639_1": "fo"},
        {"id": 50, "name": "Fijian","iso_639_1": "fj"},
        {"id": 51, "name": "Finnish","iso_639_1": "fi"},
        {"id": 52, "name": "French","iso_639_1": "fr"},
        {"id": 53, "name": "Fulah","iso_639_1": "ff"},
        {"id": 54, "name": "Gaelic","iso_639_1": "gd"},
        {"id": 55, "name": "Galician","iso_639_1": "gl"},
        {"id": 56, "name": "Ganda","iso_639_1": "lg"},
        {"id": 57, "name": "Georgian","iso_639_1": "ka"},
        {"id": 58, "name": "German","iso_639_1": "de"},
        {"id": 59, "name": "Gikuyu","iso_639_1": "ki"},
        {"id": 60, "name": "Greek, Modern (1453-)","iso_639_1": "el"},
        {"id": 61, "name": "Greenlandic","iso_639_1": "kl"},
        {"id": 62, "name": "Guarani","iso_639_1": "gn"},
        {"id": 63, "name": "Gujarati","iso_639_1": "gu"},
        {"id": 64, "name": "Haitian","iso_639_1": "ht"},
        {"id": 65, "name": "Hausa","iso_639_1": "ha"},
        {"id": 66, "name": "Hebrew","iso_639_1": "he"},
        {"id": 67, "name": "Herero","iso_639_1": "hz"},
        {"id": 68, "name": "Hindi","iso_639_1": "hi"},
        {"id": 69, "name": "Hiri Motu","iso_639_1": "ho"},
        {"id": 70, "name": "Hungarian","iso_639_1": "hu"},
        {"id": 71, "name": "Icelandic","iso_639_1": "is"},
        {"id": 72, "name": "Ido","iso_639_1": "io"},
        {"id": 73, "name": "Igbo","iso_639_1": "ig"},
        {"id": 74, "name": "Indonesian","iso_639_1": "id"},
        {"id": 75, "name": "Interlingua (International Auxiliary Language Association)","iso_639_1": "ia"},
        {"id": 76, "name": "Interlingue","iso_639_1": "ie"},
        {"id": 77, "name": "Inuktitut","iso_639_1": "iu"},
        {"id": 78, "name": "Inupiaq","iso_639_1": "ik"},
        {"id": 79, "name": "Irish","iso_639_1": "ga"},
        {"id": 80, "name": "Italian","iso_639_1": "it"},
        {"id": 81, "name": "Japanese","iso_639_1": "ja"},
        {"id": 82, "name": "Javanese","iso_639_1": "jv"},
        {"id": 83, "name": "Kannada","iso_639_1": "kn"},
        {"id": 84, "name": "Kanuri","iso_639_1": "kr"},
        {"id": 85, "name": "Kashmiri","iso_639_1": "ks"},
        {"id": 86, "name": "Kazakh","iso_639_1": "kk"},
        {"id": 87, "name": "Kinyarwanda","iso_639_1": "rw"},
        {"id": 88, "name": "Kirghiz","iso_639_1": "ky"},
        {"id": 89, "name": "Komi","iso_639_1": "kv"},
        {"id": 90, "name": "Kongo","iso_639_1": "kg"},
        {"id": 91, "name": "Korean","iso_639_1": "ko"},
        {"id": 92, "name": "Kuanyama","iso_639_1": "kj"},
        {"id": 93, "name": "Kurdish","iso_639_1": "ku"},
        {"id": 94, "name": "Lao","iso_639_1": "lo"},
        {"id": 95, "name": "Latin","iso_639_1": "la"},
        {"id": 96, "name": "Latvian","iso_639_1": "lv"},
        {"id": 97, "name": "Limburgan","iso_639_1": "li"},
        {"id": 98, "name": "Lingala","iso_639_1": "ln"},
        {"id": 99, "name": "Lithuanian","iso_639_1": "lt"},
        {"id": 100, "name": "Luba-Katanga","iso_639_1": "lu"},
        {"id": 101, "name": "Luxembourgish","iso_639_1": "lb"},
        {"id": 102, "name": "Macedonian","iso_639_1": "mk"},
        {"id": 103, "name": "Malagasy","iso_639_1": "mg"},
        {"id": 104, "name": "Malay","iso_639_1": "ms"},
        {"id": 105, "name": "Malayalam","iso_639_1": "ml"},
        {"id": 106, "name": "Maltese","iso_639_1": "mt"},
        {"id": 107, "name": "Manx","iso_639_1": "gv"},
        {"id": 108, "name": "Maori","iso_639_1": "mi"},
        {"id": 109, "name": "Marathi","iso_639_1": "mr"},
        {"id": 110, "name": "Marshallese","iso_639_1": "mh"},
        {"id": 111, "name": "Mongolian","iso_639_1": "mn"},
        {"id": 112, "name": "Nauru","iso_639_1": "na"},
        {"id": 113, "name": "Navaho","iso_639_1": "nv"},
        {"id": 114, "name": "Ndebele, North","iso_639_1": "nd"},
        {"id": 115, "name": "Ndonga","iso_639_1": "ng"},
        {"id": 116, "name": "Nepali","iso_639_1": "ne"},
        {"id": 117, "name": "Northern Sami","iso_639_1": "se"},
        {"id": 118, "name": "Norwegian","iso_639_1": "no"},
        {"id": 119, "name": "Norwegian Nynorsk","iso_639_1": "nn"},
        {"id": 120, "name": "Nuosu","iso_639_1": "ii"},
        {"id": 121, "name": "Occitan (post 1500)","iso_639_1": "oc"},
        {"id": 122, "name": "Ojibwa","iso_639_1": "oj"},
        {"id": 123, "name": "Oriya","iso_639_1": "or"},
        {"id": 124, "name": "Oromo","iso_639_1": "om"},
        {"id": 125, "name": "Ossetian","iso_639_1": "os"},
        {"id": 126, "name": "Pali","iso_639_1": "pi"},
        {"id": 127, "name": "Panjabi","iso_639_1": "pa"},
        {"id": 128, "name": "Pashto","iso_639_1": "ps"},
        {"id": 129, "name": "Persian","iso_639_1": "fa"},
        {"id": 130, "name": "Polish","iso_639_1": "pl"},
        {"id": 131, "name": "Portuguese","iso_639_1": "pt"},
        {"id": 132, "name": "Quechua","iso_639_1": "qu"},
        {"id": 133, "name": "Romanian","iso_639_1": "ro"},
        {"id": 134, "name": "Romansh","iso_639_1": "rm"},
        {"id": 135, "name": "Rundi","iso_639_1": "rn"},
        {"id": 136, "name": "Russian","iso_639_1": "ru"},
        {"id": 137, "name": "Samoan","iso_639_1": "sm"},
        {"id": 138, "name": "Sango","iso_639_1": "sg"},
        {"id": 139, "name": "Sanskrit","iso_639_1": "sa"},
        {"id": 140, "name": "Sardinian","iso_639_1": "sc"},
        {"id": 141, "name": "Serbian","iso_639_1": "sr"},
        {"id": 142, "name": "Shona","iso_639_1": "sn"},
        {"id": 143, "name": "Sindhi","iso_639_1": "sd"},
        {"id": 144, "name": "Sinhala","iso_639_1": "si"},
        {"id": 145, "name": "Slovak","iso_639_1": "sk"},
        {"id": 146, "name": "Slovenian","iso_639_1": "sl"},
        {"id": 147, "name": "Somali","iso_639_1": "so"},
        {"id": 148, "name": "Sotho, Southern","iso_639_1": "st"},
        {"id": 149, "name": "South Ndebele","iso_639_1": "nr"},
        {"id": 150, "name": "Spanish","iso_639_1": "es"},
        {"id": 151, "name": "Sundanese","iso_639_1": "su"},
        {"id": 152, "name": "Swahili","iso_639_1": "sw"},
        {"id": 153, "name": "Swati","iso_639_1": "ss"},
        {"id": 154, "name": "Swedish","iso_639_1": "sv"},
        {"id": 155 , "name": "Tagalog","iso_639_1": "tl"},
        {"id": 156, "name": "Tahitian","iso_639_1": "ty"},
        {"id": 157, "name": "Tajik","iso_639_1": "tg"},
        {"id": 158, "name": "Tamil","iso_639_1": "ta"},
        {"id": 159, "name": "Tatar","iso_639_1": "tt"},
        {"id": 160, "name": "Telugu","iso_639_1": "te"},
        {"id": 161, "name": "Thai","iso_639_1": "th"},
        {"id": 162, "name": "Tibetan","iso_639_1": "bo"},
        {"id": 163, "name": "Tigrinya","iso_639_1": "ti"},
        {"id": 164, "name": "Tonga (Tonga Islands)","iso_639_1": "to"},
        {"id": 165, "name": "Tsonga","iso_639_1": "ts"},
        {"id": 166, "name": "Tswana","iso_639_1": "tn"},
        {"id": 167, "name": "Turkish","iso_639_1": "tr"},
        {"id": 168, "name": "Turkmen","iso_639_1": "tk"},
        {"id": 169, "name": "Twi","iso_639_1": "tw"},
        {"id": 170, "name": "Uighur","iso_639_1": "ug"},
        {"id": 171, "name": "Ukrainian","iso_639_1": "uk"},
        {"id": 172, "name": "Urdu","iso_639_1": "ur"},
        {"id": 173, "name": "Uzbek","iso_639_1": "uz"},
        {"id": 174, "name": "Venda","iso_639_1": "ve"},
        {"id": 175, "name": "Vietnamese","iso_639_1": "vi"},
        {"id": 176, "name": "Volapük","iso_639_1": "vo"},
        {"id": 177, "name": "Walloon","iso_639_1": "wa"},
        {"id": 178, "name": "Welsh","iso_639_1": "cy"},
        {"id": 179, "name": "Western Frisian","iso_639_1": "fy"},
        {"id": 180, "name": "Wolof","iso_639_1": "wo"},
        {"id": 181, "name": "Xhosa","iso_639_1": "xh"},
        {"id": 182, "name": "Yiddish","iso_639_1": "yi"},
        {"id": 183, "name": "Yoruba","iso_639_1": "yo"},
        {"id": 184, "name": "Zulu","iso_639_1": "zu"},
    ]

    for c_obj in languages:

        try:
            # id is leading
            l = sa_session.query(Language).filter(Language.id == c_obj['id']).one()

            # error if the iso code mismatches. manual db adjustments should be made
            if l.iso_639_1 != c_obj['iso_639_1']:
                raise Exception('iso code mismatch: %s vs %s' % (l.id, c_obj) )

        except NoResultFound:

            print 'Creating new language: %s' % c_obj['name']

            # make a new language
            l = Language()
            l.iso_639_1 = c_obj['iso_639_1']
            l.id = c_obj['id']

        l.name = c_obj['name']

        sa_session.add(l)
    sa_session.flush()




