# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class DomainCountryManagerIcon(IconBarIcon):
    id = 'icon-domain-country-manager'
    title = _('Domain country manager')
    href = '#'
    icon_index = 60
    component = 'iq_domain_country_manager'
    group = 'internationalization'
    colorname = 'iconbar-rust'

    char_main = u'\uE800' # World icon
    char_sub = u'\uE86B' # Compass icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('manage_domain_country_mapping')

class DomainLanguageManagerIcon(IconBarIcon):
    id = 'icon-domain-language-manager'
    title = _('Domain language manager')
    href = '#'
    icon_index = 61
    component = 'iq_domain_language_manager'
    group = 'internationalization'
    colorname = 'iconbar-rust'

    char_main = u'\uE838' # Chat icon
    char_sub = u'\uE86B' # Compass icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('manage_domain_language_aliases')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """
    return [DomainCountryManagerIcon(), DomainLanguageManagerIcon()]
