# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class LicenseIcon(IconBarIcon):
    """
    Class responsible for handling the license manager icon
    """

    id = 'icon-license-manager'
    title = _('License')
    href = '#'
    icon_index = 90
    component = 'iq_license_manager'
    group = 'administration'
    group_title = _('Admin')
    colorname = 'iconbar-tapestry'

    char_main = u'\uE81e' # lock icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_any_permissions('view_all_domains', 'view_domain', 'view_license')

class MasterDataIcon(IconBarIcon):
    """
    Icon used for master data changes.
    """
    id = 'icon-master-data'
    title = _('Master data')
    href = '#'
    icon_index = 93
    component = 'iq_master_data'
    group = 'administration'
    group_title = _('Admin')
    colorname = 'iconbar-tapestry'

    char_main = u'\uE81e' # Lock icon
    char_sub = u'\uE822' # Crown icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        Only user who have master data change permissions
        """
        controller.assert_permissions('update_license_masterdata')

class PermissionIcon(IconBarIcon):
    """
    Class responsible for handling the license manager icon
    """

    id = 'icon-permission-overview'
    title = _('Permission overview')
    href = '#'
    icon_index = 92
    component = 'iq_permission_overview'
    group = 'administration'
    group_title = _('Admin')
    colorname = 'iconbar-tapestry'

    char_main = u'\uE81e' # Lock icon
    char_sub = u'\uE81f' # Unlock icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_any_permissions('view_all_permissions')

class RoleManagerIcon(IconBarIcon):
    id = 'icon-role-manager'
    title = _('Role manager')
    href = '#'
    icon_index = 93
    component = 'iq_role_manager'
    group = 'administration'
    group_title = _('Admin')
    colorname = 'iconbar-tapestry'

    char_main = u'\uE806' # Man icon
    char_sub = None # No icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_any_permissions('manage_roles')

class HookIcon(IconBarIcon):
    """
    Class responsible for handling the hook icon.
    """
    id = 'icon-hook-manager'
    title = _('Privilege hooks')
    href = "#"
    icon_index = 91
    component = 'iq_hook_manager'
    group = 'administration'
    group_title = _('Admin')
    colorname = 'iconbar-tapestry'

    char_main = u'\uE80e' # Wheelchair icon
    char_sub = u'\uE80d' # Gear icon


    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('hook_manager')

class ObjectBrowserIcon(IconBarIcon):
    """
    Class responsible for handling the hook icon.
    """
    id = 'icon-object-browser'
    title = _('Object browser')
    href = "#"
    icon_index = 95
    component = 'iq_object_browser'
    group = 'administration'
    group_title = _('Admin')
    colorname = 'iconbar-tapestry'

    char_main = u'\uE86F' # Asterix icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        # for now just sysadmin
        controller.assert_permissions('privilege_object_administrator')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [LicenseIcon(), HookIcon(), PermissionIcon(), MasterDataIcon(), RoleManagerIcon(), ObjectBrowserIcon()]
