'''
Created on Sep 25, 2013

@author: marten
'''
from django.utils.translation import ugettext, ugettext_lazy as _
from django.conf import settings

from basesoft.web.core.input.validation import plugins
from basesoft.web.core.config import DefaultConfig

class MailingConfig(DefaultConfig):
    permissions = ('create_mail', )
    label = _('Mailing settings')

    def configs(self):
        rv = [{
            'name': 'basesoft.web.contrib.mailing.default_mail_from_text',
            'label': ugettext('Default "from" text for mailings'),
            'help': ugettext('This is the text that will be visible to the recipients of your mailings.')
        }, {
            'name': 'basesoft.web.contrib.mailing.default_mail_from_email',
            'label': ugettext('Default "from" email for mailings'),
            'help': ugettext('This is the email address that will be used when the recipient replies to your mailings.')
        }]

        # we do not have a VMAIL connection
        if settings.VMAIL_DB_CONNSTR is None:
            return rv

        rv.append({
            'name': 'basesoft.web.contrib.mailing.mail_return_path',
            'label': ugettext('Bounce and rejections email address'),
            'help': ugettext('This is the email address that will be used to forward bounces and rejections to your mailings; a lead time is required before this setting becomes active.')
        })

        return rv

    def get_form_behaviour(self, validators, values):
        validators['basesoft.web.contrib.mailing.default_mail_from_email'] = [(plugins.Email, {})]

        if settings.VMAIL_DB_CONNSTR is None:
            return validators

        validators['basesoft.web.contrib.mailing.mail_return_path'] = [(plugins.Email, {})]

        return validators

    def get_save_callbacks(self, callbacks):
        callbacks['basesoft.web.contrib.mailing.mail_return_path'] = (self, 'save_forwarding_for_return_path')
        return callbacks

    def save_forwarding_for_return_path(self, original_value, value, scope):
        """
        Setting this license config option will add an entry to the set VMAIL database forwardings table.
        """
        if settings.VMAIL_DB_CONNSTR is None:
            return

        bouncer = 'bounces-license-%d@iqportalmail.com' % scope.id
        self._controller.logger.info('Setting return path forwarding for license %(license)s (%(bouncer)s) to `%(value)s`' % {
            'license': scope.name,
            'value': value,
            'bouncer': bouncer
        })

        vmaildb = self._connect_to_vmail()
        vmaildb.execute('REPLACE INTO `forwardings` (`source`, `destination`) VALUES (:bouncer, :forward)', {'bouncer': bouncer, 'forward': value})

    def _connect_to_vmail(self):

        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker

        db_engine = create_engine(settings.VMAIL_DB_CONNSTR, echo=False, pool_recycle=2)
        return scoped_session(sessionmaker(bind=db_engine, autoflush=True, autocommit=True))

license_configs = {
    'License' : (MailingConfig(), )
}