# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
import os
from basesoft.web.contrib.wiki.formatter import HTMLFormatter
from basesoft.web.contrib.wiki.models import WikiContent, Wiki
from django.conf import settings

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.license.models import Permission
from basesoft.web.contrib.manual.models import ManualChapter, ManualChapterLink
from basesoft.web.contrib.wiki.controllers import WikiController


LOCALES = []  # FIXME


class SyncManualController(AbstractController, WikiController):

    @transaction_decorator
    def save_manual_chapter(self, handle, category, permissions, wiki_state):
        chapter = ManualChapter()
        chapter.wiki = None
        chapter.handle = handle
        if permissions is not None and len(permissions) > 0:
            chapter.permissions = [p for p in self.sa_session.query(Permission).filter(Permission.name.in_(permissions)).all()]
        chapter.category = category

        chapter.wiki = self._save_wiki(chapter.wiki, wiki_state)

        self.sa_session.add(chapter)
        return chapter

    @transaction_decorator
    def update_links(self):
        self.sa_session.query(ManualChapterLink).delete()
        locale = self.sa_session.query(Locale).filter(Locale.locale == 'en_GB').one()
        for chapter in self.sa_session.query(ManualChapter):
            content = self._get_current_wiki_content(chapter.wiki, locale)
            if content is None:
                continue
            self._update_chapter_links(chapter, content.content)

    def _update_chapter_links(self, chapter, content):
        f = HTMLFormatter({}, context=self)
        f.format(content)

        wikis = f.get_linked_wikis()
        if len(wikis) == 0:
            return

        for c in self.sa_session.query(ManualChapter).filter(ManualChapter.handle.in_(set(wikis))):
            mcl = ManualChapterLink()
            mcl.chapter = chapter
            mcl.target_chapter = c
            self.sa_session.add(mcl)

def sync(session):
    """
    Loops all installed apps and looks for the wiki package. This will only insert wiki pages that do not have an entry.
    """

    smc = SyncManualController(session)
    fallback_locale = session.query(Locale).filter(Locale.locale == 'en_GB').one()

    # go through the apps and see if they have permissions to sync.
    for app in settings.INSTALLED_APPS:
        try:
            wiki_module = __import__(app + '.wiki', {}, {}, [''])
        except ImportError:
            # if we do not have a wiki package then just continue.
            continue

        if not hasattr(wiki_module, 'wikis'):
            # there are no actual wikis to sync.
            continue

        for wiki, values in wiki_module.wikis.iteritems():
            mc = session.query(ManualChapter).filter(ManualChapter.handle == wiki).first()
            if mc is not None:
                # we do not sync already existing chapters.
                print 'Not updating manual wiki: %s' % wiki
                continue

            path = os.path.dirname(wiki_module.__file__)
            wikipath = os.path.join(path, '%s-%s.txt' % (wiki, fallback_locale.locale))

            if not os.path.exists(wikipath):
                continue

            print 'Syncing manual chapter: %s' % wiki

            f = open(wikipath, 'r')
            content = f.read()
            f.close()

            wiki_state = {
                'fallback_locale_id': fallback_locale.id,
                'content': content,
                'title': values['title'],  # FIXME ???
                'locale_id': fallback_locale.id
            }

            smc.save_manual_chapter(wiki, values['category'], values['permissions'], wiki_state)

            #for l in LOCALES:
            #    locale = session.query(Locale).filter(Locale.locale == l).one()

    # update all the links
    smc.update_links()