# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon


class NotificationIcon(IconBarIcon):
    """
    """

    id = 'icon-notifications'
    title = _('Notifications')
    href = '#'
    icon_index = 60
    component = 'iq_notification_manager'
    group = 'identity'
    group_title = _('Identity')
    colorname = 'iconbar-golden-grass'

    char_main = u'\uE819' # Loadspeaker icon

    # number icon (unread notifications)
    number_type = 'important'
    number_rpc_url = 'basesoft.web.contrib.notification.NotificationRPCHandler'
    number_rpc_method = 'get_icon_count'
    number_refresh = 120 # refresh 2 minutes
    number_rpc_login_required = True # only refreshes if user is logged
    number_rpc_global_change_event = 'basesoft.web.contrib.notification.NotificationCustomQuery.newData'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        The only requirement here is that user is logged in
        """
        controller.assert_permissions('view_notification')


def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [NotificationIcon()]
