# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext, ugettext_lazy as _

from django.utils import simplejson

from basesoft.web.contrib.mailing.models import Mail
from basesoft.web.core.models import PrivilegeObject


class Notification(PrivilegeObject):
    """
    @DynamicAttrs
    """

    mark_as_deleted_permission = 'delete_notification'
    mark_as_archived_permission = 'archive_notification'

    _name_property = 'subject'

    name_i18n = _('Notification')

    def get_group_object(self):
        return (NotificationGroup, 'notification')

    def __repr__(self):
        return '<Notification id="%s">' % self.id

    def get_message(self):
        return self.text

    def get_subject(self):
        return self.subject

    def get_refresh_dataset_event_args(self):  # only used on the polling from the menu (iconbar, topmenu)
        if self.notify is None:
            return []

        from basesoft.web.core.jsonrpc.dispatcher import json_object_hook

        return simplejson.loads(self.notify, object_hook=json_object_hook)

class NotificationGroup(object):
    """
    @DynamicAttrs
    """
    pass


class NotificationReminderMail(Mail):
    """
    @DynamicAttrs
    """
    NO_REMINDER = 0
    SEND_WEEKLY = 1
    SEND_MONTHLY = 2
    SEND_DAILY = 3

    NAME_I18N = _("Notification reminder mail")

    JS_MODULE = 'iq_account_mail_plugin'

    PRIV_USE_FOR_SENDING = 64

    def get_privileges(self):
        return Mail.get_privileges(self) + [
            (self.PRIV_USE_FOR_SENDING, ugettext('Use for sending'))
        ]

    def __repr__(self):
        return '<NotificationReminderMail id="%s">' % self.id

    @staticmethod
    def replacement_fields():
        x = Mail.replacement_fields()
        x.update({
            ugettext('Notification'): {
                'notification.subjects': ugettext('List of subjects'),
                'notification.link': ugettext('Link to overview'),
            }
        })
        return x

    def _replace_field(self, person, field_name, context):

        if field_name == 'subjects':
            return '<ul>%s</ul>' % ''.join(['<li>%s</li>' % i for i in context['subjects']])
        if field_name == 'link':
            link = '%s#m=notifications' % self._base_url(context)
            return '<a href="%s">%s</a>' % (link, link)
        return Mail._replace_field(self, person, field_name, context)
