# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from iqportal.iqube.core.models import DataSource
from basesoft.web.core.management.commands.iqubesync import ProjectBase, CubeBase, RoleBase
from basesoft.web.contrib.order.models import Order, GLAccount
from iqportal.iqube.core.controllers import IQubeRoleController, IQubeRole
from basesoft.web.core.util.controllers import AbstractController,\
    get_models_of_type

class OrderProject(ProjectBase):
    project_name = 'Order'
    world_privileges = DataSource.PRIV_VIEW
    description = 'Order project'
    role_level = DataSource.ROLE_LEVEL_DB

    def __init__(self, sa_session, controller):
        ProjectBase.__init__(self, sa_session, controller)
        self._db_from_connstr(settings.DB_CONNSTR_READONLY)

    def get_cubes(self):
        return (OrderCube, )

    def get_roles(self):
        return (OrderCubeRole, )

class OrderCube(CubeBase):
    name = 'Order'

    MERCHANT_LABEL = 'Merchant'
    CLIENT_LABEL = 'Client'

    CLIENT_fancy_select_optionsRpcUrl = None
    CLIENT_fancy_select_RpcMethod = None


    def get_order_type_labels(self):
        rv = []
        models = get_models_of_type(Order, exclude_parent=True, include_models=True)
        for m in models:
            rv.append("WHEN '%s' THEN '%s'" % (m['value'], m['model']().get_name_i18n()))
        return rv

    def _xml(self):
        return """
    <Cube name="%(name)s" defaultMeasure="Amount">

        <Table name="aggOrder" />

        <!-- Polymorphic Order type -->
        <Dimension name="Order Type">
            <Hierarchy hasAll="true" allMemberName="All Order Type">
                <Level name="Order Type" column="order_type">
                    <CaptionExpression>
                        <SQL dialect="mysql">
                            CASE `aggOrder`.`order_type`
                                %(order_types)s
                                ELSE '-' END
                        </SQL>
                    </CaptionExpression>
                </Level>
            </Hierarchy>
        </Dimension>

        <!-- Credit vs. Debit -->
        <Dimension name="Invoice Type">
            <Hierarchy hasAll="true" allMemberName="All Invoice Types">
                <Level name="Invoice Type" column="invoice_type" />
            </Hierarchy>
        </Dimension>

        <Dimension name="%(merchant)s" foreignKey="merchant_id">
            <Hierarchy hasAll="true" allMemberName="All %(merchant)ss" primaryKey="id" primaryKeyTable="tblLicense">
                <Table name="tblLicense" alias="merchant" />
                <Level name="%(merchant)s" table="merchant" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />
            </Hierarchy>
        </Dimension>


        <Dimension name="%(client)s" foreignKey="client_id">
            <Hierarchy hasAll="true" allMemberName="All %(client)ss" primaryKey="id" primaryKeyTable="tblLicense">
                <Table name="tblLicense" alias="client" />
                <Level name="%(client)s" table="client" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />
            </Hierarchy>
        </Dimension>

        <Dimension name="GL Account">
            <Hierarchy hasAll="true" allMemberName="All GL Accounts">
                <Level name="GL Account" column="gl_account" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Order">
            <Hierarchy hasAll="true" allMemberName="All Orders">
                <Level name="Order" column="order_id" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Payment Method">
            <Hierarchy hasAll="true" allMemberName="All Payment Methods">
                <Level name="Payment Method" column="payment_method" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Submit Period" foreignKey="submitted_time_id" type="TimeDimension">
            <Hierarchy hasAll="true" primaryKey="id" name="Quarter">
                <Table name="aggTimeQuarter"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Quarter" column="quarter" uniqueMembers="false" levelType="TimeQuarters"  ordinalColumn="quarter" />
                <Level name="Month" column="month" type="Integer" uniqueMembers="false" levelType="TimeMonths" ordinalColumn="month" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekISO8601" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekISO8601"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekUS" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekUS"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Payment Period" foreignKey="payment_time_id" type="TimeDimension">
            <Hierarchy hasAll="true" primaryKey="id" name="Quarter">
                <Table name="aggTimeQuarter"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Quarter" column="quarter" uniqueMembers="false" levelType="TimeQuarters"  ordinalColumn="quarter" />
                <Level name="Month" column="month" type="Integer" uniqueMembers="false" levelType="TimeMonths" ordinalColumn="month" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekISO8601" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekISO8601"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekUS" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekUS"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
        </Dimension>

        <Measure name="Amount (excl. vat)" column="line_amount_excluding_vat" aggregator="sum" datatype="Numeric" />
        <Measure name="Amount (incl. vat)" column="line_amount_including_vat" aggregator="sum" datatype="Numeric" />
        <Measure name="Price per Unit" column="price_per_unit" aggregator="sum" datatype="Numeric" />
        <Measure name="Quantity" column="quantity" aggregator="sum" datatype="Integer" />
        <Measure name="Number of %(client)ss" column="client_id" aggregator="distinct-count" datatype="Integer" />

    </Cube>
        """ % {'name': self.name, 'order_types' : '\n'.join(self.get_order_type_labels()), 'merchant': self.MERCHANT_LABEL, 'client': self.CLIENT_LABEL}

    xml = property(_xml)

class OrderCubeRole(RoleBase):

    def get_role_list(self, account_id):
        """
        FIXME: haz to be optimized by splitting into cubes per domain
        """

        ac = AbstractController(self.sa_session, account_id=account_id, check_active=False)
        ac.assert_permissions('view_orders')

        has_order_access = False

        rc = IQubeRoleController(self.sa_session, account_id, self.ds, 'Order')
        rc.grant_cube(IQubeRole.ACCESS_ALL)

        clients = []
        merchants = []

        for order in ac.get_privileged_object(Order):
            if order.client.id not in clients:
                rc.grant_member('[Client].[%s]' % order.client.id, IQubeRole.ACCESS_ALL)
                clients.append(order.client.id)
            if order.merchant.id not in merchants:
                rc.grant_member('[Merchant].[%s]' % order.merchant.id, IQubeRole.ACCESS_ALL)
                merchants.append(order.merchant.id)
            has_order_access = True

        gl_accounts = self.sa_session.execute("""
            SELECT DISTINCT gl_account FROM aggOrder WHERE merchant_id IN (%s)
        """ % ",".join([str(m) for m in merchants])).fetchall()
        for gl_account in gl_accounts:
            rc.grant_member('[GL Account].[%s]' % gl_account.gl_account, IQubeRole.ACCESS_ALL)

        if has_order_access:
            rc.grant_hierarchy('[Client]', IQubeRole.ACCESS_CUSTOM, IQubeRole.ROLLUP_POLICY_PARTIAL)
            rc.grant_hierarchy('[Merchant]', IQubeRole.ACCESS_CUSTOM, IQubeRole.ROLLUP_POLICY_PARTIAL)
            rc.grant_hierarchy('[GL Account]', IQubeRole.ACCESS_CUSTOM, IQubeRole.ROLLUP_POLICY_PARTIAL)
        else:
            rc.grant_hierarchy('[Client]', IQubeRole.ACCESS_NONE, IQubeRole.ROLLUP_POLICY_HIDDEN)
            rc.grant_hierarchy('[Merchant]', IQubeRole.ACCESS_NONE, IQubeRole.ROLLUP_POLICY_HIDDEN)
            rc.grant_hierarchy('[GL Account]', IQubeRole.ACCESS_NONE, IQubeRole.ROLLUP_POLICY_HIDDEN)

        return rc.to_list()

projects = (OrderProject, )
