'''
Created on 6 Jun 2013

@author: martijn
'''
import datetime

from django.contrib.sessions.backends.base import SessionBase
from basesoft.web.core.util.db import Session

from django.conf import settings
from basesoft.web.core.exceptions import LoginRequiredException
from django.utils.translation import ugettext as _

class SessionStore(SessionBase):

    def __init__(self, session_key=None, ip=None):

        super(SessionStore, self).__init__(session_key)
        self._db_con = Session().bind
        self.ip = ip

    def load(self):
        sql = """
            SELECT session_data
            FROM tblSession
            WHERE session_key = %s AND expire_date > %s
        """
        result = self._db_con.execute(sql, self.session_key, datetime.datetime.now())

        for session_data, in result:
            return self.decode(session_data)

        # Create a new session_key for extra security.
        self._session_key = self._get_new_session_key()
        self._session_cache = {}

        # Save immediately to minimize collision
        self.save()

        # Ensure the user is notified via a new cookie.
        self.modified = True
        return {}

    def exists(self, session_key):

        sql = """
            SELECT session_data
            FROM tblSession
            WHERE session_key = %s
        """
        result = self._db_con.execute(sql, session_key)

        for session_data, in result:
            return True

        return False

    def save(self, must_create=False):

        self._get_or_create_session_key()

        session_data = self.encode(self._session)

        # change the expire date based on the fact that the user might want to stay logged in longer
        if self._session.has_key('keep_logged_in') and self._session['keep_logged_in']:
            max_age = settings.KEEP_SIGNED_COOKIE_AGE
        else:
            max_age = settings.SESSION_COOKIE_AGE

        expire_date = datetime.datetime.now() + datetime.timedelta(seconds=max_age)


        # used to determine the user to whom the session belongs
        if self._session.has_key('user_id'):
            account_id = self._session['user_id']
        else:
            account_id = None

        # check if we need to update the logged_out column
        logged_out = None
        if self._session.has_key('logged_out'):
            if self._session['logged_out'] is None:
                # freshly logged in, so the logged_out date should be empty
                logged_out = None
            else:
                # freshly logged out, so the logged_out date needs to be set and the account_id restored
                logged_out = datetime.datetime.now()
                account_id = self._session['logged_out']  # restore back the account_id for overview purposes
            del self._session['logged_out']  # this key is no longer needed in session so it's cleaned up

        # the last update time is used to determine if a session was updated by the keepalive ping or if the user
        # closed the window, no keep-alives are beeing sent, but the session might still be active if the session
        # timeout hasn't expired yet

        sql = """
            INSERT INTO tblSession (session_key, session_data, expire_date, account_id, ip, logged_in, logged_out, last_updated)
            VALUES (%s,%s,%s,%s,%s,NOW(),%s,NOW())
            ON DUPLICATE KEY UPDATE
                `session_data`=%s,
                `expire_date`=%s,
                `account_id`=%s,
                `ip`=%s,
                `logged_out`=%s,
                `last_updated`=NOW()
        """

        self._db_con.execute(sql, self.session_key, session_data, expire_date, account_id, self.ip, logged_out, session_data, expire_date, account_id, self.ip, logged_out)

    def delete(self, session_key):

        sql = """
            DELETE FROM tblSession WHERE session_key = ?
        """
        self._db_con.execute(sql, (self.session_key))

    def __getitem__(self, key):
        try:
            return self._session[key]
        except KeyError, e:
            if key == 'user_id':
                raise LoginRequiredException(_('Login required'))
            raise e
