# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy.orm import mapper, relation
import datetime
from sqlalchemy import Column, String, DateTime, Integer, ForeignKey, Text
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.session import models
from basesoft.web.contrib.account import models as account_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False


def get_tables():
    """
    return the list of all tables
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:
        tblSession = IQTable('tblSession', metadata,
            Column('session_key', String(40), primary_key=True),
            Column('session_data', Text),
            Column('expire_date', DateTime, nullable=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id'), nullable=True),
            Column('ip', String(23), nullable=True),
            Column('logged_in', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('logged_out', DateTime(), nullable=True),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )
        tables['tblSession'] = tblSession
        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:
        mapper(models.HTTPSession, tables['tblSession'],
            properties = {
                'account': relation(account_models.Account)
            }
        )
        __mapping_defined = True

