# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

import logging
import traceback

from pyws.server import SoapServer
from pyws.errors import Error, ProtocolError, ConfigurationError, ET_CLIENT, ET_SERVER
from pyws.response import Response
from pyws.utils import ENCODING

from basesoft.web.contrib.soap.protocols import IQPortalSoapProtocol

logger = logging.getLogger('basesoft.web.contrib.soap')

class IQPortalSoapServer(SoapServer):
    def __init__(self, settings=None, *args, **kwargs):
        self.soap_protocol = IQPortalSoapProtocol(*args, **kwargs)
        super(SoapServer, self).__init__(settings)

    def process_request(self, request):
        """
        This method is overridden in order to pass the original request object to the protocol get_error_response
        for Django email generation.
        """

        logger.debug(request)

        e_args = []

        try:
            protocol = self.get_protocol(request)
        except ProtocolError, e:
            return Response(unicode(e).encode(ENCODING))

        if isinstance(protocol, IQPortalSoapProtocol):
            e_args.append(request)

        try:
            # if both tail and text are empty strings, its an empty call at /api
            if request.tail.strip() == '' and request.text.strip() == '':
                raise Error('Empty request supplied')

            function = protocol.get_function(request)

            if not callable(function):
                context_data = protocol.get_context_data(request)
            else:
                context_data = protocol.get_common_context_data(request)

            with self.get_context_manager(context_data) as context:

                if callable(function):
                    return function(self, request, context)

                function = self.get_function(context, function)

                args = protocol.get_arguments(request, function.args)

                result = function(context, **args)

            response = protocol.get_response(
                result, function.name, function.return_type)

        except ConfigurationError:
            logger.error(traceback.format_exc())
            if self.settings.DEBUG:
                raise
            response = protocol.get_error_response(
                Error('Internal server error occured'), *e_args)
        except Error, e:
            logger.error(traceback.format_exc())
            response = protocol.get_error_response(e, *e_args)
        except Exception, e:
            logger.error(traceback.format_exc())
            client_error = hasattr(e, '__module__') or type(e) == Exception
            if not client_error and self.settings.DEBUG:
                raise
            e.error_type = client_error and ET_CLIENT or ET_SERVER
            response = protocol.get_error_response(e, *e_args)

        logger.debug(response)

        return response
