'''
Created on Oct 16, 2013

@author: marten
'''
from pyws.functions.register import register

from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.account.models import Account

from basesoft import logging
from basesoft.web.core.exceptions import PublicUserException

def set_session_data(request, account):
    """
    Set the session data: user_id, username and group_id for the supplied account.
    """
    ac = AccountController(request.Session(), account_id = account.id, request=request)
    ac.assert_permissions('soap_login')

    request.session['user_id'] = account.id
    request.session['username'] = account.name

    
    if not ac.config.get('basesoft.web.contrib.locale'):
        ac.config.set('basesoft.web.contrib.locale', 'en_GB')
    request.session['config'] = ac.config.get_configs()

    # get all the groups that include the current account
    account_groups = ac.get_groups_for_account(account)

    #if account has only one group, save that one
    if (len(account_groups) == 1):
        request.session['group_id'] = account_groups[0].group.id

def register_soap_functions(server):
    """
    These are global IQPortal SOAP function which can be used for the different servers.
    """

    @register(to=server, args=((str, None, ), (str, None, )), needs_context=True)
    def login(name, password, context):
        """
        Authenticates against the SOAP service. <br />
        When succesfull, a HTTP response is returned, containing the current session_id. The session_id must be passed
        to all functions that require authentication.
        """

        # TODO: fix this in WSDL with nillable set to false.
        if name is None or password is None:
            raise Exception('Missing required "name" and "password"')

        logger = logging.getLogger('basesoft.web.contrib.soap', request=context)
        logger.info('login')

        c = AccountController(context.Session())
        account = c.validate_account(name, password)
        if isinstance(account, Account):
            set_session_data(context, account)
            return
        
        raise PublicUserException('Invalid login')

