# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext

from basesoft.web.core.models import PrivilegeObject

class Ticket(PrivilegeObject):
    """
    Model representing a Ticket.
    @DynamicAttrs
    """
    PRIV_ADD_COMMENT = 32

    all_privileges = (PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE, PrivilegeObject.PRIV_ARCHIVE,
                      PrivilegeObject.PRIV_DELETE, PrivilegeObject.PRIV_DELEGATE_PRIVILEGES, PRIV_ADD_COMMENT)

    mark_as_deleted_permission = 'delete_ticket'
    mark_as_archived_permission = 'update_ticket'

    _name_property = 'trac_ticket_id'

    def get_privileges(self):
        return PrivilegeObject.get_privileges(self) + [
            (self.PRIV_ADD_COMMENT, ugettext('Add comments to the ticket')),
        ]

    def get_group_object(self):
        return (TicketGroup, 'ticket')

    def __repr__(self):
        return '<Ticket id=%s>' % self.id

class TicketGroup(object):
    """
    Model representing the link between tickets and groups. Used for permissions.
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TicketGroup>'

class TicketAttachmentHash(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TicketAttachmentHash>'
