# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

import re, cgi
from basesoft.web.contrib.wiki.formatter import HTMLFormatter


class Macro:
    def __init__(self, context, formatter=None):
        self.context = context
        self._formatter = formatter

    def format(self, wiki):
        if self._formatter is None:
            return wiki
        return self._formatter.format(wiki)


class QuoteBlock(Macro):
    """
    """
    options = re.compile('^(?:"(?P<quote>[^"]*)","(?P<quotee>[^"]*)"(?:,"(?P<link>.*)")?)|(?P<fullquote>(.*))$');

    def execute(self, quote):
        """
        """
        m = self.options.match(quote)
        if m.group('quote') is None:
            return '<div class="quote"><span class="quote">"%s"</span></div>' % m.group('fullquote')

        legend = m.group('quotee')
        if m.group('link') is not None:
            legend = '<a href="%s" target="_blank">%s</a>' % (m.group('link'), legend)

        return '<div class="quote"><span class="quote">"%s"</span> - %s</div>' % (m.group('quote'), legend)


class Image(Macro):
    def execute(self, image_hash):
        return '<img src="/basesoft.web.contrib.file/view/%s" />' % image_hash


class ExternalImage(Macro):
    def execute(self, image_location):
        return '<img src="%s" />' % image_location


class Footer(Macro):
    def execute(self, footer_text):
        return '<div class="page-footer">%s</div>' % footer_text


class BR(Macro):
    def execute(self, bitbucket):
        return '<br />'


class PageIndex(Macro):

    def execute(self, bitbucket):
        import re

        rule = re.compile(r"^(?P<hdepth>=+)\s(?P<wiki>.*)\s(?P=hdepth)\s*$", re.UNICODE)

        rv = ""
        text = self._formatter.get_original_text()
        for line in text.splitlines():
            m = re.match(rule, line)
            if m is None:
                continue

            depth = len(m.group('hdepth'))
            rv += ' '*depth
            rv += '* %s\n' % m.group('wiki')  # TODO, make anchors

        return '<div class="wiki-page-index">%s</div>' % self.format(rv)


class Parser:
    """
    """
    macros = {
        'QuoteBlock': QuoteBlock,
        'Image': Image,
        'Footer': Footer,
        'ExternalImage': ExternalImage,
        'BR': BR,
        'PageIndex': PageIndex,
    }

    def __init__(self):
        """
        """
        self.content = ''
        self.context = None
        self._wikis = None
        self._words = None

    def add_macro(self, name, obj):
        self.macros[name] = obj

    def set_content(self, content):
        """
        """
        self.content = content

    def set_context(self, context):
        self.context = context

    def set_available_wikis(self, wikis):
        self._wikis = wikis

    def set_highlight_words(self, words):
        self._words = words

    def render_to_html(self, content=None, context=None):
        """
        """
        result = ''
        if content is None:
            content = self.content
        if context is None:
            context = self.context

        try:
            f = HTMLFormatter(self.macros, context=context)
            f.set_available_wikis(self._wikis)
            f.set_highlight_words(self._words)
            result = f.format(content)
        except WikiMacroException, e:
            result = '<div class="wiki-error">%s</div>' % e.message

        return result

class WikiMacroException(Exception):
    """
    """

