# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the compose application.
"""
import datetime
from sqlalchemy import and_, Column, Integer, ForeignKey, String, \
                       DateTime, UnicodeText, Unicode, SmallInteger, UniqueConstraint
from sqlalchemy.dialects.mysql import MEDIUMTEXT
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

from basesoft.web.contrib.i18n import models as i18n_models
from sqlalchemy.types import Boolean
from basesoft.web.contrib.wiki.models import Wiki, WikiContent
from basesoft.web.contrib.i18n.models import Locale

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblWiki = IQTable('tblWiki', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblWikiContent = IQTable('tblWikiLocaleRevision', metadata,
            Column('wiki_id', Integer, ForeignKey('tblWiki.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='RESTRICT'), nullable=True),
            Column('revision', Integer, primary_key=True), # increments per wiki_id/locale_id
            Column('title', String(255), nullable=False),
            Column('content', MEDIUMTEXT, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        )

        tables['tblWiki'] = tblWiki
        tables['tblWikiContent'] = tblWikiContent

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation

        from basesoft.web.contrib.account import models as account_models

        mapper(Wiki, tables['tblWiki'], properties = {
            'fallback_locale': relation(Locale),
            'available_locales': relation(Locale, secondary=tables['tblWikiContent'])
        })

        mapper(WikiContent, tables['tblWikiContent'], properties = {
            'wiki': relation(Wiki),
            'locale': relation(Locale),
            'account': relation(account_models.Account),
        })

        __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Wiki')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    return configs

