"""
A set of request processors that return dictionaries to be merged into a
template context. Each function takes the request object as its only parameter
and returns a dictionary to add to the context.

These are referenced from the setting TEMPLATE_CONTEXT_PROCESSORS and used by
RequestContext.
"""

from django.conf import settings

from basesoft import logging
from basesoft.web.contrib.branding.controllers import SiteController
logger = logging.getLogger("basesoft.core")

def debug(request):
    "Returns context variables helpful for debugging."
    context_extras = {}

    if settings.DEBUG:
        context_extras['debug'] = True

    if settings.YUI_DEBUG_FILES:
        context_extras['yui_debug_files'] = True

    return context_extras


def extras(request):
    "Returns a context of variables that are used for generic purposes."
    from datetime import datetime
    context_extras = {}

    # iqportal version
    context_extras['VERSION'] = settings.VERSION
    context_extras['SITE_NAME'] = None if not settings.SHOW_SITE_NAME else settings.SITE_NAME


    # current date; in templates, parts of the date can be accessed like this: {{ current_date.year }}
    context_extras['current_date'] = datetime.now()

    sc = SiteController(request.Session(), request=request)
    site = sc.get_current_site()

    # google analytics tracking
    context_extras['google_analytics_id'] = site.analytics_id
    return context_extras


def auth(request):
    """
    Returns context variables required by apps that use the sqlalchemy authentication system
    """
    username = False

    # check if we have the username
    try:
        username = request.session['username']
    except:
        pass

    return {
        'username': username,
    }


def i18n(request):

    context_extras = {}
    context_extras['LANGUAGES'] = settings.LANGUAGES
    context_extras['LANGUAGE_CODE'] = request.session.get('django_language', None)

    return context_extras
