# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
Provides methods that will be called from rpc.
"""

from django.conf import settings

from django.forms.util import ValidationError

import logging
from basesoft.web.core.exceptions import SecurityException
logger = logging.getLogger()

class FormValidator:
    """
    This can do form validation.

    Documentation: http://trac.office.basesoft.com/iqportal/wiki/IQReportFormValidation

    Example of group plugins:
    ---
    form_handler = FormValidator(
        group_plugins = [(group_plugins.Required, {
            'email': {'username': False},
            'username': {'email': False}
        })]
    )


    """
    def __init__(self, fields=None, group_plugins=None):
        """
        """
        if fields is None:
            self.fields = {}
        else:
            self.fields = fields

        if group_plugins is None:
            self.group_plugins = []
        else:
            self.group_plugins = group_plugins

    def validate(self, form_fields):
        """
        """
        assert isinstance(form_fields, dict)

        result = {}
        # if settings form validation is False we dont check anything.
        if not settings.FORM_VALIDATION:
            return result

        # check field plugins
        for field_name, field_values in form_fields.iteritems():

            # recursive row control validation
            if isinstance(field_values, list):
                for value in field_values:
                    if type(value) != dict:
                        continue
                    validate_rv = self.validate(value)
                    if not result.has_key(field_name):
                        result[field_name] = []
                    result[field_name].append(validate_rv)

            # apparently we don't need to check this field.
            if field_name not in self.fields:
                logger.debug('FormValidator: not validating: %s' % field_name)
                continue

            for plugin_class, plugin_params in self.fields[field_name]:
                try:
                    fb = plugin_class(field_values)
                    fb.check(plugin_params)
                except ValidationError, e:
                    if field_name not in result:
                        result[field_name] = []
                    # e.messsages is of type ErrorList
                    result[field_name].append(e.messages[0])

        # check group plugins
        for gp, params in self.group_plugins:
            try:
                fb = gp(form_fields)
                fb.check(params)
            except Exception,e:
                field_name = e.fieldName
                if field_name not in result:
                    result[field_name] = []
                result[field_name].append(e.message)

        return result

    def set_field_validation(self, field_name, validation):
        """
        """
        if field_name not in self.fields:
            self.fields[field_name] = []

        self.fields[field_name].append(validation)
