# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in, assert_user_not_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

from basesoft import logging
logger = logging.getLogger("basesoft.core")

class LoginIcon(IconBarIcon):
    """
    Class responsible for handling the login icon
    """

    id = 'icon-login'
    title = _('Login')
    icon_index = 1000
    component = 'icon_login'
    href = '#'
    group = 'login'
    colorname = 'iconbar-eastern-blue'

    char_main = u'\uE833' # Login icon

    @assert_user_not_logged_in
    def assert_permissions(self, controller):
        """
        Login is shown only if the user is logged out
        """
        logger.info('Asserting permissions for the login icon')
        pass

class LogoutIcon(IconBarIcon):
    """
    Class responsible for handling the logout icon
    """

    id = 'icon-logout'
    title = _('Logout')
    icon_index = 1000
    component = 'icon_logout'
    href = '#'
    group = 'logout'
    group_title = _('Exit')
    colorname = 'iconbar-nevada'

    char_main = u'\uE825' # Logout icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        Logout is shown only if the user is logged in
        """
        logger.info('Asserting permissions for the logout icon')
        pass

class LogManagerIcon(IconBarIcon):
    """
    Class responsible for handling the log manager icon
    """

    id = 'icon-log-manager'
    title = _('Log overview')
    href = '#'
    icon_index = 93
    component = 'iq_log_manager'
    group = 'administration'
    colorname = 'iconbar-tapestry'

    char_main = u'\uE824' # Tabloid icon
    char_sub = u'\uE820' # Magnifier icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_log')

class LoginOverviewIcon(IconBarIcon):
    """
    Class responsible for handling the login overview icon
    """

    id = 'icon-login-overview'
    title = _('Login overview')
    href = '#'
    icon_index = 94
    component = 'iq_login_manager'
    group = 'administration'
    colorname = 'iconbar-tapestry'

    char_main = u'\uE824' # Tabloid icon
    char_sub = u'\uE802' # 3 lines icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_login_overview')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """
    return [LoginIcon(), LogoutIcon(), LogManagerIcon(), LoginOverviewIcon()]
