# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

import codecs
import csv

from chardet.universaldetector import UniversalDetector

class Utf8CsvImporter():
    """
    Importer for CSV
    """
    class UTF8Recoder:
        def __init__(self, f, encoding):
            self.reader = codecs.getreader(encoding)(f)

        def __iter__(self):
            return self

        def next(self):
            return self.reader.next().encode("utf-8")

    class UnicodeCsvReader:

        def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
            f = Utf8CsvImporter.UTF8Recoder(f, encoding)
            self.reader = csv.reader(f, dialect=dialect, **kwds)

        def next(self):
            row = self.reader.next()
            return [unicode(s, "utf-8") for s in row]

        def __iter__(self):
            return self

    def _read_row(self, file_path):
        # TODO, refactor method name to _read_rows.

        csvfile = open(file_path, "rb")

        #sniff 10MB. it goes wrong if a value has a comma in it which can be way up ahead the file
        dialect = csv.Sniffer().sniff(csvfile.read(1024 * 1024 * 10))
        csvfile.seek(0)

        # search for the proper encoding
        encoding = self.__detect_encoding(file_path)

        reader = Utf8CsvImporter.UnicodeCsvReader(csvfile, dialect=dialect, encoding=encoding)

        for row in reader:
            yield row

    def __detect_encoding(self, file_path):
        detector = UniversalDetector()
        infile = open(file_path, "r")
        i = 0
        for line in infile:
            detector.feed(line)
            if detector.done:
                break
            # only check for the first 1000 rows otherwise it could timeout
            if i == 1000:
                break
            i += 1
        detector.close()
        return detector.result['encoding']


class XlsxImporter():
    """
    Importer for XLSX
    """

    def _read_row(self, file_path):
        """
        :param file_path:
        :return: Active sheet object
        """
        from openpyxl.reader.excel import load_workbook

        wb = load_workbook(filename = file_path, use_iterators = True)
        active_sheet = wb.get_active_sheet()

        return active_sheet
