# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
"""
Extension of the vat.py in share.
http://trac.office.basesoft.com/iqportal/wiki/VAT_numbers
"""
import re, datetime
from basesoft.web.core.input.validation.share.vat import check_vat as share_check_vat


def check_vat_no(vat):
    '''
    Check Norwegian VAT number.
    '''
    import stdnum.no.mva
    return stdnum.no.mva.is_valid(vat)


def check_vat_is(vat):
    '''
    Check Icelandic VAT number.
    '''
    import stdnum.is_.vsk
    return stdnum.is_.vsk.is_valid(vat)


def check_vat_ch(vat):
    '''
    Check Switzerland VAT number. (2013 and later only)
    '''
    __check_vat_ch_re = re.compile(r'CHE([0-9]{9})(MWST|TVA|IVA)$')

    match = __check_vat_ch_re.match(vat)
    if not match:
        return False

    # mod11 check on the numeric part
    num = match.group(1)
    factor = (5, 4, 3, 2, 7, 6, 5, 4)
    csum = sum([int(num[i]) * factor[i] for i in range(8)])
    check = (11 - (csum % 11)) % 11
    return check == int(num[8])


def check_vat_mx(vat):
    '''
    Check Mexico VAT number
    contributed by <moylop260@hotmail.com> and Panos Christeas <p_christ@hol.gr>
    '''
    __check_vat_mx_re = re.compile(r"(?P<primeras>[A-Za-z&]{3,4})" \
                                    r"(?P<ano>[0-9]{2})(?P<mes>[01][0-9])(?P<dia>[0-3][0-9])" \
                                    r"(?P<code>[A-Za-z0-9&]{3})$")

    if vat[:2] != 'MX':
        return False
    m = __check_vat_mx_re.match(vat[2:])
    if not m:
        return False
    try:
        ano = int(m.group('ano'))
        if ano > 30:
            ano = 1900 + ano
        else:
            ano = 2000 + ano
        datetime.date(ano, int(m.group('mes')), int(m.group('dia')))
    except ValueError:
        return False

    return True


def check_vat(vat):
    '''
    Check VAT number.
    '''
    code = vat[:2].lower()
    try:
        checker = globals()['check_vat_%s' % code]
    except KeyError:
        return share_check_vat(vat)
    return checker(vat)