# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.iconbar.controllers import IconBarController
from basesoft.web.contrib.iconbar.views import iconbar_dyn_scss

from basesoft.web.core import views
from django.conf import settings
from twisted.python.reflect import namedAny




# define base modules, these will be added to the dynamc modules in the end of this file
modules = {

    # iq_core is the base of iqportal javascript land.. stuff where namespace declaration is defined
    'iq_core': {
        'type': 'js',
        'path': 'iq_core-min.js',
        'requires': [
            'yahoo', 'dom', 'event',
            'event-mouseenter', 'event-delegate', 'selector',
            'jquery', 'jquery_spinner',

            'iq_reset_scss',
            'iq_core_scss',
            'iq_core_init_scss',
        ]
    },

    'iq_core_scss': {
        'type': 'scss',
        'path': 'iq_core-min.scss',
        'requires': ['iq_debug_helper_scss', 'container_mixins_scss', 'palette_mixins_scss']
    },

    'iq_core_init_scss': {
        'type': 'scss',
        'path': 'iq_core_init-min.scss'
    },

    # include button debug helper
    'iq_debug_helper_scss': {
        'type': 'scss',
        'path': 'scss/iq_debug_helper-min.scss'
    },

    # include button mixins
    'iq_buttons_scss': {
        'type': 'scss',
        'path': 'scss/iq_buttons-min.scss'
    },

    # include containers mixins
    'container_mixins_scss': {
        'type': 'scss',
        'path': 'widget/container/container_mixins-min.scss'
    },

    # include palette mixins
    'palette_mixins_scss': {
        'type': 'scss',
        'path': 'widget/palette/palette_mixins-min.scss'
    },

    # include iconbar mixins
    'iconbar_mixins_scss': {
        'type': 'scss',
        'path': 'widget/iconbar/iconbar_mixins-min.scss'
    },

    'iconbar_dyn_scss': {
        'type': 'scss',
        'path': iconbar_dyn_scss,
        'requires': ['iconbar_mixins_scss']
    },

    'iq_reset_scss': {
        'type': 'scss',
        'path': 'iq_reset-min.scss'
    },

    'standard_theme_scss': {
        'type': 'scss',
        'path': 'standard_theme-min.scss'
    },

    'default_variables_scss': {
        'type': 'scss',
        'path': 'default_variables-min.scss'
    },

    # provides the LoadingDialog and Loader
    'iq_loader': {
        'path': 'util/loader-min.js',
        'type': 'js',
        'requires': ['iq_broker', 'get', 'iq_theme', 'iq_loader_scss']
    },

    'iq_loader_scss': {
        'path': 'util/loader-min.scss',
        'type': 'scss'
    },

    'iq_theme': {
        'type': 'js',
        'path': 'iq_theme-min.js',
        'requires': []
    },

    # iqportal_init initialises the application.. onDomReady it creates a loadingbar and initialises the WT
    'iqportal_init': {
        'type': "js",
        'path': "iqportal_init-min.js",
        'requires': []
    },

    # iq_bootloader
    'iq_bootloader': {
        'type': 'js',
        'path': 'iq_bootloader-min.js',
        'requires': ['history', 'iq_iconbar', 'iq_accountconfig', 'iq_notification', 'iq_eventlistener', 'jquery_storage']
    },

    'sockjs': {
        'type': 'js',
        'path': 'sockjs-min.js',
    },

    'iq_sockjs': {
        'type': 'js',
        'path': 'iq_sockjs-min.js',
        'requires': ['sockjs']
    },

    'iq_boot_register': {
        'type': 'js',
        'path': views.js_module_boot_register
    },

    # abstract style for iqportal
    'iqportal_scss': {
        'type': "scss",
        'path': "css/iqportal.scss",
        'requires': ["grids", 'reset', 'skin-sam-layout']
    },

    # base values used for styling the layout in js-land
    'default_theme_env': {
        'type': 'js',
        'path': 'default_theme_env-min.js',
        'requires': []
    },

    'basesoft_theme_env' :{
        'type': "js",
        'path': "themes/basesoft/theme_env-min.js",
        'requires': ["default_theme_env"]
    },

    # css used to override the style of the website (before logging)
    'basesoft_website_theme': {
        'type': 'css',
        'path': 'themes/basesoft/website-min.css',
        'requires': []
    },


    'basesoft_theme_default_scss': {
        'type': 'scss',
        'path': 'themes/basesoft/main-min.scss',
        'requires': ['iq_core_scss']
    },

    'basesoft_theme_default': {
        'type': 'css',
        'path': 'themes/basesoft/main-min.css',
        'requires': ['basesoft_theme_env', 'basesoft_website_theme', 'basesoft_theme_default_scss', 'standard_theme_scss']  # skin-sam-layout comes from yui
    },

    'newcom_theme_env' :{
        'type': "js",
        'path': "themes/newcom/theme_env-min.js",
        'requires': ["default_theme_env"]
    },

    # css used to override the style of the website (before logging)
    'newcom_website_theme': {
        'type': 'css',
        'path': 'themes/newcom/website-min.css',
        'requires': []
    },

    'newcom_theme_default_scss': {
        'type': 'scss',
        'path': 'themes/newcom/main-min.scss',
        'requires': ['iq_core_scss']
    },

    'newcom_theme_default': {
        'type': 'css',
        'path': 'themes/newcom/main-min.css',
        'requires': ['skin-sam-layout', 'newcom_theme_env', 'newcom_website_theme', 'newcom_theme_default_scss', 'standard_theme_scss']  # skin-sam-layout comes from yui
    },

    # UNIFIED 3C
    'unified3c_theme_env' :{
        'type': "js",
        'path': "themes/unified3c/theme_env-min.js",
        'requires': ["default_theme_env"]
    },

    'unified3c_theme': {
        'type': 'css',
        'path': 'themes/basesoft/main-min.css',
        'requires': ['unified3c_theme_env', 'basesoft_website_theme']  # skin-sam-layout comes from yui
    },


    # default theme (for now)
    'iq_theme_icecream': {
        'type': 'css',
        'path': 'themes/icecream/main-min.css',
        'requires': ['skin-sam-layout', 'default_theme_env']  # from yui
    },

    # debug theme (yui sam)
    'iq_theme_debug': {
        'type': 'css',
        'path': 'themes/yui_sam/main-min.css',
        'requires': ['skin-sam-layout', 'default_theme_env']
    },

    # jsonrpc and other async communication
    'iq_broker': {
        'type': "js",
        'path': "util/broker-min.js",
        'requires': ["event", "json", "connection", "iq_core", 'iq_util']
    },

    'iq_validation': {
        'type': "js",
        'path': "util/validation.js",
        'requires': ["element", "iq_broker", "iq_util", "event", "iq_growl"]
    },

    'iq_util': {
        'type': "js",
        'path': "util/util-min.js"
    },

    'iq_md5': {
        'path': 'util/md5-min.js',
        'type': 'js'
    },

    'iq_eventlistener': {
        'type': "js",
        'path': "util/eventlistener-min.js",
    },

    'iq_container': {
        'type': "js",
        'path': "widget/container/container-min.js",
        'requires': ['element', 'container', 'iq_util', 'iq_container_scss', 'jquery_ui_resizable']
    },

    'iq_container_scss': {
        'type': "scss",
        'path': "widget/container/container-min.scss",
        'requires':['container_mixins_scss']
    },

    'iq_details_panel': {
        'type': "js",
        'path': "widget/container/details_panel-min.js",
        'requires': ['container', 'iq_form_manager']
    },

    'iq_dragdrop': {
        'type': "js",
        'path': "widget/dragdrop/dragdrop-min.js",
        'requires': ['dragdrop']
    },

    'iq_tabview': {
        'type': "js",
        'path': "widget/tabview/tabview-min.js",
        'requires': ['tabview']
    },

    'iq_accountconfig': {
        'type': "js",
        'path': "util/account_config-min.js",
        'requires': ['event']
    },

    'iq_notification': {
        'type': "js",
        'path': "util/notification-min.js",
        'requires': ['event', "iq_broker"]
    },

    'iq_dialog': {
        'type': "js",
        'path': "widget/dialog/dialog-min.js",
        'requires': ['event', 'iq_container', 'iq_dialog_scss']
    },

    'iq_dialog_scss': {
        'type': "scss",
        'path': "widget/dialog/dialog-min.scss"
    },

    'iq_wizard': {
        'type': "js",
        'path': "widget/wizard/wizard-min.js",
        'requires': ["dom", "event", "element", "button", "history", "iq_wizard_css", "iq_wizard_scss"]
    },

    'iq_wizard_css': {
        'type': "css",
        'path': "widget/wizard/wizard-min.css"
    },

    'iq_wizard_scss': {
        'type': "scss",
        'path': "widget/wizard/wizard-min.scss"
    },

    'iq_iconbar': {
        'type': "js",
        'path': "widget/iconbar/iconbar-min.js",
        'requires': ["iq_broker", "button", 'animation', "iq_iconbar_css", "iq_iconbar_scss", "jquery_corner", "jquery_velocity"]
    },

    'iq_iconbar_css': {
        'type': "css",
        'path': "widget/iconbar/iconbar-min.css"
    },

    'iq_iconbar_scss': {
        'type': "scss",
        'path': "widget/iconbar/iconbar-min.scss",
        'requires': ['iconbar_mixins_scss', ]
    },

    'iq_editor_fix': {
        'type': "js",
        'path': "widget/editor_fix-min.js",
        'requires': ["iq_util"]
    },

    'iq_filter_fieldset': {
        'type': "js",
        'path': "widget/filter/filter-min.js",
        'requires': ["iq_form_manager", "container", "button", "iq_filter_fieldset_scss", "iq_fancyselect"]
    },

    'iq_filter_fieldset_scss': {
        'type': "scss",
        'path': "widget/filter/filter-min.scss",
        'requires': ["iq_form_core_scss", "iq_animate_properties_scss", "iq_animate_fading_exits_scss", "iq_animate_fading_entrances_scss"]
    },

    'iq_custom_query': {
        'type': "js",
        'path': "widget/custom_query/custom_query-min.js",
        'requires': ["dom", "history", "iq_customdatatable", "container", "button", "iq_broker", "iq_custom_query_scss", "iq_custom_query_css", "iq_util", "iq_filter_fieldset"]
    },

    'iq_custom_query_css': {
        'type': "css",
        'path': "widget/custom_query/custom_query-min.css"
    },

    'iq_custom_query_scss': {
        'type': "scss",
        'path': "widget/custom_query/custom_query-min.scss"
    },

    'iq_coverflow': {
        'type': "js",
        'path': "widget/coverflow/coverflow-min.js",
        'requires': ["carousel", "iq_coverflow_css"]
    },

    'iq_coverflow_css': {
        'type': "css",
        'path': "widget/coverflow/coverflow-min.css"
    },

    'iq_notify': {
        'type': "js",
        'path': "widget/notify/notify-min.js",
        'requires': ['iq_container', 'iq_notify_css']
    },

    'iq_notify_css': {
        'type': "css",
        'path': "widget/notify/notify-min.css"
    },

    'iq_growl': {
        'type': "js",
        'path': "widget/growl/growl-min.js",
        'requires': ['iq_container', 'iq_growl_scss', 'animation', 'iq_md5']
    },

    'iq_growl_scss': {
        'type': "scss",
        'path': "widget/growl/growl-min.scss",
        'requires': ['iq_growl_mixins_scss']
    },

    # include growl mixins, used by: iq_core_scss
    'iq_growl_mixins_scss': {
        'type': "scss",
        'path': "widget/growl/growl_mixins-min.scss"
    },

    'iq_history': {
        'type': "js",
        'path': "widget/history/history-min.js",
        'requires': ['container', 'tabview', 'button', 'iq_dialog', 'iq_history_css']
    },

    'iq_history_css': {
        'type': "css",
        'path': "widget/history/history-min.css"
    },

    'iq_palette': {
        'type': "js",
        'path': "widget/palette/palette-min.js",
        'requires': ['iq_container', 'button', 'dragdrop', 'resize', 'iq_palette_scss', 'iq_dragdrop', 'iq_compose']
    },

    'iq_palette_scss': {
        'type': "scss",
        'path': "widget/palette/palette-min.scss",
        'requires': ['palette_mixins_scss']
    },

    'iq_actionpalette': {
        'type': "js",
        'path': "widget/actionpalette/actionpalette-min.js",
        'requires': ['container', 'button', 'iq_actionpalette_scss']
    },

    'iq_actionpalette_scss': {
        'type': "scss",
        'path': "widget/actionpalette/actionpalette-min.scss"
    },

    'iq_rte': {
        'type': 'js',
        'path': 'widget/rte/rte-min.js',
        'requires': ['editor', 'iq_rte_css', 'iq_menu']
    },
    'iq_rte_css': {
        'type': 'css',
        'path': 'widget/rte/rte-min.css',
    },

    'iq_editor_manager': {
        'type': "js",
        'path': "util/editor_manager/editor_manager-min.js",
        'requires': ['iq_editor_manager_scss']
    },

    'iq_editor_manager_scss': {
        'type': "scss",
        'path': "util/editor_manager/editor_manager-min.scss"
    },

    'iq_fancyselect': {
        'type': "js",
        'path': "widget/fancyselect/fancyselect-min.js",
        'requires': ['button', 'iq_fancyselect_scss', 'iq_util']
    },

    'iq_fancyselect_scss': {
        'type': 'scss',
        'path': "widget/fancyselect/fancyselect-min.scss"
    },

    'iq_multipartview': {
        'type': "js",
        'path': "widget/multipartview/multipartview-min.js",
        'requires': ['tabview', 'iq_multipartview_css']
    },

    'iq_multipartview_css': {
        'type': 'css',
        'path': "widget/multipartview/multipartview-min.css"
    },

    'iq_formhandler': {
        'type': "js",
        'path': "util/formhandler-min.js",
        'requires': ["iq_validation"]
    },

    'iq_customdatatable': {
        'type': "js",
        'path': "widget/customdatatable/customdatatable-min.js",
        'requires': ["dragdrop", "datatable", "paginator", "iq_util", "iq_customdatatable_scss"]
    },

    'iq_customdatatable_scss': {
        'type': "scss",
        'path': "widget/customdatatable/customdatatable-min.scss",
        'requires': ["iq_customdatatable_mixins_scss"]
    },

    'iq_editors': {
        'type': 'js',
        'path': 'widget/customdatatable/editors-min.js',
        'requires': ['datatable', 'iq_editors_scss']
    },

    'iq_editors_scss': {
        'type': 'scss',
        'path': 'widget/customdatatable/editors-min.scss'
    },

    'iq_compose': {
        'type': "js",
        'path': "widget/compose/compose-min.js",
        'requires': ['iq_dialog', 'iq_container', 'animation', "iq_compose_scss", 'iq_compose_translations']
    },

    'iq_compose_scss': {
        'type': "scss",
        'path': "widget/compose/compose-min.scss",
        'requires': ['palette_mixins_scss']
    },

    'iq_compose_runner_scss': {
        'type': "scss",
        'path': "widget/compose/compose_runner-min.scss",
        'requires': ['default_variables_scss']
    },

    'iq_compose_properties': {
        'type': 'js',
        'path': "widget/compose/compose_properties-min.js",
        'requires': ['container', 'iq_fancyselect']
    },

    'iq_compose_translations': {
        'type': 'js',
        'path': "widget/compose/compose_translations-min.js",
        'requires': ['container']
    },

    'iq_compose_column_properties': {
        'type': 'js',
        'path': "widget/compose/column_properties-min.js",
        'requires': ['container', 'iq_form_manager']
    },


    # iq_form
    # bundles iq_form_core with basic input types
    'iq_form': {
        'type': "js",
        'path': "form/form-min.js",
        'requires': [
            'iq_form_core',
            'iq_form_layout',
            'iq_form_row_control',
            'iq_form_core_input',
            'iq_form_text',
            'iq_form_button',
            'iq_form_select',
            'iq_form_checkbox',
        ],
    },

    # iq_form_manager is deprecated!
    # this includes too many modules. use iq_form
    'iq_form_manager': {
        'type': "js",
        'path': "form/form-manager-min.js",
        'requires': [
            'iq_form',
            'iq_form_date',
            'iq_form_autocomplete',
            'iq_form_colorpicker',
            'iq_form_editable',
            'iq_form_upload',
            'iq_toolbar'
        ]
    },

    # iq_form_core only contains the Form class itself
    'iq_form_core': {
        'type': "js",
        'path': "form/form_core-min.js",
        'requires': [
            'element', 'iq_validation', 'iq_broker', 'iq_growl',
            'iq_form_core_css', 'iq_form_core_scss',
        ]
    },
    'iq_form_core_css': {
        'type': "css",
        'path': "form/form_core-min.css",
    },
    'iq_form_core_scss': {
        'type': "scss",
        'path': "form/form_core-min.scss",
        'requires': [
            "iq_animate_properties_scss", "iq_animate_sliders_scss"
        ]
    },

    # layout
    'iq_form_layout': {
        'type': "js",
        'path': "form/layout-min.js",
        'requires': ['iq_form_core'],
    },

    # row_control
    'iq_form_row_control': {
        'type': "js",
        'path': "form/row_control-min.js",
        'requires': ['iq_form_core'],
    },


    # iq_form_core_input contains basic and abstract form input classes
    'iq_form_core_input': {
        'type': "js",
        'path': "form/form_core_input-min.js",
        'requires': ['iq_form_core'],
    },

    # basic text inputs (text, decimal, textarea etc)
    'iq_form_text': {
        'type': "js",
        'path': "form/text-min.js",
        'requires': ['iq_form_core_input', 'jquery_numpadDecSeparator'],
    },

    # button
    'iq_form_button': {
        'type': "js",
        'path': "form/button-min.js",
        'requires': ['iq_form_core_input'],
    },

    # select
    'iq_form_select': {
        'type': "js",
        'path': "form/select-min.js",
        'requires': ['iq_form_core_input'],
    },

    # form: checkbox
    'iq_form_checkbox': {
        'type': "js",
        'path': "form/checkbox-min.js",
        'requires': ['iq_form_core_input']
    },

    # form date inputs
    'iq_form_date': {
        'type': "js",
        'path': "form/date-min.js",
        'requires': ['iq_form_core_input', 'calendar']
    },

    # autocomplete. deprecated! use autocomplete2
    'iq_form_autocomplete': {
        'type': "js",
        'path': "form/autocomplete-min.js",
        'requires': ['iq_form_core_input'],
    },

    # autocomplete2
    'iq_form_autocomplete2': {
        'type': "js",
        'path': "form/autocomplete2-min.js",
        'requires': ['iq_form_core_input', 'iq_form_autocomplete2_scss', 'jquery_ui_autocomplete', 'iq_datasource']
    },
    'iq_form_autocomplete2_scss': {
        'type': "scss",
        'path': "form/autocomplete2-min.scss",
        'requires': []
    },

    # colorpicker
    'iq_form_colorpicker': {
        'type': "js",
        'path': "form/colorpicker-min.js",
        'requires': ['iq_form_core_input', 'colorpicker']
    },

    # editable (contains InlineTextInput, RTEInput etc)
    'iq_form_editable': {
        'type': "js",
        'path': "form/editable-min.js",
        'requires': ['iq_form_core_input', 'iq_editor_manager']
    },

    # upload
    'iq_form_upload': {
        'type': "js",
        'path': "form/upload-min.js",
        'requires': ['iq_form_core_input', 'jquery_fileupload']
    },

    # end form


    'iq_datasource': {
        'type': 'js',
        'path': 'util/datasource-min.js',
        'requires': ['datasource']
    },


    'iq_manager': {
        'type': "js",
        'path': "widget/manager/manager-min.js",
        'requires': ["iq_container", "iq_menu", "button", "iq_manager_css"]
    },
    'iq_manager_css': {
        'type': "css",
        'path': "widget/manager/manager-min.css"
    },

    'iq_cq_manager': {
        'type': "js",
        'path': "widget/manager/cq_manager-min.js",
        'requires': ['iq_manager', 'iq_custom_query']
    },

    'iq_csv_parser': {
        'type': 'js',
        'path': 'util/csv_parser-min.js',
        'requires': ['iq_broker']
    },

    'iq_dynamictable': {
        'type': 'js',
        'path': 'widget/dynamictable/dynamictable-min.js',
        'requires': ['iq_dynamictable_css', 'iq_editor_manager']
    },

    'iq_dynamictable_css': {
        'type': "css",
        'path': "widget/dynamictable/dynamictable-min.css"
    },

    'iq_dynamictable_properties': {
        'type': 'js',
        'path': 'widget/dynamictable/dynamictable_properties-min.js',
        'requires': ['iqreport_fusioncharts_properties']
    },

    # Icon configuration. This modules are for the icons
    'icon_login': {
        'type': "js",
        'path': "icon-login-min.js",
        'requires': ["iq_registration_dialog"]
    },

    'icon_logout': {
        'type': "js",
        'path': "icon-logout-min.js",
        'requires': ["iq_registration_dialog"]
    },
    # end of icon configuration

    'iq_menu': {
        'type': 'js',
        'path': 'widget/menu/menu-min.js',
        'requires': ['iq_container', 'animation', 'iq_menu_scss']
    },

    'iq_menu_scss': {
        'type': 'scss',
        'path': 'widget/menu/menu-min.scss',
    },

    'iq_infobox': {
        'type': 'js',
        'path': 'widget/infobox/infobox-min.js',
        'requires': ['element', 'iq_infobox_scss']
    },
    'iq_infobox_scss': {
        'type': 'scss',
        'path': 'widget/infobox/infobox-min.scss',
    },
    'iq_testrunner': {
        'type': 'js',
        'path': 'util/testrunner/testrunner-min.js',
        'requires': ['container', 'yuitest', 'event-simulate', 'cookie', 'iq_testrunner_css', \
                     'iqportal_scss', 'iq_accountconfig', 'iq_notification', 'iq_eventlistener', "jquery_storage"]
    },
    'iq_testrunner_css': {
        'type': 'css',
        'path': 'util/testrunner/testrunner-min.css',
        'requires': ['standard_theme_scss']
    },

    'iq_chart_legend': {
        'type': "js",
        'path': "widget/chartlegend/chart_legend-min.js",
        'requires': ['iq_container', 'iq_chart_legend_css']
    },

    'iq_chart_legend_css': {
        'type': "css",
        'path': "widget/chartlegend/chart_legend-min.css"
    },

    'bs_wireit': {
        'type': "js",
        'path': 'util/wireit-min.js',
        'requires': ['wireit']
    },

    'iq_log_manager': {
        'type': "js",
        'path': "log_manager-min.js",
        'requires': ["iq_manager", "iq_custom_query"]
    },

    'iq_login_manager': {
        'type': "js",
        'path': "login_manager-min.js",
        'requires': ["iq_manager", "iq_custom_query", "iq_login_manager_scss"]
    },

    'iq_customdatatable_mixins_scss': {
        'type': 'scss',
        'path': "widget/customdatatable/customdatatable_mixins-min.scss"
    },

    'iq_login_manager_scss': {
        'type': 'scss',
        'path': "login_manager-min.scss",
        'requires': ['iq_customdatatable_mixins_scss']
    },

    'iq_toolbar': {
        'type': "js",
        'path': "widget/toolbar/toolbar-min.js",
        'requires': ["iq_toolbar_scss"]
    },

    'iq_toolbar_scss': {
        'type': "scss",
        'path': "widget/toolbar/toolbar-min.scss",
    },

}

if settings.USE_SOCK_JS:
    modules['iq_bootloader']['requires'].append('iq_sockjs')

