# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
Contains a command object to dump a iqportal application
databases in json format
"""
from django.core.management.base import BaseCommand, CommandError

from optparse import make_option

class Command(BaseCommand):
    # TODO: maybe one day support other formats
    option_list = BaseCommand.option_list + (
        make_option('--indent', default=None, dest='indent', type='int',
            help='Specifies the indent level to use when pretty-printing output'),
    )
    help = 'Output the contents of the database in json encoded format'
    args = '[appname ...]'

    def handle(self, *app_names, **options):
        import sqlalchemy
        from django.conf import settings

        options.get('indent', None)
        show_traceback = options.get('traceback', False)
        # we get a tuple of app_names
        # check we are being specific and
        # make a list
        if len(app_names) < 1:
            apps_to_dump = settings.INSTALLED_APPS
        else:
            apps_to_dump = list(app_names)
            for app in apps_to_dump:
                if app not in settings.INSTALLED_APPS:
                    raise CommandError("The iqportal application %s is not installed" % app)

        db_engine = sqlalchemy.create_engine(settings.DB_CONNSTR, echo=False)
        from django.utils import simplejson
        from django.core.serializers.json import DjangoJSONEncoder
        for app in apps_to_dump:
            try:
                sa_module = __import__(app + '.sa_datamapper', {}, {}, [''])
                metadata_to_dump = sa_module.do_get_metadata()
                for table in metadata_to_dump.table_iterator():
                    resultproxy = db_engine.execute(sqlalchemy.select([table]))
                    for i in resultproxy.fetchall():
                        print simplejson.dumps([app, table.fullname, dict(i)], cls=DjangoJSONEncoder)

            except ImportError:
                # if we do not have a datamapper then just continue as there
                # is no resetting to be done anyway
                continue
            except Exception:
                if show_traceback:
                    raise
                raise CommandError("Unable to serialize objects for app: %s" % app)
