# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
Contains a command object to load a iqportal application
database in json format, from a previously dumped file
"""
from django.core.management.base import BaseCommand, CommandError

class Command(BaseCommand):

    help = 'Load the contents of the database from json encoded format'
    args = '[appname ...] datafile'

    def handle(self, *app_names, **options):
        import os.path
        import sqlalchemy
        from django.conf import settings

        show_traceback = options.get('traceback', False)

        if len(app_names) < 1:
            raise CommandError("You need to specify a datafile to load")

        apps_to_load = list(app_names)

        # pop it out of the list
        datafile = apps_to_load.pop()

        if not os.path.exists(datafile) or not os.path.isfile(datafile) :
            raise CommandError("Data file does not exist or is not a datafile")

        if len(apps_to_load) < 1:
            apps_to_load = settings.INSTALLED_APPS
        else:
            for app in apps_to_load:
                if app not in settings.INSTALLED_APPS:
                    raise CommandError("The iqportal application %s is not installed" % app)

        db_engine = sqlalchemy.create_engine(settings.DB_CONNSTR, echo=False)

        try:
            from django.utils import simplejson
            f = open(datafile, 'r')
            try:
                imported_modules = {}
                for line in f.readlines():
                    json_list = simplejson.loads(line)
                    # check its for an app we want
                    if not json_list[0] in apps_to_load:
                        continue
                    # import the correct sa_mapper module if not already done
                    if not imported_modules.has_key(json_list[0]):
                        imported_modules[json_list[0]] = __import__(json_list[0] + '.sa_datamapper', {}, {}, [''])
                    # get a handle on the correct table object
                    table = getattr(imported_modules[json_list[0]], json_list[1])
                    db_engine.execute(table.insert(values=json_list[2]))
            finally:
                f.close()
        except Exception, e:
            if show_traceback:
                raise
            raise CommandError("Unable to load data in database: %s" % e)
