# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
"""

from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.account import models as account_models
from basesoft.web.core import models as core_models

from sqlalchemy import Column, Integer, ForeignKey, DateTime, UnicodeText, SmallInteger, String, Boolean
from basesoft.sqlalchemy.table import IQTable

from sqlalchemy.orm import mapper, relation
from django.utils.translation import ugettext as _
import datetime

from basesoft.logging import getLogger
logger = getLogger('basesoft.web.core')

__mapping_defined = False
__tables_defined = False
tables = {}


def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblLog = IQTable('tblLog', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('account_id', Integer, ForeignKey('tblAccount.id'), nullable=True),
            Column('ip', String(23), nullable=True),
            Column('level', Integer, nullable=False, default=0), # logging.NOTSET
            Column('logger_name', String(128), nullable=False),
            Column('message', UnicodeText, nullable=False),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblLogGroup = IQTable('tblLogGroup', metadata,
            Column('log_id', Integer, ForeignKey('tblLog.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblSchedulerTask = IQTable('tblSchedulerTask', metadata,
            Column('name', String(255), nullable=False, primary_key=True),
            Column('enabled', Boolean(), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblLog'] = tblLog
        tables['tblLogGroup'] = tblLogGroup
        tables['tblSchedulerTask'] = tblSchedulerTask

        __tables_defined = True


def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:
        from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper

        mapper(core_models.Log, tables['tblLog'],
            extension = [PrivilegeObjectMapper()],
            properties = {
                'account': relation(account_models.Account),
                'groups': relation(core_models.LogGroup, backref='log', cascade="all", passive_deletes=True),
            }
        )

        mapper(core_models.LogGroup, tables['tblLogGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(core_models.SchedulerTask, tables['tblSchedulerTask'])

        __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Core')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'scheduler_manager',
            u"""Permission for Scheduler Management. This includes the Scheduler Widget and the ability to manually trigger Tasks.""",
            _("Scheduler manager access")),
        license_models.Permission(u'mark_as_deleted',
            u"""Permission to mark privilege objects as deleted, this permission can be overridden on a per object base""",
            _("Privilege object mark as deleted")),
        license_models.Permission(u'mark_as_archived',
            u"""Permission to mark privilege objects as archived, this permission can be overridden on a per object base""",
            _("Privilege object mark as archived")),

        # log permissions (no need for create)
        license_models.Permission(u'update_log', u"""
            This gives the right to update a Log.
            """, _("Update logs")),
        license_models.Permission(u'delete_log', u"""
            This gives the right to delete a Log.
            """, _("Delete logs")),
        license_models.Permission(u'view_log', u"""
            This gives the right to view logs.
            """, _("View logs")),
        license_models.Permission(u'view_unbound_log', u"""
            This gives the right to view system logs that don't belong to a user.
            """, _("View unbound logs")),

        # login overview permissions
        license_models.Permission(u'view_login_overview', u"""
            This gives the right to view the login overview permissions.
            """, _("View the login overview")),
        license_models.Permission(u'kill_session', u"""
            This gives the right to terminate a user session.
            """, _("Kill user sessions")),

    ]

def app_configs():
    """
    The account configs available within all IQPortal. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.core.media.themes'
    config.type = 'string'
    config.default = None # no default, take from Site
    config.description = u'The display theme of the website.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.core.media.theme.icons'
    config.type = 'string'
    config.default = 'basesoft'
    config.description = u'The theme of the icons used in the website.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.core.growl.position'
    config.type = 'string'
    config.default = 'bl'
    config.description = u'The position where "Growl"-messages appear.'
    configs.append(config)


    config = account_models.Config()
    config.name = 'basesoft.web.core.console_log'
    config.type = 'integer'
    config.default = 4 # debug=1, info=2, warning=3, error=4
    config.description = u'Level of logging to be sent to server'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.core.default.docked'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the dockable panels are docked or not.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.core.auto.update.overviews.on.new.data'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Automatically update overviews when new data is available.'
    configs.append(config)

    return configs


def post_reset():
    """
    executed by iqresetdb. here we clear the writable directries
    """
    from django.conf import settings
    import subprocess, os
    for p in [settings.WRITABLE_DIRECTORY, settings.TMP_FILE_REPOSITORY]:

        logger.info("Emptying writable directory %s" % p)
        subprocess.check_call(['rm', '-rf', p])
        os.mkdir(p)



