# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from datetime import timedelta, datetime
from sqlalchemy import and_

from basesoft.scheduler.task import ScheduleTask
from basesoft.web.core.util.controllers import get_models_of_type
from basesoft.web.core.models import PrivilegeObject, ComposePrivilegeObject

from basesoft import logging
logger = logging.getLogger("basesoft.core")

from sqlalchemy.orm.util import class_mapper

class PrivilegeObjectGarbageCollectorTask(ScheduleTask):

    INTERVAL = 60 #86400 # every day

    def _is_sa_mapped(self, cls):
        try:
            class_mapper(cls)
            return True
        except:
            return False

    def execute(self):
        objects = get_models_of_type(PrivilegeObject, True)
        if self.sa_session.is_active:
            self.sa_session.begin_nested()
        else:
            self.sa_session.begin()
        try:
            for properties in objects:

                # skip not sqlalchemy mapped classes
                if not self._is_sa_mapped(properties['model']):
                    continue

                if properties['model'] != ComposePrivilegeObject:
                    self._delete_objects(properties['model'])

            self.sa_session.commit()

        except Exception, e:
            logger.warning('Rolling back Garbage Collection with exception message "%s"' % e.message)
            self.sa_session.rollback()
            raise

    def _delete_objects(self, model):

        max_date = datetime.now()+timedelta(days=-model.DELETION_GRACE_PERIOD)

        result = self.sa_session.query(model).filter(
            and_(model.deleted==True, model.last_updated<max_date)
        ).all()
        for obj in result:
            obj.cleanup_garbage()
            self.sa_session.delete(obj)
            logger.info('Deleted marked as deleted privilege object %s' % obj)

# for now we dont run this task till this has been tested properly
#tasks = (PrivilegeObjectGarbageCollectorTask, )
