# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
"""
The Requirements class checks for writable dirs, package versions that are needed, etc.

for version checking, each app from the settings INSTALLED_APP section needs to have a requirements.py module, that
asserts if the packages it requires have the desired version
"""

from twisted.python.reflect import namedAny
from django.conf import settings

class Requirements:
    """
    Checks if the versions of needed libraries (python, reportlab, sqlalchemy, twisted, etc) are met, if writable dirs
    exist and are writable, etc.
    """

    def check(self):
        """
        This loops all applications and checks if the version requirements are met
        """
        messages = []

        for app in settings.INSTALLED_APPS:

            try:
                namedAny(app+'.requirements')
            except AttributeError:
                continue
            except Exception, e:
                messages.append("App %s raised: %s" % (app, e.message))

        if len(messages) > 0:
            raise Exception("\n" + "\n".join(messages))

        return True
