# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

""" Views for IQPortal dev application"""

from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher

from basesoft.web.dev.compose.controllers import MockupComposeController
from basesoft.web.dev.compose.models import Mockup


LOCALE_PATH = 'basesoft.web.dev.compose'


class MockupComposeRPCHandler:
    """
    Handles rpc calls from the survey compose application. Required is to be logged in
    url: /basesoft.web.dev/compose/mockup/rpc/
    """

    def __init__(self, session, locale, account_id, group_id):

        self.account_id = account_id
        self.session = session
        self.group_id = group_id
        self.locale = locale

        # instanciate a compose controller
        self.controller = MockupComposeController(session, locale, account_id)

    def get_compose_vars(self):
        """
        called before initiating the manager.. returns the last_id and whatever is required
        """

        last_form_id = None

        form_list = self.controller.get_mockups(order_by=Mockup.last_updated)
        if len(form_list) > 0:
            last_form_id = form_list[len(form_list)-1].id

        return {
            'lastID': last_form_id
        }

    def get_categories(self):
        return [{'label': c.label, 'value': c.id} for c in self.controller.get_categories()]

    def get_mockups_for_category(self, category):
        """
        Return all mockups that belong to provided category
        """
        cat = self.controller.get_category_by_label(category)[0]
        return [{'name': m.name, 'id': m.id} for m in self.controller.get_mockups(None, None, cat)]

    def save(self, state):
        """
        Saves the form state
        """
        f = self.controller.save_state(state)
        return {state['mockup']['domID']: f.id}

    def load(self, mockup_id):
        """
        It returns a state of the form
        """
        return self.controller.load_state(mockup_id)

    def delete(self, mockup_id):
        """
        Removes form(s) with provided id(s)
        """
        return self.controller.delete_mockups(mockup_id)

@login_required
def mockup_compose_rpc_view(request):
    """View that handles all rpc calls"""

    dispatcher = JsonRpcDispatcher()

    locale = request.session['config']['basesoft.web.contrib.locale']

    if request.session.has_key('user_id'):
        account_id = request.session['user_id']
    else:
        account_id = None

    if request.session.has_key('group_id'):
        group_id = request.session['group_id']
    else:
        group_id = None

    dispatcher.register_instance(MockupComposeRPCHandler(request.Session(), locale, account_id, group_id))

    return dispatcher.handle_request(request)
