# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.mailing.models import MailLocale, Mailing

import os

import datetime
from django.conf import settings

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator, StateManager
from basesoft.web.contrib.file.controllers import UploadFileController, CSVFileController

from basesoft import logging
from basesoft.web.contrib.contact.models import Company, ContactList
from iqportal.iquest.core.models import IQuestMail, SurveyPublication
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.core.exceptions import PublicUserException
from iqportal.iquest.response.controllers import ParticipantController, ResponseController
from basesoft.web.contrib.compose.models import ComposePublicationMailing
logger = logging.getLogger("basesoft.web.dev.util")

from iqportal.iquest.core import models as iquest_models

class IQuestPHP4AbstractConversionController(AbstractController):
    db_edit = None
    db_history = None
    db_publish = None
    target_licensee = None

    def __init__(self, sa_session=None, account_id=None):
        AbstractController.__init__(self, sa_session, account_id=account_id)
        self.assert_permissions('develop_tools')

    def set_connection(self, host, db, user, password, port=3306):
        """
        This sets the connection for both a EDIT database and a HISTORY database.
        """
        if not db[-5:] == '_EDIT':
            raise Exception('Only use IQuest PHP4 `edit` databases')

        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker

        connstr = 'mysql://%s:%s@%s:%s/%s' % (user, password, host, port, db)
        db_engine = create_engine(connstr, echo=False)

        # create a new scoped session.
        self.db_edit = scoped_session(sessionmaker(
                bind=db_engine, autoflush=True, autocommit=True))

        connstr = 'mysql://%s:%s@%s:%s/%s_HISTORY' % (user, password, host, port, db[:-5])
        db_engine = create_engine(connstr, echo=False)

        # create a new scoped session.
        self.db_history = scoped_session(sessionmaker(
                bind=db_engine, autoflush=True, autocommit=True))

        connstr = 'mysql://%s:%s@%s:%s/%s_PUBLISH' % (user, password, host, port, db[:-5])
        db_engine = create_engine(connstr, echo=False)

        # create a new scoped session.
        self.db_publish = scoped_session(sessionmaker(
                bind=db_engine, autoflush=True, autocommit=True))

    def convert_accounts(self, target_licensee, group_name=None, target_group=None, active_only=True):
        """
        Please note that there are no email addresses connected to any of the accounts in IQuest PHP4
        so for now we assign them all to $accountname@mail-test.basesoft.com

        If no group_name is supplied, convert all accounts for all groups.

        TODO: add support for address & company ?
        """
        self.target_licensee = target_licensee

        sql = """
            SELECT
                U.USER_ID, U.USER_NAME, U.USER_PASSWORD, G.GROUP_NAME,
                IP.INFO_PERSON_FIRST_NAME, IP.INFO_PERSON_MIDDLE_NAME, IP.INFO_PERSON_LAST_NAME
            FROM TBL_USER as U
            INNER JOIN TBL_GROUP_USER as GU
                ON U.USER_ID = GU.USER_ID
            INNER JOIN TBL_GROUP as G
                ON GU.GROUP_ID = G.GROUP_ID
            INNER JOIN TBL_INFO_PERSON as IP
                ON U.INFO_PERSON_ID = IP.INFO_PERSON_ID
        """
        where = None
        if group_name is not None:
            where = " WHERE G.GROUP_NAME = '%s' " % group_name
        if active_only:
            if where is None:
                where = ' WHERE '
            else:
                where = '%s AND ' % where
            where = "%s U.USER_ACTIVE = %s " % (where, 1)

        if where is not None:
            sql = sql + where

        result = self.db_edit.execute(sql)

        map = {}

        from basesoft.web.contrib.account.controllers import AccountController
        from basesoft.web.contrib.contact.models import Person

        ac = AccountController(self.sa_session, account_id=self._account.id)

        # for each entry we are gonna add a new account.
        for row in result:
            person = Person()
            person.email = u'%s@mail-test.basesoft.com' % row['USER_NAME']
            person.first_name = unicode(row['INFO_PERSON_FIRST_NAME'])
            person.middle_name = unicode(row['INFO_PERSON_MIDDLE_NAME'])
            person.last_name = unicode(row['INFO_PERSON_LAST_NAME'])
            # TODO: unraaah password
            account = ac.create_account(ac.generate_account_name_from_person(person), row['USER_PASSWORD'], person)

            map['user_%s' % row['USER_ID']] = account
        return map

    def get_publications(self, include_hidden=False):
        """
        For the current connection get all the publications from the PHP IQuest server
        """
        if self.db_edit is None:
            raise Exception('No connection')

        hidden = ' AND PUBLISH_HIDDEN = 0 '
        if include_hidden:
            hidden = ''

        # only get surveys that are finished.
        result = self.db_edit.execute('SELECT PUBLISH_ID, PUBLISH_LABEL FROM TBL_PUBLISH WHERE PUBLISH_HIDDEN = 0 AND PUBLISH_TYPE = 1 AND PUBLISH_STATUS = 3 %s ORDER BY PUBLISH_LABEL' % hidden)
        return [{'value': i.PUBLISH_ID, 'label': i.PUBLISH_LABEL} for i in result]

class IQuestPHP4ConversionController(IQuestPHP4AbstractConversionController):
    """
    This controller is used to convert old BaseSoft Solution PHP4 IQuest sites to the new IQPortal
    setup.
    """

    @transaction_decorator
    def convert_survey_publications(self, source_settings, target_settings):
        """
        Convert all PHP4 publications to IQPortal IQuest publications.
        Assign all the newly created items (survey, publication, contacts) to the target group
        """
        if self.db_edit is None:
            raise Exception('No connection')

        from basesoft.web.contrib.license.models import License

        publications = source_settings['publications']
        self.target_licensee = self.sa_session.query(License).filter(License.id==target_settings['license']).one()

        for publish_id in publications:
            publish_id = int(publish_id)
            if source_settings.has_key('surveys') and source_settings['surveys']:
                sc = SurveyConverter(self, publish_id)
                sc.convert()
            if source_settings.has_key('contacts') and source_settings['contacts']:
                cc = ContactConverter(self, publish_id, target_settings['unique_identifier'])
                cc.convert()
            #mail converter
            #response converter

    def __repr__(self):
        return '<IQuestPHP4ConversionController>'

class MailConverter:
    mail_object = IQuestMail
    mail_locale_object = MailLocale
    mailing_object = ComposePublicationMailing

    def __init__(self, controller, publish_id, contact_list_map=None):
        self.controller = controller
        self.publish_id = publish_id
        self.contact_list_map = contact_list_map

    def convert(self, publication_object):
        """
        """
        self.publication_object = publication_object
        query = """
            SELECT PM.PUBLISH_MAIL_ID as pmID,
                PM.MAIL_ID as MailID,
                PM.MAIL_FILTER as Filter,
                PM.MAIL_DEF_LANGUAGE as FallbackLanguage
            FROM TBL_PUBLISH_MAIL as PM
            WHERE PM.PUBLISH_ID = %s
        """ % self.publish_id
        mails = self.controller.db_edit.execute(query).fetchall()
        for mail in mails:
            self._convert_mail(mail)

    def _get_locale_from_language_id(self, language_id):

        if language_id == 31:
            code = 'nl_NL'
        elif language_id == 44:
            code = 'en_GB'
        elif language_id == 49:
            code = 'de_DE'
        elif language_id == 33:
            code = 'fr_FR'
        else:
            raise Exception('Language ID cant be converted to Locale')

        return self.controller.sa_session.query(Locale).filter(Locale.locale==code).one()

    def _locale_callback(self, mail_locale):
        pass

    def _mailing_callback(self, mailing):
        mailing.publication = self.publication_object
        return mailing

    def _convert_mail(self, mail):
        # first we copy the mail and its locales
        m = self.mail_object()
        m.fallback_locale = self._get_locale_from_language_id(mail['FallbackLanguage'])

        query = """
            SELECT LANGUAGE_ID as LanguageID,
                STONE_SUBJECT as Subject,
                STONE_TEXT as Content,
                DATE_MODIFIED as Datetime
            FROM TBL_STONE_MAIL
            WHERE STONE_ID = %s
        """ % mail['MailID']
        for translation in self.controller.db_publish.execute(query).fetchall():
            # if it is the default language, set the name to the subject
            if translation['LanguageID'] == mail['FallbackLanguage']:
                m.name = translation['Subject']
            l = self.mail_locale_object()
            l.locale = self._get_locale_from_language_id(translation['LanguageID'])
            l.subject = translation['Subject']
            l.content = translation['Content']
            l.created = translation['DateTime']
            l.last_updated = translation['DateTime']
            self._locale_callback(l)
            m.locales.append(l)

        self.controller.sa_session.add(m)

        # get all dates for the mailing
        query = """
            SELECT MAIL_TIME as Datetime FROM TBL_PUBLISH_MAIL_DATE WHERE PUBLISH_MAIL_ID = %s
        """ % mail['pmID']

        for mail_date in self.controller.db_edit.execute(query).fetchall():
            mailing = self.mailing_object()
            mailing.filter = mail['Filter']
            mailing.mail = m
            mailing.status = Mailing.STATUS_SENT
            mailing.send_date = datetime.datetime.fromtimestamp(mail_date['Datetime'])
            self._mailing_callback(mailing)

            # for now we only do the contact list in case a map was provided
            if self.contact_list_map is None:
                self.controller.sa_session.add(mailing)
                continue

            q = """
                SELECT MAILLIST_ID as ListID
                FROM TBL_PUBLISH_MAILLIST
                WHERE PUBLISH_MAIL_ID = %s
            """ % mail['pmID']
            for maillists in self.controller.db_edit.execute(q).fetchall():
                mailing.contact_lists.append(self.contact_list_map['maillist_%s' % maillists['ListID']])

            self.controller.sa_session.add(mailing)

class ContactConverter:
    """
    TODO: add hooks, for example to split up last_names for FME where they have title included in the last name
    """

    company_object = Company
    # TODO: implement
    unique_identifier = None

    def __init__(self, controller, publish_id, unique_identifier, include_contacts, map=None):
        self.publish_id = publish_id
        self.controller = controller
        self.unique_identifier = unique_identifier
        if map is None:
            self.map = {}
        else:
            self.map = map
        self.include_contacts = include_contacts

    def convert(self):
        """
        """
        # get the invitees of a publication
        maillists = self.controller.db_edit.execute("""
            SELECT MAILLIST_ID as ID
            FROM TBL_PUBLISH_PARTICIPANTLIST
            WHERE PUBLISH_ID = %s
        """ % self.publish_id).fetchall()

        maillists = [m['ID'] for m in maillists]
        # if there were no participant lists attached, we dont have to continue
        if len(maillists) == 0:
            return

        rv = []
        for maillist_id in maillists:
            cl = self._convert_maillist(maillist_id)
            self.map['maillist_%s' % maillist_id] = cl
            rv.append(cl)
        return rv

    def _get_contact(self, fields):
        return None

    def get_company_field_map(self):
        # TODO: implement this
        return {}

    def _convert_maillist(self, id):
        """
        TODO: implement custom field conversion stuff.
        TODO: participant username and password?
        """

        field_map = self.get_company_field_map()
        from basesoft.web.contrib.contact.models import Contact, Person, ContactList

        contact_list_contacts = []
        if self.include_contacts:
            query = """
                SELECT
                    %s,
                    R.RELATION_ID as relationID,
                    R.ADDRESS_ID as fkAddr,
                    IP.INFO_PERSON_ID as personID,
                    IP.LANGUAGE_ID as languageID
                FROM TBL_MAILLIST_RELATION as MR
                INNER JOIN TBL_RELATION as R
                    ON MR.RELATION_ID = R.RELATION_ID
                INNER JOIN TBL_INFO_PERSON as IP
                    ON R.INFO_PERSON_ID = IP.INFO_PERSON_ID
                WHERE MR.MAILLIST_ID = %s
            """ % (",".join(field_map.keys()), id)

            contacts = self.controller.db_publish.execute(query).fetchall()

            logger.info('PHP4 to IQPortal - CONTACTS SIZE: %s' % len(contacts))

            new = 0
            existing = 0

            for contact in contacts:
                c = self._get_contact(contact)
                if c is not None:
                    if c not in contact_list_contacts:
                        existing = existing + 1
                        contact_list_contacts.append(c)
                    continue
                new = new + 1
                c = Contact()
                c.person = Person()
                c.company = self.company_object()
                self._attach_permissions(c)
                for field in contact.keys():
                    if field == 'fkAddr':
                        continue
                    if field == 'relationID':
                        continue
                    if field_map.has_key(field):
                        if callable(field_map[field]):
                            field_map[field](c, contact['relationID'], contact[field], contact)
                        else:
                            obj, f = field_map[field]
                            setattr(getattr(c, obj), f, contact[field])
                self.controller.sa_session.add(c)
                self.controller.sa_session.flush()
                contact_list_contacts.append(c)

            logger.info('PHP4 to IQPortal - NEW CONTACTS: %s' % new)
            logger.info('PHP4 to IQPortal - EXISTING CONTACTS: %s' % existing)

        if self.map.has_key('maillist_%s' % id):
            cl = self.map['maillist_%s' % id]
            if len(contact_list_contacts) > 0:
                cl.contacts = cl.contacts + contact_list_contacts
                self.controller.sa_session.add(cl)
            return cl

        list = self.controller.db_publish.execute("""
            SELECT MAILLIST_NAME as name
            FROM TBL_MAILLIST
            WHERE MAILLIST_ID = %s
        """ % id).fetchone()
        cl = ContactList()
        cl.name = list.name
        cl.contacts = contact_list_contacts
        self._attach_permissions(cl)
        self.controller.sa_session.add(cl)
        return cl

    def _attach_permissions(self, obj):
        obj.group = self.controller.get_default_group()
        obj.group_privileges = obj.all_privileges
        obj.domain_privileges = 0
        obj.license_privileges = 0
        obj.world_privileges = 0

class SurveyConverter:

    # maps required for responses
    question_map = {}
    subquestion_map = {}
    answer_map = {}

    def __init__(self, controller, publish_id):
        """
        Initialize the Survey Converter
        """
        element = controller.db_edit.execute('SELECT ELEMENT_ID FROM TBL_PUBLISH WHERE PUBLISH_ID = %s' % publish_id).fetchone()
        self.element_id = element['ELEMENT_ID']
        self.db = controller.db_history
        self.controller = controller
        self.load_filters()

    def load_filters(self):
        """
        Load all filter
        """
        self._filters = {}
        for f in self.controller.sa_session.query(iquest_models.IQuestQAFilter).all():
            self._filters[f.name] = f.id

    def convert(self):
        """
        Convert the PPH4 survey to an IQPortal IQuest version.
        """
        from basesoft.web.contrib.i18n import models as i18n_models
        from iqportal.iquest.compose.controllers import ComposeController

        state = self.get_state()
        locale = self.controller.sa_session.query(i18n_models.Locale).filter_by(locale=state['survey']['fallbackLocale']['locale']).one()

        cc = ComposeController(self.controller.sa_session, locale, account_id=self.controller.get_account().id)
        return cc.save_state(state, self.controller.get_default_group())

    def get_state(self):
        """
        This will create a IQuest survey by creating a state object from the settings we have.
        TODO: keep a map between domID's and the current Question/Answer id's for response saving.
        """
        # we now have to query the history database to get the actual survey which was linked to the publication.
        def_lang = self.db.execute("""
            SELECT
                Q.QUESTIONNAIRE_DEF_LANGUAGE as language_id,
                QL.QUESTIONNAIRE_LANGUAGE_TITLE as name
            FROM TBL_QUESTIONNAIRE as Q
            INNER JOIN TBL_QUESTIONNAIRE_LANGUAGE as QL
                ON Q.QUESTIONNAIRE_ID = QL.QUESTIONNAIRE_ID
                    AND Q.QUESTIONNAIRE_DEF_LANGUAGE = QL.LANGUAGE_ID
            WHERE Q.QUESTIONNAIRE_ID = %s
        """ % self.element_id).fetchone()

        # get all languages that are used in this survey
        languages = self.db.execute("""
            SELECT LANGUAGE_ID as id
            FROM TBL_QUESTIONNAIRE_LANGUAGE
            WHERE QUESTIONNAIRE_ID = %s
        """ % self.element_id).fetchall()

        survey = {
            'name': unicode(def_lang['name']), # default survey language label
            'objectName': u'Survey',
            'domID': u'survey-%s' % self.element_id,
            'dbID': None,
            'availableLocales': [self._get_locale_from_id(i['id'])['locale'] for i in languages],
            'theme': { # use the default theme for now.
                "module": u"iquest_survey_theme_basesoft",
                "name": u"BaseSoft",
                "cssClass": u"survey-basesoft",
            },
            "fallbackLocale": self._get_locale_from_id(def_lang['language_id']),
            "children": {
                'blocks': self._get_blocks()
            }
        }

        state = {'survey': survey}
        return state

    def _get_branch_end_points(self):
        """
        Get all the end points of branches. These are stone ids in the PHP IQuest version.
        """
        rv = []

        # get them from QA level
        stones = self.db.execute("""
            SELECT
                Q.BRANCH_STONE_ID as stoneID
            FROM TBL_QUESTION as Q
            INNER JOIN TBL_STONE_SURVEY_QUESTION as SSQ
                ON Q.QUESTION_ID = SSQ.QUESTION_ID
            INNER JOIN TBL_STONE as S
                ON SSQ.STONE_ID = S.STONE_ID
            WHERE Q.BRANCH_STONE_ID != 0
                AND S.STONE_ELEMENT_ID = %s
                AND S.STONE_ELEMENT_TYPE = 1
        """ % self.element_id).fetchall()

        for stone in stones:
            rv.append(stone['stoneID'])

        stones = self.db.execute("""
            SELECT
                A.BRANCH_STONE_ID as stoneID
            FROM TBL_ANSWER as A
            INNER JOIN TBL_QUESTION as Q
                ON A.QUESTION_ID = Q.QUESTION_ID
            INNER JOIN TBL_STONE_SURVEY_QUESTION as SSQ
                ON Q.QUESTION_ID = SSQ.QUESTION_ID
            INNER JOIN TBL_STONE as S
                ON SSQ.STONE_ID = S.STONE_ID
            WHERE A.BRANCH_STONE_ID != 0
                AND S.STONE_ELEMENT_ID = %s
                AND S.STONE_ELEMENT_TYPE = 1
        """ % self.element_id).fetchall()

        for stone in stones:
            rv.append(stone['stoneID'])

        return rv

    def _get_blocks(self):
        """
        Load all the blocks for this survey with their text and qa's
        """

        self._get_branch_end_points()

        blocks = []

        # TODO: also include math stones if IQPortal IQuest can handle them.
        stones = self.db.execute("""
            SELECT
                S.STONE_ID as ID,
                SSQ.QUESTION_ID as QuestionID,
                SSQ.SHOW_QUESTIONTEXT as showTitle,
                SP.STONE_PAGEBREAK_PREVIEW as pageEnd,
                S.STONE_TYPE as Type
            FROM TBL_STONE as S
            LEFT JOIN TBL_STONE_SURVEY_QUESTION as SSQ
                ON S.STONE_ID = SSQ.STONE_ID
            LEFT JOIN TBL_STONE_PAGEBREAK as SP
                ON S.STONE_ID = SP.STONE_ID
            WHERE S.STONE_ELEMENT_ID = %s
                AND S.STONE_ELEMENT_TYPE = 1
            ORDER BY S.STONE_POSITION
        """ % self.element_id).fetchall()

        items = []
        for stone in stones:
            if stone['QuestionID'] is not None:
                items.append(self._get_qa(stone['QuestionID'], stone['showTitle']))
            elif stone['Type'] == 2:# text stone
                items.append(self._get_text(stone['ID']))
            elif stone['pageEnd'] == 1:
                blocks.append({
                    'objectName': u'SurveyBlock',
                    'pageEnd': True,
                    'domID': u'block-%s' % stone['ID'],
                    'dbID': None,
                    'children': {
                        'columns': [{
                            'objectName': u'SurveyBlockColumn',
                            'domID': u'column-%s' % stone['ID'],
                            'dbID': None,
                            'width': 100,
                            'children': {
                                'items': items
                            }
                        }]
                    }
                })
                items = []

        if len(items)> 0:
            blocks.append({
                'objectName': u'SurveyBlock',
                'pageEnd': False,
                'domID': u'last-block',
                'dbID': None,
                'children': {
                    'columns': [{
                        'objectName': u'SurveyBlockColumn',
                        'domID': u'last-column',
                        'dbID': None,
                        'width': 100,
                        'children': {
                            'items': items
                        }
                    }]
                }
            })

        return blocks

    def _get_text(self, stone_id):
        """
        Get text items
        TODO: convert the replacement fields to IQPortal ones.
        """

        locales = {}
        texts = self.db.execute("""
            SELECT
                LANGUAGE_ID as LanguageID,
                STONE_TEXT as html
            FROM TBL_STONE_TEXT
            WHERE STONE_ID = %s
        """ % stone_id).fetchall()

        for text in texts:
            locales[self._get_locale_from_id(text['LanguageID'])['locale']] = {
                'html': unicode(text['html'], 'utf-8')
            }

        return {
            'objectName': u'SurveyItemHtml',
            'dbID': None,
            'domID': u'text-%s' % stone_id,
            'locales': locales
        }

    def _get_qa(self, question_id, show_title):
        """
        Get QA items
        """
        item = {
            'objectName': u'SurveyItemQA',
            'trailingQuestions': False, # this option was not possible in the PHP4 version
            'layout': None,
            'domID': u'qa-%s' % question_id,
            'dbID': None,
            'showTitle': (show_title==0),
            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
            "answersAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
            'layoutAttributes': {},
            'filters': [],
            'branching': [],
            'children': {
                'questions': [],
                'answers': []
            }
        }

        questions = self.db.execute("""
            SELECT
                Q.ANSWERTYPE_ID as Type,
                Q.QUESTION_OBLIGED as Required,
                QL.LANGUAGE_ID as LanguageID,
                QL.QUESTION_LANG_QUESTIONTEXT as Label
            FROM TBL_QUESTION as Q
            INNER JOIN TBL_QUESTION_LANG as QL
                ON Q.QUESTION_ID = QL.QUESTION_ID
            WHERE Q.QUESTION_ID = %s
        """ % question_id).fetchall()
        locales = {}
        for question in questions:
            for k, v in self._get_question_settings(question['Type']).iteritems():
                item[k] = v
            item['required'] = (question['Required']==1)
            locales[self._get_locale_from_id(question['LanguageID'])['locale']] = {
                'label': unicode(question['Label'], "utf-8")
            }
        item['locales'] = locales

        subquestions = self.db.execute("""
            SELECT
                SQ.SUBQUESTION_ID as ID,
                QL.LANGUAGE_ID as LanguageID,
                QL.QUESTION_LANG_QUESTIONTEXT as Label
            FROM TBL_SUBQUESTION as SQ
            INNER JOIN TBL_QUESTION_LANG as QL
                ON SQ.SUBQUESTION_ID = QL.SUBQUESTION_ID
                    AND SQ.QUESTION_ID = QL.QUESTION_ID
            WHERE SQ.QUESTION_ID = %s
            ORDER BY SQ.SUBQUESTION_POSITION
        """ % question_id).fetchall()

        questions = [{'objectName': u'SurveyQuestion', 'domID': u'single-question-%s' % question_id, 'dbID': None, 'locales': {}}]

        if len(subquestions) > 0:
            last_subquestion_id = None
            questions = []
            locales = {}
            for sq in subquestions:
                if last_subquestion_id is not None and sq['ID'] != last_subquestion_id:
                    questions.append({
                        'objectName': u'SurveyQuestion',
                        'domID': u'subquestion-%s' % sq['ID'],
                        'dbID': None,
                        'locales': locales})
                    locales = {}
                locales[self._get_locale_from_id(sq['LanguageID'])['locale']] = {
                    'label': unicode(sq['Label'], "utf-8")
                }
                last_subquestion_id = sq['ID']
            # always add the last one
            if len(locales) > 0:
                questions.append({'objectName': u'SurveyQuestion',
                        'domID': u'subquestion-last-%s' % question_id,
                        'dbID': None,
                        'locales': locales})

        item['children']['questions'] = questions

        answers = self.db.execute("""
            SELECT
                A.ANSWER_ID as ID,
                AL.LANGUAGE_ID as LanguageID,
                AL.ANSWER_ANSWERTEXT as Label
            FROM TBL_ANSWER as A
            INNER JOIN TBL_ANSWER_LANG as AL
                ON A.ANSWER_ID = AL.ANSWER_ID
            WHERE A.QUESTION_ID = %s
            GROUP BY A.ANSWER_POSITION, AL.LANGUAGE_ID
            ORDER BY A.ANSWER_POSITION
        """ % question_id).fetchall()

        converted_answers = [{'objectName': u'SurveyAnswer', 'domID': u'single-answer-%s' % question_id, 'dbID': None, 'locales': {}}]

        if len(answers) > 0:
            last_answer_id = None
            converted_answers = []
            locales = {}
            for answer in answers:
                if last_answer_id is not None and answer['ID'] != last_answer_id:
                    converted_answers.append({
                        'objectName': u'SurveyAnswer',
                        'domID': u'answer-%s' % answer['ID'],
                        'dbID': None,
                        'locales': locales})
                    locales = {}
                locales[self._get_locale_from_id(answer['LanguageID'])['locale']] = {
                    'label': unicode(answer['Label'], "utf-8")
                }
                last_answer_id = answer['ID']
            if len(locales) > 0:
                converted_answers.append({
                        'objectName': u'SurveyAnswer',
                        'domID': u'answer-last-%s' % question_id,
                        'dbID': None,
                        'locales': locales})

        item['children']['answers'] = converted_answers

        return item

    def _get_question_settings(self, type):
        """
        Get settings of the QA based on its type.
        """
        if type == 2: # open qa
            return {'type': iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT}
        if type == 10: # multi open
            return {
                'type': iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                'questionsAlign': iquest_models.SurveyItemQA.ALIGN_TOP,
                'answersAlign': iquest_models.SurveyItemQA.ALIGN_LEFT
            }
        if type in [3, 12]: # multiple choice, range
            return {
                'type': iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                'questionsAlign': iquest_models.SurveyItemQA.ALIGN_TOP,
                'answersAlign': iquest_models.SurveyItemQA.ALIGN_RIGHT
            }
        if type == 4: # single choice
            return {
                'type': iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                'questionsAlign': iquest_models.SurveyItemQA.ALIGN_TOP,
                'answersAlign': iquest_models.SurveyItemQA.ALIGN_RIGHT
            }
        if type == 5: # matrix single choice
            return {'type': iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE}
        if type in [6, 7]: # matrix numerical, normal numeric
            return {
                'type': iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                'filters': [{
                    'dbID': self._filters['Numeric'],
                    'questionDbID': None,
                    'answerDbID': None,
                    'attributes': {'domain': 'float'}
                }]
            }
        if type == 8: # Single choice last open. TODO: implement when available
            return {
                'type': iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                'questionsAlign': iquest_models.SurveyItemQA.ALIGN_TOP,
                'answersAlign': iquest_models.SurveyItemQA.ALIGN_RIGHT
            }
        if type == 9: # Multiple choice last open: TODO: implement when available
            return {
                'type': iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                'questionsAlign': iquest_models.SurveyItemQA.ALIGN_TOP,
                'answersAlign': iquest_models.SurveyItemQA.ALIGN_RIGHT
            }
        if type == 11: # drop down
            return {'type': iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE, 'layout': 2}

        return {'type': iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE}

    def _get_locale_from_id(self, language_id):
        """
        TODO: make a mapping between all languages and locales (for TNT eCLM mostly)
        """
        if language_id in [31, 39]:
            return {'locale': u'nl_NL', 'name': u'Dutch'}
        if language_id == 44:
            return {'locale': u'en_GB', 'name': u'English'}

    def __repr__(self):
        return '<SurveyConverter id="%s">' % self.element_id


class IQuestPHP4ImportResponseController(AbstractController):

    QA_ROW_INDEX = 0
    QUESTION_ROW_INDEX = 2
    ANSWER_ROW_INDEX = 3

    def _get_file_qas(self):
        return [i for i in self.csv.get_file_rows()[self.QA_ROW_INDEX] if i != '']

    def build_qas_from_file(self, rows):
        qas = {}
        qa_indexes = [index for index, val in enumerate(rows[self.QA_ROW_INDEX]) if val != '']

        # we have the indexes
        for i, index in enumerate(qa_indexes):

            try:
                end = qa_indexes[i+1]
            except IndexError:
                end = len(rows[self.QA_ROW_INDEX])

            questions = [qindex for qindex, val in enumerate(rows[self.QUESTION_ROW_INDEX]) if val != '' and qindex >= index and qindex < end]
            if len(questions) == 0:
                questions = [index]
                answers = [aindex for aindex, val in enumerate(rows[self.ANSWER_ROW_INDEX]) if val != '' and aindex >= index and aindex < end]
                if len(answers) == 0:
                    answers = [index]
                qas['index_%s' % index] = {'questions': questions, 'answers': answers}
                continue

            try:
                qend = questions[1]
            except:
                qend = end
            answers = [aindex for aindex, val in enumerate(rows[self.ANSWER_ROW_INDEX]) if val != '' and aindex >= index and aindex < qend]
            if len(answers) == 0:
                answers = [index]
            qas['index_%s' % index] = {'questions': questions, 'answers': answers}

        return qas

    def validate_import_file(self, survey_id, file_hash, company_nr_field, contact_list_id, qa_matches, publish_label):
        failures = []
        rows = self.set_import_file(file_hash)
        file_qas = self._get_file_qas()

        contact_fails = self.match_contacts(rows[self.ANSWER_ROW_INDEX+1:], contact_list_id, company_nr_field)
        if contact_fails != True:
            failures.append({'message': 'Not all companies are found in the contact list', 'values': [r[int(company_nr_field)] for r in contact_fails]})

        c = ComposeController(self.sa_session, None, self.account_id)
        survey = c.get_surveys(survey_id).pop()
        self._survey = survey

        if len(survey.itemQAs) != len(file_qas):
            failures.append({'message': 'The count of Survey QAs and File QAs does not match', 'values': ['IQPortal: %s <> File: %s' % (len(survey.itemQAs),len(file_qas))]})

        # match the individual QA's
        file_qas = self.build_qas_from_file(rows)
        for qa in survey.itemQAs:

            index = int(qa_matches['qa_%s' % qa.id])

            match = file_qas['index_%s' % index]
            if len(qa.questions) != len(match['questions']):
                failures.append({
                    'message': 'The count of questions for QA do not match',
                    'values': ['%s' % rows[self.QA_ROW_INDEX][index], 'IQPortal: %s <> File: %s' % (len(qa.questions), len(match['questions']))]
                })
            if len(qa.answers) != len(match['answers']):
                failures.append({
                    'message': 'The count of answers for QA do not match',
                    'values': ['%s' % rows[self.QA_ROW_INDEX][index], 'IQPortal: %s <> File: %s' % (len(qa.answers), len(match['answers']))]
                })

        values = ['Number of respondents: %s' % len(rows[self.ANSWER_ROW_INDEX+1:]), 'Number of QA\'s: %s' % len(survey.itemQAs)]

        return (failures, [{'message': 'All matches are valid, press Finish to import Response data to "%s - %s"' % (survey.name, publish_label), 'values': values}])

    def match_contacts(self, lines, contact_list, field):

        if not isinstance(contact_list, ContactList):
            # get the contact list by id
            c = ContactController(self.sa_session, self.account_id)
            contact_list = c.get_contact_lists(contact_list).pop()

        company_numbers = [str(c.company.company_number) for c in contact_list.contacts]
        failures = []
        field = int(field)
        for i in lines:
            if str(i[field]) not in company_numbers:
                failures.append(i)
            # check for multiple contacts with the same company_number in contact list

        if len(failures) > 0:
            return failures

        return True

    def set_import_file(self, file_hash):
        ufc = UploadFileController(self.sa_session)
        assert len(ufc.get_hash_temp_files(file_hash)) == 1
        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, file_hash)
        self.csv = CSVFileController(file_path)
        return self.csv.get_file_rows()

    @transaction_decorator
    def import_responses(self, survey_id, file_hash, company_nr_field, contact_list_id, qa_matches, publish_label, begin_date=None, force_import=False):
        failures, messages  = self.validate_import_file(survey_id, file_hash, company_nr_field, contact_list_id, qa_matches, publish_label)
        if len(failures) > 0 and not force_import:
            raise PublicUserException('Validation of the import file has failed, cannot import responses')

        if begin_date is None:
            begin_date = datetime.datetime.now()

        # create a publication with selected survey and contact_list
        state = {
            'survey_id': int(survey_id),
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": None,
            "status": SurveyPublication.STATUS_SCHEDULED,
            "label": publish_label,
            "show_page_nrs": False,
            "anonymous": False,
            "contact_lists": [int(contact_list_id)],
            "mailings": [],
            "viewer_print": True
        }
        locale = self._survey.fallback_locale

        pc = PublicationStateController(self.sa_session, StateManager(locale, self.get_account().id, None))
        publication = pc.save(state, group_id=self._survey.groups[0].group.id)

        publication.status = SurveyPublication.STATUS_ONLINE
        self.sa_session.add(publication)

        # have invitees created
        rc = ResponseController(self.sa_session)
        rc.save_or_update_invitees(publication)

        self.sa_session.expire(publication)

        # for each response row make a response state and save as new participant response state
        rows = self.set_import_file(file_hash)
        for response in rows[self.ANSWER_ROW_INDEX+1:]:
            self._import_response_row(response, int(company_nr_field), publication, publication.contact_lists[0], qa_matches)


        # close the publication.
        publication.status = SurveyPublication.STATUS_FINISHED
        self.sa_session.add(publication)
        return True

    def _import_response_row(self, response, company_nr_field, publication, contact_list, qa_matches):
        invitee = self._get_invitee_by_company_number(response[company_nr_field], publication, contact_list)
        controller = ParticipantController(self.sa_session, self._survey.fallback_locale, invitee.invitee_hash)

        state = []
        for qa in self._survey.itemQAs:
            state.append({
                'item_id': qa.id,
                'responses': self._get_responses_for_item(qa, response, qa_matches['qa_%s' % qa.id])
            })

        failures = controller.update_responses(state)
        if failures is not None:
            logger.info('Failures on response update for INVITEE (%s): %s' % (invitee.id, failures))

        return True

    def _get_responses_for_item(self, qa, response, match):
        rv = []
        match = int(match)
        for question in qa.questions:
            for answer in qa.answers:
                if response[match] != '':
                    rv.append({'q': question.id, 'a': answer.id, 'v': response[match]})
                else:
                    rv.append({'q': question.id, 'a': answer.id, 'v': None})
                match = match + 1

        return rv

    def _get_invitee_by_company_number(self, company_number, publication, contact_list):
        contact = None
        for c in contact_list.contacts:
            if str(company_number) == str(c.company.company_number):
                contact = c
                break
        if contact is None:
            raise PublicUserException('Contact not found in Contact List for Company Number, %s' % company_number)

        for i in publication.invitees:
            if i.contact == contact:
                return i

        raise PublicUserException('Invitee not found for contact, %s' % contact)


