# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
import operator

from twisted.python.reflect import namedAny

from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.mobile.models import MobileIcon

class MobileController(AbstractController):

    def __init__(self, sa_session, account_id=None):
        AbstractController.__init__(self, sa_session, account_id=account_id)
        self.icon_list = []

    def get_icons(self):
        from django.conf import settings
        # the list of applications that we check for icon modules
        apps = settings.INSTALLED_APPS

        icons_to_add = []
        # see if there is a icons module in the app
        for app in apps:
            try:
                icon_module = namedAny(app + '.mobile')
            except AttributeError:
                # if we do not have an icons module then the app doesn't add icons
                continue

            if hasattr(icon_module, 'get_icons'):
                # this is the list of icons that the application defines
                for icon in icon_module.get_icons():
                    assert isinstance(icon, MobileIcon)
                    # we need to check if the icons returned have the proper permissions, in
                    # order to decide if we want to add the icons to the bar or not
                    if icon.has_permissions(self):
                        icons_to_add.append({'icon': icon, 'icon_index': icon.icon_index})
        icons_to_add.sort(key=operator.itemgetter('icon_index'))
        return [i['icon'] for i in icons_to_add]
