# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.orm import mapper
from sqlalchemy import Column, Integer, ForeignKey, String, DateTime
from basesoft.sqlalchemy.table import IQTable
from django.utils.translation import ugettext as _
from basesoft.web.contrib.license import models as license_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if __tables_defined:
        return

    # Arvato Contact related tables
    tblArvatoContact = IQTable('tblArvatoContact', metadata,
        Column('contact_id', Integer, ForeignKey('tblContact.id', ondelete='CASCADE'), primary_key=True),
        Column('case_id', Integer, nullable=False),
        Column('case_created', DateTime, nullable=False),
        Column('case_created_team', String(255), nullable=False),
        Column('case_status', String(128), nullable=False),
        Column('complaint', String(32), nullable=False),
        Column('channel', String(255), nullable=False),
        #Column('taxonomy1', String(32), nullable=False),
        #Column('taxonomy2', String(32), nullable=False),
        #Column('taxonomy3', String(32), nullable=False),
    )

    tables['tblArvatoContact'] = tblArvatoContact

    __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from basesoft.web.contrib.contact import models as contact_models
    from iqportal.client.arvato import models as client_models

    mapper(client_models.ArvatoContact, tables['tblArvatoContact'],
        polymorphic_identity='ArvatoContact',
        inherits=contact_models.Contact
    )

    mapper(client_models.ArvatoCompany,
        polymorphic_identity='ArvatoCompany',
        inherits=contact_models.Company
    )

    mapper(client_models.ArvatoPerson,
        polymorphic_identity='ArvatoPerson',
        inherits=contact_models.Person
    )

    __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Arvato')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the `iqsyncpermissions` command.

    Titles are with gettext calls, but in the database they appear in English. We need it like this so we get the
    titles in the language files. Later, in the js interfaces we show them translated.
    """
    return [
        license_models.Permission(u'arvato_contact_access', u"""
            This gives the right to Arvato specific contact manager.
            """, _("Arvato Contact manager"))
        ]
