# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon
from basesoft.web.core.util.controllers import get_models_of_type
from iqportal.iqreport.core.models import ReportPublication
from basesoft.web.core.exceptions import PermissionDeniedException

class IQReportComposeIcon(IconBarIcon):
    """
    Class responsible for handling the iqreport compose icon
    """
    id = 'icon-iqreport'
    title = _('IQReport')
    app = 'iqreport'
    icon_index = 40
    group = 'iqreport'
    colorname = 'iconbar-scampi'

    char_main = u'\uE83C' # TH icon
    char_sub = u'\uE811' # Pencil icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('create_report','view_report')

class IQReportManagerIcon(IconBarIcon):
    """
    Class responsible for handling the iqreport compose icon
    """
    id = 'icon-iqreport-manager'
    title = _('Reports overview')
    icon_index = 41
    component = 'iqreport_manager'
    group = 'iqreport'
    colorname = 'iconbar-scampi'

    char_main = u'\uE823' # TH list icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_report')

class IQReportPublicationsManagerIcon(IconBarIcon):
    """
    Class responsible for handling the iqreport compose icon
    """
    id = 'icon-iqreport-publication-manager'
    title = _('Publications overview')
    icon_index = 42
    component = 'iqreport_publication_manager'
    group = 'iqreport'
    colorname = 'iconbar-scampi'

    char_main = u'\uE805' # Layers icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_report', 'view_report_publication')

class IQReportPublicationWizardIcon(IconBarIcon):
    """
    Icon to directly create a new publication
    """
    id = 'icon-report-publication-wizard'
    title = _('New report publication wizard')
    icon_index = 43
    component = 'iqreport_publication_wizard'
    group = 'iqreport'
    colorname = 'iconbar-scampi'

    char_main = u'\uE805' # Layers icon
    char_sub = u'\uE827' # Arrow right icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        This is only available if the user can create new publications.
        """
        controller.assert_permissions('create_report_publication')

        for model in get_models_of_type(ReportPublication, False, None, True):
            if controller.has_permissions(*model['model'].permissions):
                return

        raise PermissionDeniedException('Missing use report publication permissions')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [IQReportComposeIcon(), IQReportManagerIcon(), IQReportPublicationsManagerIcon(), IQReportPublicationWizardIcon()]
