# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.dashboard.controllers import DashboardWidgetCQStateController, DashboardWidgetStateController
from basesoft.util.ordered_dict import OrderedDict

from iqportal.iqreport.core.controllers import DashboardDynamicTableController

class ReportPublicationOverviewWidget(DashboardWidgetCQStateController):
    """
    Publication Overview Widget
    """
    label = _('Publication overview')
    name = 'iq_db_report_publish_overview_widget'
    icon = '16x16/categories/applications-accessories.png'
    category = 'IQReport'
    permissions = ['view_report_publication']
    classname = 'palette_group__menu--manager'


class DynamicTableWidget(DashboardDynamicTableController):
    name = 'db_dynamic_table'
    icon = '16x16/mimetypes/x-office-presentation.png'
    label = _('Dynamic table')
    category = 'IQReport'
    permissions = ['use_dynamic_table_on_dashboard']
    classname = 'palette_group__menu--iqreport'

class ReportManagerWidget(DashboardWidgetCQStateController):
    """
    Survey Manager Widget
    """
    label = _('Report overview')
    name = 'iq_db_report_manager'
    icon = '16x16/places/folder.png'
    category = 'IQReport'
    permissions = ['view_report']
    classname = 'palette_group__menu--manager'

class ItemResultWidget(DashboardWidgetStateController):
    name = 'db_item_result'
    icon = '16x16/mimetypes/x-office-presentation.png'
    label = _('Results')
    category = 'IQReport'
    permissions = ['item_result']
    classname = 'palette_group__menu--iqreport'

widgets_mapper = OrderedDict([
     ('ReportPublicationOverviewWidget', ReportPublicationOverviewWidget),
     ('DynamicTableWidget', DynamicTableWidget),
     ('ReportManagerWidget', ReportManagerWidget),
     ('DbItemResultWidget', ItemResultWidget),
])

