# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from sqlalchemy.orm import join

from basesoft.web.contrib.license.models import Group, License
from iqportal.iqreport.core.models import IQReportGraphType
from sqlalchemy.sql.expression import and_

def sync(session):
    """
    This synchronizes the themes and filters for IQuest.
    """

    graph_types = [{
        'module': 'iqreport_graph_datatablecube',
        'name': 'datatablecube',
        'label': _('Cube datatable'),
        'is_always_available': True
    },{
        'module': 'iqreport_graph_table',
        'name': 'table',
        'label': _('Table'),
        'is_always_available': True
    },{
        'module': 'iqreport_fusion',
        'name': 'iqreport_fusion',
        'label': _('Configurable Graph'),
        'is_always_available': True
    }]

    current_types = [t.name for t in session.query(IQReportGraphType).all()]
    for gt in graph_types:
        if gt['name'] in current_types:
            continue
        type = IQReportGraphType()
        for attr, value in gt.iteritems():
            setattr(type, attr, value)
        session.add(type)

    session.flush()

def post_sync(session, app_name):
    """
    Sync objects only after app_name has ran it's sync()
    @param session: the sqlalchemy session
    @param app_name: the name of the app that has been sync-ed
    """
    if app_name == 'iqportal.iqreport.core':
        sync_report_themes(session)

def sync_report_themes(session, report_themes=None):
    from iqportal.iqreport.core.models import ReportTheme
    from basesoft.web.contrib.graph.models import GraphTheme

    group = session.query(Group).join(License).filter(and_(License.name=='BaseSoft Solutions B.V.', Group.is_default==True)).one()

    if report_themes is None:

        report_themes = [{
            'name': 'BaseSoft',
            'module': 'iqreport_canvas_theme_basesoft',
            'css_class': 'iqreport-basesoft',
            'graph_theme': None,
            'group': group,
            'group_privileges': ReportTheme.all_privileges,
            'license_privileges': ReportTheme.no_privileges,
            'domain_privileges': ReportTheme.no_privileges,
            'world_privileges': ReportTheme.PRIV_VIEW
        }, {
            'name': 'Summer',
            'module': 'iqreport_canvas_theme_summer',
            'css_class': 'iqreport-summer',
            'graph_theme': session.query(GraphTheme).filter(GraphTheme.name=='Summer').one(),
            'group': group,
            'group_privileges': ReportTheme.all_privileges,
            'license_privileges': ReportTheme.no_privileges,
            'domain_privileges': ReportTheme.no_privileges,
            'world_privileges': ReportTheme.PRIV_VIEW
        }, {
            'name': 'Winter',
            'module': 'iqreport_canvas_theme_winter',
            'css_class': 'iqreport-winter',
            'graph_theme': session.query(GraphTheme).filter(GraphTheme.name=='Winter').one(),
            'group': group,
            'group_privileges': ReportTheme.all_privileges,
            'license_privileges': ReportTheme.no_privileges,
            'domain_privileges': ReportTheme.no_privileges,
            'world_privileges': ReportTheme.PRIV_VIEW
        }, {
            'name': 'Debug',
            'module': 'iqreport_canvas_theme_debug',
            'css_class': 'iqreport-debug',
            'graph_theme': None,
            'group': group,
            'group_privileges': ReportTheme.all_privileges,
            'license_privileges': ReportTheme.no_privileges,
            'domain_privileges': ReportTheme.no_privileges,
            'world_privileges': ReportTheme.no_privileges
        }]

    current_themes = [t.name for t in session.query(ReportTheme).all()]
    for theme in report_themes:
        # if we already have a theme with such a name, skip it for now.
        if theme['name'] in current_themes:
            continue

        if not theme.has_key('group'):
            theme['group'] = group

        t = ReportTheme()
        for attr in theme.keys():
            setattr(t, attr, theme[attr])
        session.add(t)

    session.flush()
