# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from iqportal.iqube.core.reporters import ReporterRpcHandler
from basesoft.util.ordered_dict import OrderedDict

import time
import os

from django.conf import settings
from django.utils.translation import ugettext as _

from basesoft.web.contrib.graph.controllers import GraphController
from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.contrib.compose.rpc import ComposePublicationHandler

from basesoft.web.contrib.mailing.rpc import MailingRPCHandler
from basesoft.web.contrib.compose.controllers import PublicationMailingStateController,\
    PublicationStateController
from basesoft.web.core.util.controllers import StateManager

from iqportal.iqube.core.controllers import DataSourceController
from iqportal.iqube.xmla.result import MockMdxResult
from iqportal.iqreport.display.controllers import ReportDisplayController
from iqportal.iqreport.core.models import ReportPublication

from iqportal.iqreport.core.controllers import ReportPublicationStateController

from basesoft import logging
from django.utils import simplejson
logger = logging.getLogger("iqportal.iqreport.core")

class ResultHandler:
    def __init__(self, request, **kwargs):
        self._request = request
        self.sa_session = request.Session()
        if request.session.has_key('user_id'):
            self.account_id = request.session['user_id']
        else:
            self.account_id = None

        self.dsc = DataSourceController(self.sa_session, account_id=self.account_id)
        self.gc = GraphController(self.sa_session, account_id=self.account_id, request=request)
        self._ds = None

    def _connect(self, datasource_id):
        if self._ds is None:
            self._ds = self.dsc.get_datasource(datasource_id)
        client = self._ds.connect(self.sa_session, account_id=self.account_id)
        return client

    def _execute_mdx(self, datasource_id, mdx_info, is_graph=False):
        """
        receives a mdx object
        """

        if isinstance(mdx_info, (str, unicode)):
            return self._execute_raw_mdx(datasource_id, mdx_info)

        assert isinstance(mdx_info, dict)

        mdx = {}
        for key, value in mdx_info.iteritems():
            mdx[str(key)] = value

        if mdx.has_key('calculated_members'):
            del mdx['calculated_members']

        mdx['is_graph'] = is_graph
        mdx['allow_empty_rows'] = True

        client = self._connect(datasource_id)
        if mdx.has_key('context'):
            client.set_context(mdx['context'])
            del mdx['context']

        if mdx.has_key('run_mode'):
            del mdx['run_mode']

        if mdx.has_key('result_full_headers'):
            del mdx['result_full_headers']

        mdx = client.build_select_statement(**mdx)
        result = client.execute_mdx(mdx)

        if mdx_info.has_key('result_full_headers') and len(mdx_info['result_full_headers']) > 0:
            result.process_full_headers(mdx_info.get('result_full_headers'), datasource_id, mdx_info.get('cube'), with_ancestors=True)

        if len(result.get_data()) == 0:
            raise PublicUserException(_('No data found for the provided selection'))

        return result

    def _wrap(self, mdx_info):
        #TODO: IQuest specific function, should be moved to iquest and requires new iqreport _wrap function
        if mdx_info.has_key('run_mode') and mdx_info['run_mode']:

            force_recache = False

            if self._request.session.has_key('report_viewer_id'):
                c = ReportDisplayController(self.sa_session, None, None, self._request.session['report_viewer_id'])
                publication = c.get_publication()

                if publication.status < ReportPublication.STATUS_ONLINE:
                    force_recache = True
                if publication.polymorphic_type == 'ReportIQuestPublication':
                    mdx_info['context'] = {'publication': publication, 'contact': c.get_report_viewer().invitee.contact, 'force_recache': force_recache}

            elif self._request.session.has_key('report_publication_id') and self._request.session['report_publication_id'] is not None:

                p = self.sa_session.query(ReportPublication).filter(ReportPublication.publication_id==self._request.session['report_publication_id']).one()

                if p.status < ReportPublication.STATUS_ONLINE:
                    force_recache = True
                if p.polymorphic_type == 'ReportIQuestPublication':
                    mdx_info['context'] = {'publication': p, 'contact': None, 'force_recache': force_recache}

    def _execute_raw_mdx(self, datasource_id, raw_mdx):
        self.dsc.assert_permissions('use_raw_mdx')
        client = self._connect(datasource_id)

        return client.execute_mdx(raw_mdx)

    def make_mdx_result(self, datasource_id, mdx_info, is_graph=False):
        # raw mdx in pivot table
        if isinstance(mdx_info, (str, unicode)):
            rs = self._execute_mdx(datasource_id, mdx_info, is_graph)
            return [(None, rs.get_dict(), )]
        self._wrap(mdx_info)

        if mdx_info.has_key('execution_style'):

            if mdx_info['execution_style'] == 'one_shot':
                mdx_info = self.get_one_shot_result(datasource_id, mdx_info)

            elif mdx_info['execution_style'] == 'mock_mdx_result':
                result = MockMdxResult(mdx_info['data'])
                return [(None, result.get_dict(), )]


        if not isinstance(mdx_info, OrderedDict):

            time_a = time.time()
            rs = self._execute_mdx(datasource_id, mdx_info, is_graph)
            logger.info('make_new_datatable took %s seconds' % (time.time() - time_a) )

            return [(None, rs.get_dict(), )]

        rv = []
        for label, mdx in mdx_info.iteritems():
            rs = self._execute_mdx(datasource_id, mdx['mdx_config'], is_graph)
            rv.append((label, rs.get_dict(), ))
        return rv

    def export_datatable_excel(self, header_defs, data):
        """
        Writes datatable cube data into xls file
        """
        import xlwt
        import random
        from hashlib import sha256 as sha

        workbook = xlwt.Workbook(encoding="UTF-8")
        sheet = workbook.add_sheet('data')

        # add headers to the sheet
        # first column with empty header for row headers
        sheet.write(0, 0, '')
        for i, col in enumerate(header_defs['columns']):
            sheet.write(0, i+1, col['label'])

        # now data
        for row_index, row_data in enumerate(data):
            # row headers
            sheet.write(row_index+1, 0, header_defs['rows'][row_index]['label'])
            for column_index, column in enumerate(header_defs['columns']):
                sheet.write(row_index+1, column_index+1, row_data[column['key']])

        hash = sha(str(random.random())).hexdigest()[:8]
        workbook.save(os.path.join(settings.TMP_FILE_REPOSITORY, 'data_%s.xls' % hash))

        return hash

    def get_one_shot_result(self, datasource_id, mdx_info):
        self._connect(datasource_id)

        override = ['context', 'run_mode']
        for o in override:
            if not mdx_info.has_key(o):
                continue
            mdx_info['config'][o] = mdx_info[o]

        reporter = ReporterRpcHandler(self.sa_session, self._ds, mdx_info['config'], account_id=self.account_id)
        return reporter.get_mdx_config(mdx_info['appliedConfig'])

    def make_table(self, datasource_id, mdx_info):
        """
        returns data needed to make a simple 2d table (iquest default report)
        """
        result = self._execute_mdx(datasource_id, mdx_info)

        headers = result.fetch_headers()
        data = result.fetch_all()
        slicers = result.fetch_slicers()

        return {
            'headers': headers,
            'data': data,
            'slicers': slicers
        }

class PublicationMailingRPCHandler(MailingRPCHandler):

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = PublicationMailingStateController(self.sa_session, StateManager(None, request.session['user_id'], None))

class ReportPublicationHandler(ComposePublicationHandler):
    controller_class = ReportPublicationStateController

    def reset_cache(self, *ids):
        return self.controller.reset_cache(*ids)


class ReportPublicationRpcHandler(ComposePublicationHandler):

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        if request.session.has_key('user_id'):
            self.account_id = request.session['user_id']
        else:
            self.account_id = None

    def get_child_models(self):
        #get all publication models for which  the user has permissions
        models = super(ReportPublicationRpcHandler, self).get_child_models()
        return [{'label': model['label'], 'value': model['value']} for model in models]


handlers = (ResultHandler, PublicationMailingRPCHandler, ReportPublicationHandler, ReportPublicationRpcHandler, )
