# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.scheduler.task import ScheduleTask
from basesoft.web.contrib.compose import schedule as compose_schedule

from basesoft import logging
from iqportal.iqreport.core.controllers import PublicationCacheController
from iqportal.iqreport.core.models import ReportPublication
from basesoft.web.contrib.compose.models import ComposePublicationMailing
from basesoft.web.contrib.mailing.schedule import MailingTask
logger = logging.getLogger('basesoft.scheduler')

class ReportPublicationCacheTask(ScheduleTask):
    INTERVAL = 3600 # every hour
    def execute(self):
        pcc = PublicationCacheController(self.sa_session)
        pcc.generate_cache(self.loop_validator)

class PublishReportsTask(compose_schedule.PublishComposeTask):
    INTERVAL = 600 # every ten minutes
    model = ReportPublication

    def execute(self):
        """
        Overwrite in order to make sure mailing runs right after.
        """
        rv = compose_schedule.PublishComposeTask.execute(self)
        # immediately send mails to invitees
        # this will also send reminder emails for online publications
        ReportMailing().execute()
        return rv

class CreatePublicationInviteesTask(compose_schedule.CreatePublicationInviteesTask):
    INTERVAL = 90
    model = ReportPublication

    def execute(self):
        affected = compose_schedule.CreatePublicationInviteesTask.execute(self)
        # we have report publications which created new invitees, set the cache status to not processed
        if len(affected) == 0:
            return

        for p in affected:
            p.cache_status = ReportPublication.STATUS_CACHE_REPROCESS
            self.sa_session.add(p)
        self.sa_session.flush()

class ReportMailing(compose_schedule.PublicationMailing):

    publication_model = ReportPublication
    mailing_model = ComposePublicationMailing

    def _filter_contact(self, mailing, contact):
        return mailing.publication.filter_contact(mailing, contact)

# publish task so the scheduler can pick them up
tasks = (ReportPublicationCacheTask, PublishReportsTask, CreatePublicationInviteesTask, ReportMailing, )