# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class IQuestComposeIcon(IconBarIcon):
    """
    Class responsible for generating the compose icon
    """

    id = 'icon-iquest'
    title = _('Survey composer')
    app = 'iquest'
    icon_index = 30
    group = 'iquest'
    colorname = 'iconbar-curious-blue'

    char_main = u'\uE83A' # Composer edit icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('create_survey', 'view_survey')

class IQuestSurveyOverview(IconBarIcon):
    """
    """
    id = 'icon-iquest-survey-overview'
    title = _('Survey overview')
    icon_index = 31
    component = 'iquest_survey_manager'
    group = 'iquest'
    colorname = 'iconbar-curious-blue'

    char_main = u'\uE802' # Bullet list icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_permissions('view_survey')

class IQuestPublicationWizardIcon(IconBarIcon):
    """
    Icon to directly schedule a new publication
    """
    id = 'icon-iquest-publication-wizard'
    title = _('New publication wizard')
    icon_index = 33
    component = 'iquest_survey_publication_wizard'
    group = 'iquest'
    colorname = 'iconbar-curious-blue'

    char_main = u'\uE805' # Layers icon
    char_sub = u'\uE827' # Arrow right icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        This is only available if the user can create new publications.
        """
        controller.assert_permissions('create_survey_publication')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [IQuestComposeIcon(), IQuestSurveyOverview(), IQuestPublicationWizardIcon()]
