# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.mailing.rpc import MailingRPCHandler
from basesoft.web.contrib.compose.controllers import PublicationMailingStateController
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.compose.rpc import ComposePublicationHandler
from iqportal.iquest.compose.controllers import PublicationStateController

class PublicationMailingRPCHandler(MailingRPCHandler):

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = PublicationMailingStateController(self.sa_session, StateManager(None, request.session['user_id'], None))

class PublicationHandler(ComposePublicationHandler):
    controller_class = PublicationStateController

    def get_publications(self, status=None):
        """
        Return all publications the user has access
        """
        if status is None:
            status = []
        if not isinstance(status, list):
            status = [status]
        publications = self.controller.get_publications()
        ret = [{'label': '%s - %s' % (p.survey.name, p.label), 'value': p.id} for p in publications if p.status in status]
        return  sorted(ret, key=lambda k: k['label'])

    def reopen_publication(self, _id):
        self.controller.reopen_publication(_id)
        return True

handlers = (PublicationMailingRPCHandler, PublicationHandler, )