# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy.sql.expression import and_

def sync(session):
    """
    This synchronizes the themes and filters for IQuest.
    """
    from sqlalchemy.orm import join

    from basesoft.web.contrib.license.models import Group, License
    from iqportal.iquest.core.models import IQuestTheme, IQuestQAFilter

    group = session.query(Group).join(License).filter(and_(License.name=='BaseSoft Solutions B.V.', Group.is_default==True)).one()

    default_themes = [{
            'name':'BaseSoft',
            'module': 'iquest_survey_theme_basesoft',
            'css_class': 'survey-basesoft',
            'group': group,
            'group_privileges': IQuestTheme.all_privileges,
            'license_privileges': IQuestTheme.no_privileges,
            'domain_privileges': IQuestTheme.no_privileges,
            'world_privileges': IQuestTheme.PRIV_VIEW
        }, {
            'name':'Newcom',
            'module': 'iquest_survey_theme_newcom',
            'css_class': 'survey-newcom',
            'group': group,
            'group_privileges': IQuestTheme.all_privileges,
            'license_privileges': IQuestTheme.no_privileges,
            'domain_privileges': IQuestTheme.no_privileges
        }, {
            'name':'TNT',
            'module': 'iquest_survey_theme_tnt',
            'css_class': 'survey-tnt',
            'group': group,
            'group_privileges': IQuestTheme.all_privileges,
            'license_privileges': IQuestTheme.no_privileges,
            'domain_privileges': IQuestTheme.no_privileges,
            'world_privileges': IQuestTheme.PRIV_VIEW
        }, {
            'name':'Debug',
            'module': 'iquest_survey_theme_debug',
            'css_class': 'survey-debug',
            'group': group,
            'group_privileges': IQuestTheme.all_privileges,
            'license_privileges': IQuestTheme.no_privileges,
            'domain_privileges': IQuestTheme.no_privileges
        }]
    props = ['name', 'module', 'css_class', 'group', 'group_privileges', 'license_privileges',
             'domain_privileges', 'world_privileges']
    current_themes = [t.name for t in session.query(IQuestTheme).all()]
    for theme in default_themes:
        # if we already have a theme with such a name, skip it for now.
        if theme['name'] in current_themes:
            continue

        t = IQuestTheme()
        for attr in props:
            if theme.has_key(attr):
                setattr(t, attr, theme[attr])

        session.add(t)

    session.flush()


    # add the default filters and attributes too
    # in the following list add the name, the name of the filter it extends or None if it's a base filter, and the list
    # of attributes. if filter extends another filter, only add attributes that are extra, the ones from the extended
    # filter don't need to be passed here again. if group, group_privileges, license_privileges, domain_privileges or
    # world_privileges are different, specify them with the keys just mentioned ('group, 'group_privileges', etc).
    # this are the default values:
    #        'group': group,
    #        'group_privileges': IQuestQAFilter.all_privileges,
    #        'license_privileges': IQuestQAFilter.no_privileges,
    #        'domain_privileges': IQuestQAFilter.no_privileges,
    #        'world_privileges': IQuestQAFilter.PRIV_VIEW

    default_filters = [{
        'name': 'Text',
        'extends_filter': None,
    }, {
        'name': 'AlphaNumeric',
        'extends_filter': 'Text',
    }, {
        'name': 'Numeric',
        'extends_filter': None,
    }, {
        'name': 'Date',
        'extends_filter': None,
    }, {
        'name': 'DateTime',
        'extends_filter': None,
#    }, {
#        'name': 'Boolean',
#        'extends_filter': None,
#        'attributes': [], # doesn't inherit anything, but doesn't have any attributes either
    }, { # these are 'plugins'
        'name': 'Email',
        'extends_filter': None,
        'attributes': []
    }, {
        'name': 'URL',
        'extends_filter': 'Text',
        'attributes': []
#    }, {
#        'name': 'IBAN',
#        'extends_filter': 'Text',
#    }, {
#        'name': 'VAT',
#        'extends_filter': 'Text',
#    }, {
#        'name': 'Grade',
#        'extends_filter': 'Numeric',
#    }, {
#        'name': 'Percentage',
#        'extends_filter': 'Numeric',
    }]



    current_filters = [f.name for f in session.query(IQuestQAFilter).all()]
    for filter in default_filters:
        # if we already have a filter with the given name, skip it
        if filter['name'] in current_filters:
            continue

        f = IQuestQAFilter()
        f.name = unicode(filter['name'])
        if filter['extends_filter'] is not None:
            extended_filter = session.query(IQuestQAFilter).filter(IQuestQAFilter.name==unicode(filter['extends_filter'])).one()
            f.extended_filter_id = extended_filter.id

        if filter.has_key('group'):
            f.group = filter['group']
        else:
            f.group = group

        if filter.has_key('group_privileges'):
            f.group_privileges = filter['group_privileges']
        else:
            f.group_privileges = IQuestQAFilter.all_privileges

        if filter.has_key('license_privileges'):
            f.license_privileges = filter['license_privileges']
        else:
            f.license_privileges = IQuestQAFilter.no_privileges

        if filter.has_key('domain_privileges'):
            f.domain_privileges = filter['domain_privileges']
        else:
            f.domain_privileges = IQuestQAFilter.no_privileges

        if filter.has_key('world_privileges'):
            f.world_privileges = filter['world_privileges']
        else:
            f.world_privileges = IQuestQAFilter.PRIV_VIEW

        session.add(f)
        session.flush()

