# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iquest.compose.controllers import ItemPoolController
from iqportal.iquest.core.models import IQuestItemPoolItemQA, SurveyPublication
from django.utils.translation import ugettext as _

class ItemPoolRPCHandler:

    def __init__(self, request, **kwargs):

        self.session = request.Session()
        self.locale = request.session['config']['basesoft.web.contrib.locale']
        self.account_id = request.session['user_id']

        self.controller = ItemPoolController(self.session, self.locale, self.account_id)

    def save_pool_item_data(self, id, data):
        """
        Save name and/or category for a pool item
        """
        return self.controller.update_pool_item(id, data['name'], data['category_id']).id

    def save_standalone_pool_item(self, state):
        """
        Save a pool item.
        """
        if state.has_key('poolItemCategory') and state['poolItemCategory']:
            c = state['poolItemCategory']
        category = self.controller.get_category_by_name(c)
        if category is None:
            category = self.controller.save_or_update_category(None, c)
        return self.controller.save_standalone_item(category, state)

    def get_item_state_from_pool(self, pool_item_id):
        state = self.controller.load_item_state(pool_item_id)
        # state is the pool item state, so we need to make it a survey item state
        # dbID None and poolItemID will become current states dbID
        state = self.controller.remove_item_state_dbIDs(state)
        return state

    def get_pool_items(self):
        self._html_count = 0
        self._multi_count = 0
        self._single_count = 0
        self._text_count = 0
        return [{'state': self.controller.load_item_state(item.id), 'category': item.pool_category.name,
                 'label': self._get_item_label(item)}
                for item in self.controller.get_items()]

    def _get_item_label(self, item, chars=20):
        if item.name is not None:
            return item.name

        if item.polymorphic_type == "IQuestItemPoolItemHtml":
            self._html_count = self._html_count + 1
            return _('Unlabeled text item %s' % self._html_count)

        def get_locale_label(el, loc):
            locale = None
            for l in el.locales:
                if l.locale.locale == loc:
                    locale = l
                    break
            if locale is not None:
                if len(locale.label) <= chars:
                    return locale.label
                mid = chars/2
                return locale.label[0:mid]+'...'+locale.label[-mid:]
            return None

        # get ItemQA locale label
        label = get_locale_label(item, self.locale)
        if label is not None:
            return label

        # get locale of the first question
        if len(item.questions) > 0:
            label = get_locale_label(item.questions[0], self.locale)
            if label is not None:
                return label

        if item.type == IQuestItemPoolItemQA.TYPE_MULTIPLE_CHOICE:
            self._multi_count = self._multi_count + 1
            return _('Unlabeled multi QA %s' % self._multi_count)
        if item.type == IQuestItemPoolItemQA.TYPE_SINGLE_CHOICE:
            self._single_count = self._single_count + 1
            return _('Unlabeled single QA %s' % self._single_count)
        self._text_count = self._text_count + 1
        return _('Unlabeled text QA %s' % self._text_count)

    def save_category(self, id, name):
        return self.controller.save_or_update_category(id, name).id

    def get_categories(self):
        return [{'value': c.id, 'label': c.name} for c in self.controller.get_categories()]

    def get_extra_pool_item_cq_data(self, category_id=None):

        categories = [{'value': c.id, 'text': c.name} for c in self.controller.get_categories()]
        if category_id is None:
            items = self.controller.get_items()
        else:
            items = self.controller.get_items_by_category(category_id)

        self._html_count = 0
        self._multi_count = 0
        self._single_count = 0
        self._text_count = 0

        items_display = {}
        for item in items:
            items_display[item.id] = self._get_item_label(item, 40)

        return {'categories': categories, 'items_display': items_display}

class SurveyPublicationRPCHandler:

    def __init__(self, request, **kwargs):

        self.session = request.Session()
        self.account_id = request.session['user_id']

    def get_multi_session_options(self):
        rv = []
        for o in SurveyPublication.get_multi_session_options():
            value, label = o
            rv.append({'value': value, 'label': label})

        return rv

handlers = (ItemPoolRPCHandler, SurveyPublicationRPCHandler, )
