# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from iqportal.iqube.core.models import DataSourceWidget

from django.utils.translation import ugettext as _

class NPSItemWidget(AbstractConfigurableWidget):
    category = 'IQ Report - IQuest'
    cube = 'VirtualResponse'
    datasource = 'IQuest'
    label = _("NPS Item")
    availability = DataSourceWidget.AVAILABLE_AS_DASHBOARD_WIDGET+DataSourceWidget.AVAILABLE_AS_REPORT_ITEM

    def config(self):
        return {
            "icon": "16x16/table.png",
            "category": "IQuest",
            "show_hide_empty": True,
            "transposable": True,
            "dimensions": [
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": True,
                    "available_as_slicer": True,
                    "caption": "Item",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Item].[Locale]",
                            "has_children_available": False,
                            "caption": "Locale",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Item].[Locale]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Item].[Label]",
                            "has_children_available": False,
                            "caption": "Label",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Item].[Label]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": True,
                    "required": True,
                    "available_as_slicer": False,
                    "caption": "Survey",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Survey].[Survey]",
                            "has_children_available": False,
                            "caption": "Survey",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Survey].[Survey]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Survey].[Publication]",
                            "has_children_available": False,
                            "caption": "Publication",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Survey].[Publication]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "caption": "Question",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Question].[Item]",
                            "has_children_available": False,
                            "caption": "Item",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Question].[Item]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Question].[Locale]",
                            "has_children_available": False,
                            "caption": "Locale",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Question].[Locale]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Question].[Label]",
                            "has_children_available": False,
                            "caption": "Label",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Question].[Label]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "caption": "Answer",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Answer].[Item]",
                            "has_children_available": False,
                            "caption": "Item",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Answer].[Item]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Answer].[Locale]",
                            "has_children_available": False,
                            "caption": "Locale",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Answer].[Locale]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Answer].[Label]",
                            "has_children_available": False,
                            "caption": "Label",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Answer].[Label]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": True,
                    "available_as_slicer": True,
                    "caption": "Measures",
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": False,
                            "caption": "MeasuresLevel",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                        }
                    ]
                }
            ],
            "reporter": "iqportal.iquest.report.NPSItemReporter",
            "has_drill_through": False,
            "datasource_id": 1,
            "label": "NPS Item",
            "show_excel_export": True,
            "has_drill_position": False,
            "cube": "Response",
            "has_graph": True,
            "id": 7,
            "table_object_name": "iqube_graph_datatablecube",
            "table_module": "iqube_graph_datatablecube",
            "has_table": True,
            "graph_config": {},
            "has_drill_replace": False,
            "has_calculated_member": True,
            'filter_panel_module': 'iquest_nps_item',
            'global_filters_module': 'iquest_global_report_filters',
        }

    def applied_filters(self):
        return {
            "axes_1": "rows",
            "axes_2": "",
            "axes_3": "",
            "axes_4": "columns",
            "axes_0": "columns",
            "dimensions": {
                "Survey": "1",
                "Question": "2",
                "Answer": "3",
                "Measures": "4",
                "Item": "0"
            },
            "column_order": ['Item', 'Measures'],
            "row_order": ['Survey'],
            "show_table": True,
            "dimension_0": [],
            "dimension_1": [],
            "dimension_2": [],
            "dimension_4": [
                "[Measures].[Respondents]"
            ],
            "dimension_3": [],
            'initialRefresh': False,
        }
