# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqreport.core.models import ReportPublication
from django.utils.translation import ugettext_lazy as _
from basesoft.web.contrib.compose.models import ComposePublicationMailing
from iqportal.iquest.response.models import SurveyParticipant

class ReportIQuestPublication(ReportPublication):
    """
    Model representing periods in which the report is available.
    @DynamicAttrs
    """

    NAME_I18N = _('IQuest report publication')

    permissions = ('use_iquest_report_publication', )

    js_module = 'iqreport_iquest_publication'

    def get_controller_instance(self):
        from iqportal.iquest.report.controllers import IQuestReportPublicationStateController
        return IQuestReportPublicationStateController

    def __repr__(self):
        return '<ReportIQuestPublication id=%s>' % (self.id)

    def filter_contact(self, mailing, contact):
        """
        This filters the contacts based on the applied filter. In case the mailing publication was linked
        to a survey publication, applied filter checks that survey publication's respondents
        """
        survey_publication = mailing.publication.survey_publication
        if survey_publication is None or mailing.filter == ComposePublicationMailing.FILTER_NONE:
            return False

        # if this contact is not linked to the survey publication, do not filter
        survey_contacts = [i.contact.id for i in survey_publication.invitees if i.contact is not None]
        if contact.id not in survey_contacts:
            return False

        from iqportal.iquest.response.controllers import ResponseController
        rc = ResponseController()

        if mailing.filter == ComposePublicationMailing.FILTER_NON_RESPONDENTS_AND_BACKOFFICE:
            return not rc.is_respondent(contact, survey_publication, respondent_types=[SurveyParticipant.RESPONSES_INTERNET])

        is_respondent = rc.is_respondent(contact, survey_publication)
        if mailing.filter == ComposePublicationMailing.FILTER_NON_RESPONDENTS:
            return not is_respondent
        else:
            return is_respondent

class ReportIQuestPublicationXMLACache(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<ReportIQuestPublicationXMLACache>'
