# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

import datetime

from sqlalchemy import Column, Integer, ForeignKey, DateTime, UnicodeText, DECIMAL, Boolean, UniqueConstraint, SmallInteger, String, Text
from sqlalchemy.dialects.mysql import MEDIUMTEXT
from sqlalchemy.schema import Index

from django.utils.translation import ugettext as _

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models


__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        aggFullIQuestResponse = IQTable('aggFullIQuestResponse', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('publication_id', Integer, index=True, nullable=False),
            Column('participant_id', Integer, index=True, nullable=False),
            Column('participant_time_id', Integer, index=True, nullable=False),
            Column('company_id', Integer, index=True, nullable=True),
            Column('person_id', Integer, index=True, nullable=True),
            Column('question_id', Integer, index=True, nullable=False),
            Column('answer_id', Integer, index=True, nullable=False),
            Column('item_id', Integer, index=True, nullable=False),
            Column('participant_session_id', Integer, index=True, nullable=False),
            Column('response_int', Integer, index=True, nullable=True),
            Column('response_float', DECIMAL(20,5), index=True, nullable=True),
            Column('weighted_int', Integer, index=True, nullable=True),
            Column('weighted_float', DECIMAL(20,5), index=True, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            mysql_engine='InnoDB',
        )

        aggFilteredIQuestResponse = IQTable('aggFilteredIQuestResponse', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('publication_id', Integer, index=True, nullable=False),
            Column('participant_id', Integer, index=True, nullable=False),
            Column('participant_time_id', Integer, index=True, nullable=False),
            Column('company_id', Integer, index=True, nullable=True),
            Column('person_id', Integer, index=True, nullable=True),
            Column('question_id', Integer, index=True, nullable=False),
            Column('answer_id', Integer, index=True, nullable=False),
            Column('item_id', Integer, index=True, nullable=False),
            Column('participant_session_id', Integer, index=True, nullable=False),
            Column('response_int', Integer, index=True, nullable=True),
            Column('response_float', DECIMAL(20,5), index=True, nullable=True),
            Column('weighted_int', Integer, index=True, nullable=True),
            Column('weighted_float', DECIMAL(20,5), index=True, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            mysql_engine='InnoDB',
        )

        aggSurveyItemQALocale = IQTable('aggSurveyItemQALocale', metadata,
            Column('item_id', Integer, nullable=False),
            Column('locale_id', Integer, nullable=False),
            Column('label', UnicodeText(), nullable=False),
            Column('position', Integer, nullable=False),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('item_id', 'locale_id', name='item_locale_unique'),
            mysql_engine='InnoDB',
        )

        aggSurveyItemQAElementLocale = IQTable('aggSurveyItemQAElementLocale', metadata,
            Column('item_id', Integer, nullable=False),
            Column('item_element_id', Integer, nullable=False),
            Column('locale_id', Integer, nullable=False),
            Column('label', UnicodeText(), nullable=False),
            Column('is_question', SmallInteger, nullable=False),
            Column('position', Integer, nullable=False),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('item_id', 'item_element_id', 'locale_id', name='item_element_locale_unique'),
            mysql_engine='InnoDB',
        )

        aggSurveyPublicationStatistics = IQTable('aggSurveyPublicationStatistics', metadata,
            Column('invitee_id', Integer, primary_key=True, autoincrement=False, nullable=False),
            Column('publication_id', Integer, index=True, nullable=False),
            Column('is_participant', Boolean, nullable=False, default=False),
            Column('is_respondent', Boolean, nullable=False, default=False),
            Column('participant_time_id', Integer, index=True, nullable=True),
            Column('company_id', Integer, index=True, nullable=True),
            Column('person_id', Integer, index=True, nullable=True),
            Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            mysql_engine='InnoDB',
        )

        tblReportIQuestPublication = IQTable('tblReportIQuestPublication', metadata,
            Column('report_publication_id', Integer, ForeignKey('tblReportPublication.publication_id'), primary_key=True),
            Column('survey_publication_id', Integer, ForeignKey('tblSurveyPublication.publication_id'))
        )

        tblReportIQuestPublicationXMLACache = IQTable('tblReportIQuestPublicationXMLACache', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('report_publication_id', Integer, ForeignKey('tblReportPublication.publication_id'), nullable=False),
            Column('company_id', Integer, ForeignKey('tblCompany.id'), nullable=True),

            Column('hash', String(32), nullable=False, index=True),
            Column('xmla_response', MEDIUMTEXT, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )
        Index('index_lookup', tblReportIQuestPublicationXMLACache.c.report_publication_id, tblReportIQuestPublicationXMLACache.c.company_id, tblReportIQuestPublicationXMLACache.c.hash)

        tables['aggFullIQuestResponse'] = aggFullIQuestResponse
        tables['aggSurveyItemQALocale'] = aggSurveyItemQALocale
        tables['aggSurveyItemQAElementLocale'] = aggSurveyItemQAElementLocale
        tables['aggFilteredIQuestResponse'] = aggFilteredIQuestResponse
        tables['aggSurveyPublicationStatistics'] = aggSurveyPublicationStatistics

        tables['tblReportIQuestPublication'] = tblReportIQuestPublication
        tables['tblReportIQuestPublicationXMLACache'] = tblReportIQuestPublicationXMLACache

        __tables_defined = True

def do_mappings():
    global __mapping_defined
    if not __mapping_defined:

        from sqlalchemy.orm import mapper, relation
        from basesoft.web.contrib.contact import models as contact_models
        from iqportal.iqreport.core import models as iqreport_models
        from iqportal.iquest.report import models as report_models
        from iqportal.iquest.core import models as core_models

        mapper(report_models.ReportIQuestPublication, tables['tblReportIQuestPublication'],
            properties = {
                'survey_publication': relation(core_models.SurveyPublication)
                },
            polymorphic_identity='ReportIQuestPublication',
            inherits=iqreport_models.ReportPublication
        )

        mapper(report_models.ReportIQuestPublicationXMLACache, tables['tblReportIQuestPublicationXMLACache'],
            properties = {
                'report_publication': relation(iqreport_models.ReportPublication),
                'company': relation(contact_models.Company)
            }
        )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'IQ Report - IQuest')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'create_individual_reports', u"""
            This gives the right to create a reports with Individual figures included.
            """, _("Create individual reports")),
        license_models.Permission(u'use_iquest_report_basket_analysis', u"""
            This gives the right to use Basket Analysis/Crossreferencing in IQuest reports.
            """, _("Use Basket Analysis/Crossreferencing")),
        license_models.Permission(u'use_iquest_time_dimension', u"""
            This gives the right to usage of the Time dimension
            """, _("Use Time Dimension")),
        license_models.Permission(u'use_iquest_person_dimensions', u"""
            This gives the right to usage of the person dimensions
            """, _("Use Person Dimensions")),
        license_models.Permission(u'use_iquest_company_dimensions', u"""
            This gives the right to usage of the company dimensions
            """, _("Use Company Dimensions")),
        license_models.Permission(u'use_iquest_report_publication', u"""
            This gives the right to use an iquest report publication.
            """, _("Use iquest report publications")),
        ]

def app_roles():
    return [{
        'name': 'IQuest report (basic)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_datasource', 'view_datasource_widget',
            'view_report', 'create_report', 'update_report',

        ],
        'license_configs': [],
    }, {
        'name': 'IQuest report (advanced)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_datasource', 'view_datasource_widget',
            'view_report', 'create_report', 'update_report', 'schedule_report',
            'view_report_theme', 'use_advanced_graphs', 'pivot_table',
            'view_graph_preset', 'create_graph_preset', 'update_graph_preset', 'delete_graph_preset',
            'view_report_publication', 'create_report_publication', 'update_report_publication', 'schedule_report_publication', 'use_iquest_report_publication',
            'view_iqreport_mail', 'create_iqreport_mail', 'update_iqreport_mail', 'schedule_iqreport_mail',
            'create_individual_reports', 'use_iquest_report_basket_analysis', 'use_iquest_time_dimension',
        ],
        'license_configs': [],
    }]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'iquest.report.week_time_dimension'
    config.type = 'string'
    config.default = "WeekISO8601" # WeekISO8601 or WeekUS
    config.description = u'Variant of the week time dimension to use'
    configs.append(config)

    return configs
