# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class IQuestPublicationOverview(IconBarIcon):
    """
    Class responsible for generating the Publication Overview icon
    """

    id = 'icon-iquest-publication-overview'
    title = _('Publication Overview')
    icon_index = 32
    component = 'iquest_publication_manager'
    group = 'iquest'
    colorname = 'iconbar-curious-blue'

    char_main = u'\uE805' # Layers icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_survey_publication')


class IQuestCloneResponsesWizard(IconBarIcon):
    id = 'icon-iquest-clone-responses-wizard'
    title = _('Clone responses wizard')
    icon_index = 33
    component = 'iquest_clone_responses'
    group = 'iquest'
    colorname = 'iconbar-curious-blue'

    char_main = u'\uE805'  # Layers icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('clone_survey_publication_responses')


def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [
        IQuestPublicationOverview(),
        IQuestCloneResponsesWizard(),
    ]