# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.scheduler.controllers import SchedulerServerController
from iqportal.iquest.response.controllers import SurveyCloneResponseController


class CloneResponseRpcHandler:

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']

        self.controller = SurveyCloneResponseController(self.sa_session, account_id=self.account_id, request=request)
        # this RPC handler is only for groups with the clone survey publication responses permission
        self.controller.assert_permissions('clone_survey_publication_responses')

    def get_licenses_with_survey_permissions(self):
        return sorted([{'label': '%s - %s' % (l.domain.name, l.name), 'value': l.id} for l in self.controller.get_licenses_with_survey_permissions()], key=lambda k: k['label'].strip())

    def get_surveys_by_license(self, license_id):
        return sorted([{'label': s.name, 'value': s.id} for s in self.controller.get_surveys_by_license(license_id)], key=lambda k: k['label'].strip())

    def get_available_contact_lists(self, license_id=None):
        if license_id is None:
            return []
        return sorted([{'label': cl.name, 'value': cl.id} for cl in self.controller.get_available_contact_lists(license_id)], key=lambda k: k['label'].strip())

    def get_company_matches(self, survey_id, contact_list_id):
        return self.controller.get_company_matches(survey_id, contact_list_id)

    def get_source_information(self, survey_id):
        return self.controller.get_source_information(survey_id)

    def get_summary(self, source, target, matches):
        return self.controller.get_summary(source, target, matches)

    def validate_target(self, source_survey_id, target_survey_id):
        if source_survey_id == target_survey_id:
            return False, 'Source and target surveys cannot be the same.'
        return self.controller.is_same(source_survey_id, target_survey_id)

    def start_cloning(self, source_survey_id, target_survey_id, target_contact_list_id, company_matches):
        sc = SchedulerServerController(self.sa_session)
        sc.system_start_task('iqportal.iquest.response.schedule.CloneResponsesTask', {'settings': {
            'source_survey_id': source_survey_id,
            'target_survey_id': target_survey_id,
            'target_contact_list_id': target_contact_list_id,
            'company_matches': company_matches,
            'account_id': self.controller.get_account().id
        }});
        return True

handlers = (CloneResponseRpcHandler, )
