# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
import time
from django.utils.translation import ugettext as _

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher, JsonRpcJSONEncoder
from basesoft.web.contrib.i18n import models as i18n_models

from iqportal.iquest.response.controllers import ParticipantController, PublicationResponseController, BranchController
from iqportal.iquest.core.qa_filters.controllers import IQuestQAFilterController
from basesoft.web.core.exceptions import PublicUserException
from django.shortcuts import render_to_response
from django.template.context import RequestContext

from datetime import datetime
from django.template import TemplateDoesNotExist
from django.utils import simplejson
from basesoft.web.contrib.compose.views import compose_runner

from basesoft import logging
from django.utils import translation
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.branding.controllers import LicenseThemeController

logger = logging.getLogger("iqportal.iquest.response")
# we need to make sure that we serialize requests per session
from threading import Lock
session_locks = {}
get_set_session_lock = Lock()

def bootstrap(request, version, type, extra_requires):
    requires = ['iq_core', 'iq_loader']

    # parse extra_requires
    if extra_requires == '':
        extra_requires = []
    else:
        extra_requires = extra_requires.split(',')
    requires.extend(extra_requires)
    from basesoft.web.core.views import loader_get

    return loader_get(request, version, type, '', ','.join(requires))

def display(request, hash, viewer_print=None):
    """
    """
    time_a = time.time()

    print_ = 1 if viewer_print else 0

    sa_session = request.Session()

    user_id = None
    if request.session.has_key('user_id'):
        user_id = request.session['user_id']

    ltc = LicenseThemeController(sa_session, account_id=user_id, request=request)
    current_theme = ltc.get_current_theme()
    theme_css = current_theme.css_class
    theme_module_name = current_theme.module_name
    request.session['theme_module_name'] = theme_module_name


    participant_session_id = None
    #if request.session.has_key('participant_session_id'):
    #    participant_session_id = request.session['participant_session_id']

    lc = LicenseController(request.Session())
    licensee = lc.get_license_for_request(request)

    # grab the global lock to ensure no race condition for creating and then acquiring the per-session lock
    session_id = request.session.session_key
    with get_set_session_lock:
        if session_id not in session_locks:
            session_locks[session_id] = Lock()
        session_locks[session_id].acquire()

    # locale is none by default, meaning the participant controller will take care of choosing a locale
    # if the user postsed his own locale override it here
    locale = None
    if request.POST.get('locale'):
        locale = sa_session.query(i18n_models.Locale).filter_by(locale=request.POST['locale']).one()

    # try to init participant controller, fallback to login dialog
    try:
        pc = ParticipantController(sa_session, locale, hash, participant_session_id)
        request.session['participant_session_id'] = pc.get_participant_session_id()
    except PublicUserException, e:
        try:
            return render_to_response('licensee/%s_login.html' % licensee.http_host, {'error': e.message}, context_instance=RequestContext(request))
        except TemplateDoesNotExist:
            return render_to_response('default_login.html', {'error': e.message}, context_instance=RequestContext(request))
    finally:
        #release the per-session lock
        session_locks[session_id].release()

    # at this point the participant runner has a valid locale chosen
    # set runner locale in session. We cannot use django internal locale system for this because
    # there might be more or other locales for compose than are translated
    request.session['runner_locale'] = pc.locale.locale

    # set the django locale so interface language changes (next/prev buttons). the locale might not be translated, then it will fallback to english
    translation.activate(pc.locale.locale)
    request.LANGUAGE_CODE = translation.get_language()

    # setting the django_language in session makes sure that new requests (like rpc) are in the currenty locale
    # converts a locale to language like  # nl_NL -> nl
    # we might have to fix this if we get django locales with country codes
    lang_code = pc.locale.locale.split('_')[0]
    request.session['django_language'] = lang_code

    # log the user out. If we dont do this the django language wont change because of our locale middleware (locale.py)
    ac = AccountController(request.Session())
    ac.logout(request)

    publication = pc.get_publication()
    # render survey
    _vars = {
        'print': print_,
        'invitee_hash': hash,
        'participant_session_id': pc.get_participant_session().id,
        'compose_name': publication.survey.name,
        'publication_label': publication.label,
        'show_compose_label': publication.show_compose_label,
        'show_publication_label': publication.show_publication_label,
        'availableLocales': [(x.locale, x.name)  for x in  publication.survey.available_locales],
        'currentLocale': pc.locale.locale,
        'theme_css': theme_css,
    }

    title = 'IQuest - %s - %s' % (pc.get_publication().survey.name, pc.get_publication().label)
    rv = compose_runner(request, ['iquest_response_init'], _vars, title, licensee)

    logger.info('!!! ----- Total time display():  %.4f seconds' % (time.time() - time_a))

    return rv


def login(request):
    pass

def jsonrpc_view(request):
    """View that handles rpc calls"""
    participant_session_id = None
    if request.session.has_key('participant_session_id'):
        participant_session_id = request.session['participant_session_id']

    class RPCHandler:

        def __init__(self, sa_session, participant_session_id):
            self.sa_session = sa_session
            self.participant_session_id = participant_session_id

        def load(self, invite_hash, participant_id, locale_id):

            # FIXME: implement participant_id, locale_id.. ticket #3827

            time_a = time.time()

            locale = None
            if request.session.has_key('runner_locale') and request.session['runner_locale'] is not None:
                locale = self.sa_session.query(i18n_models.Locale).filter_by(locale=request.session['runner_locale']).one()

            controller = ParticipantController(self.sa_session, locale, invite_hash, self.participant_session_id)
            request.session['participant_session_id'] = controller.get_participant_session_id()

            filter_controller = IQuestQAFilterController(self.sa_session)
            available_filters = filter_controller.existing_filters()

            existing_filters = {}
            for f in available_filters:
                existing_filters[f.name] = {'id': f.id}

            # return the state and responses
            rv = []
            rv.append(controller.load_survey_state())

            rv.append(controller.get_responses())
            rv.append(existing_filters)

            logger.info('!!! ----- iquest load() took %.4f' % (time.time() - time_a))

            return rv

        def update_responses(self, invite_hash, responses, blocks=None):

            time_a = time.time()

            controller = ParticipantController(self.sa_session, False, invite_hash, self.participant_session_id)
            request.session['participant_session_id'] = controller.get_participant_session_id()

            # send responses to controller
            rv = controller.update_responses(responses, blocks=blocks)
            if rv is None:
                bc = BranchController(self.sa_session, responses)
                rv = {'next_blocks': bc.get_next_blocks(blocks)}

            else:
                rv = {'validation_response': rv}

            logger.info('!!! ----- iquest update_responses() took %.4f' % (time.time() - time_a))

            return rv


        def validate_responses(self, invite_hash, responses, blocks=None):
            """
            TODO: validate that the invite is part of the publication that the responses belong to.
            """
            from iqportal.iquest.response.controllers import ResponseFilterController, ResponseRequiredController

            vr = [] # validation response list

            rrc = ResponseRequiredController(self.sa_session, responses)
            if not rrc.is_valid(blocks):
                vr.extend(rrc.get_result())

            rfc = ResponseFilterController(self.sa_session, responses)
            if not rfc.is_valid(blocks):
                vr.extend(rfc.get_result())

            # return any error messages if we have them
            if len(vr) > 0:
                return {'validation_response': vr}

            bc = BranchController(self.sa_session, responses)
            return {'next_blocks': bc.get_next_blocks(blocks)}


        def clear_session(self):
            """
            """
            if request.session.has_key('participant_session_id'):
                del request.session['participant_session_id']

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(RPCHandler(request.Session(), participant_session_id))
    return dispatcher.handle_request(request)

def publication_responses_view(request):
    """View that handles rpc calls for manager"""

    class Handler:

        def __init__(self, sa_session, account_id):
            self.sa_session = sa_session
            self.account_id = account_id
            self.controller = PublicationResponseController(self.sa_session, self.account_id)

        def get_publication_invitees_and_participants(self, publication_id):
            """
            Return a list with publication invitees and participants that will be displayed in the invitees select
            in the survey invitee preview toolbox.
            """

            participants, invitees = self.controller.get_publication_invitees_and_participants(publication_id)
            ret = []
            anon_count = 1
            for p in participants:
                if p.invitee.contact is not None:
                    person = p.invitee.contact.person
                    company_name = ''
                    if p.invitee.contact.company is not None:
                        company_name = '(%s)' % p.invitee.contact.company
                    label = "%s - %s %s" % ( person.email, person.full_name, company_name)
                else:
                    label = "%s %s" % (_("anonymous"), anon_count)
                    anon_count = anon_count + 1


                ret.append({'value': '%s_%s' % (p.invitee.id, p.id),
                            'label': label})
            for i in invitees:
                person = i.contact.person
                company_name = ''
                if i.contact.company is not None:
                    company_name = '(%s)' % i.contact.company
                label = "%s - %s %s" % ( person.email, person.full_name, company_name)
                ret.append({'value': i.id,
                            'label': label})
            return ret

        def get_publication_details(self, publication_id):
            """
            """
            from iqportal.iquest.compose.controllers import PublicationStateController
            from basesoft.web.core.util.controllers import StateManager
            manager = StateManager(None, self.account_id, None)
            controller = PublicationStateController(self.sa_session, manager)

            details = self.controller.get_publication_details(publication_id)

            return {
                'state': controller.load(publication_id),
                'survey_title': details['name'],
                'details': {
                    'invitees': details['invitees'],
                    'participants': details['participants'],
                    'respondents': details['respondents']}
            }

        def load_invitee_survey(self, invitee_id, participant_id=None, locale_id=None):
            """
            Loads survey state and responses for given participant
            """

            locale = None
            if locale_id is not None and locale_id != '':
                locale = self.sa_session.query(i18n_models.Locale).filter_by(id=locale_id).one()
                self.controller.locale = locale

            filter_controller = IQuestQAFilterController(self.sa_session)
            available_filters = filter_controller.existing_filters()

            existing_filters = {}
            for f in available_filters:
                existing_filters[f.name] = {'id': f.id}

            if participant_id is not None:
                # return the state and responses
                rv = [self.controller.load_survey_state(participant_id), self.controller.get_responses(participant_id), existing_filters]
            else:
                # no need or perms and privs checks because it is not about viewing responses
                rv = [self.controller.load_survey_state_by_invitee(invitee_id), {}, existing_filters]

            # this method is called when showing responses from the publication overview
            # dont allow survey printing from as the urls wont work (it adds +print/) to /
            # instead print buttons are added from the overviews directly
            rv[0]['survey']['viewer_print'] = False
            rv[0]['survey']['viewer_pdf'] = False

            return rv



        def update_invitee_responses(self, invitee_id, responses, blocks=None, participant_id=None):
            """
            Updates responses for a certain invitee
            """
            rv = self.controller.update_responses(invitee_id, responses, blocks, participant_id)
            if rv is None:
                bc = BranchController(self.sa_session, responses)
                return {'next_blocks': bc.get_next_blocks(blocks)}
            return {'validation_response': rv}

        def delete_responses(self, participant_ids):
            """
            Removes all responses for certain participants
            """
            if not isinstance(participant_ids, list):
                participant_ids = [participant_ids]
            deleted = 0
            for ih in participant_ids:
                p = self.controller.delete_responses(ih)
                if p is not None:
                    deleted = deleted + 1
            return deleted

    if request.session.has_key('user_id'):
        account_id = request.session['user_id']
    else:
        account_id = None

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(Handler(request.Session(), account_id))
    return dispatcher.handle_request(request)
