# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import func, and_
from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel
from iqportal.sap.core.models import SAPProject, SAPProjectEstimate, SAPProjectActual, SAPProjectComponentElement,\
    SAPProjectPeriod, SAPProjectGroup
from basesoft.web.contrib.license.models import Group, License

class ProjectCustomQuery(CustomQuery):
    permissions = ['view_sap_project']

    def tables(self):
        # we assume that sap project manager has access to all projects
        if self.has_permissions('sap_project_manager'):
            return (
                CustomQueryModel(SAPProject, priv=None),
                # FIXME
                CustomQueryModel(SAPProjectGroup, onclause=and_(SAPProject.id==SAPProjectGroup.project_id,
                                                                SAPProjectGroup.license_has_privilege==1)),
                CustomQueryModel(Group, onclause=Group.id==SAPProjectGroup.group_id),
                CustomQueryModel(License, onclause=License.id==Group.license_id),
            )
        else:
            return (
                CustomQueryModel(SAPProject),
            )
    def fields(self):
        c = ()
        if self.has_permissions('sap_project_manager'):
            c = (CustomQueryField(License.name, _('Company'), key='license_name', is_filter=True),)
        return (
            CustomQueryField(SAPProject.id, _('ID'), key='id'),
            CustomQueryField(SAPProject.name, _('Name'), key='name', is_filter=True),
            ) + c + (
            CustomQueryField(SAPProject.id, _('Estimates'), key='estimates'),
            CustomQueryField(SAPProject.id, _('Actuals'), key='actuals'),
            CustomQueryField(SAPProject.created, _('Created'), key='created'),
            )

class ProjectEstimatesCustomQuery(CustomQuery):
    permissions = ['view_sap_project']

    def tables(self):
        return (
            CustomQueryModel(SAPProjectEstimate),
            CustomQueryModel(SAPProjectPeriod, onclause=SAPProjectEstimate.project_period_id==SAPProjectPeriod.id),
            CustomQueryModel(SAPProjectComponentElement, onclause=SAPProjectEstimate.project_component_element_id==SAPProjectComponentElement.id),
            CustomQueryModel(SAPProject, onclause=SAPProjectComponentElement.project_id==SAPProject.id),
        )
    def fields(self):
        return (
            CustomQueryField(SAPProject.name, _('Project Name'), key='project_name', is_filter=True),
            CustomQueryField(SAPProjectPeriod.year, _('Year'), key='year', is_filter=True),
            CustomQueryField(SAPProject.id, _('View'), key='project_id'),
            CustomQueryField(SAPProjectPeriod.id, _('Period ID'), key='period_id')
        )
    group_by = (SAPProject.id, SAPProjectPeriod.id)

class ProjectActualsCustomQuery(CustomQuery):
    permissions = ['view_sap_project']

    def tables(self):
        return (
            CustomQueryModel(SAPProjectActual),
            CustomQueryModel(SAPProjectPeriod, onclause=SAPProjectActual.project_period_id==SAPProjectPeriod.id),
            CustomQueryModel(SAPProjectComponentElement, onclause=SAPProjectActual.project_component_element_id==SAPProjectComponentElement.id),
            CustomQueryModel(SAPProject, onclause=SAPProjectComponentElement.project_id==SAPProject.id),
        )
    def fields(self):
        return (
            CustomQueryField(SAPProject.name, _('Project Name'), key='project_name', is_filter=True),
            CustomQueryField(SAPProjectPeriod.year, _('Year'), key='year', is_filter=True),
            CustomQueryField(SAPProjectActual.actuals_date, _('Actuals date'), key='actuals_date', is_filter=True),
            CustomQueryField(SAPProject.id, _('View'), key='project_id'),
            CustomQueryField(SAPProjectPeriod.id, _('Period ID'), key='period_id')
        )
    group_by = (SAPProject.id, SAPProjectPeriod.id)

custom_queries = (ProjectCustomQuery, ProjectEstimatesCustomQuery, ProjectActualsCustomQuery, )