# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from iqportal.sap.core.controllers import SAPCoreController
from iqportal.sap.core.models import SAPProjectEstimate, SAPProjectActual

class CoreRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = SAPCoreController(request.Session(), account_id=request.session['user_id'])

    def load_project_data(self, project_id=None):
        ps = {}
        if project_id is not None:
            ps = self.controller.get_project_state(project_id)
        return {'project': ps,
                'top_components': self.get_child_components(None, project_id)
                }

    def get_child_components(self, component_id=None, project_id=None):
        """
        Returns childs of the provided component_id, or root components when components_id is None
        If project_id is supplied, it also returns highlight state for each node
        (0 - not selected, 1 - selected, 2 - has at least one child selected)
        """
        if component_id is not None:
            c = self.controller.get_components(component_id)[0]
            components = c.child_component_elements
        else:
            components = [c for c in self.controller.get_components() if c.parent_component_element is None]
        if project_id is None:
            return [self.controller.get_component_state(c) for c in components]

        ret = []
        project_components = self.controller.get_project_state(project_id)['components']
        for c in components:
            s = self.controller.get_component_state(c)
            s['highlight'] = 0
            if c.id in project_components:
                s['highlight'] = 1
            else:
                for ch in c.child_component_elements:
                    if ch.id in project_components:
                        s['highlight'] = 2
                        break
            ret.append(s)
        return  ret

    def save_project(self, state):
        return self.controller.save_project(state).id

    def load_project_form_data(self, type, project_id, period_id=None):
        if type == 'estimates':
            measures = SAPProjectEstimate.cols_i18n
            values = self.controller.load_project_estimates(project_id, period_id)
        else:
            measures = SAPProjectActual.cols_i18n
            values = self.controller.load_project_actuals(project_id, period_id)
        components, labels = self.controller.load_project_components(project_id)
        return {'components': components,
                'labels': labels,
                'measures': measures,
                'values': values
                }

    def save_project_form_data(self, type, project_id, state):
        if type == 'estimates':
            return self.controller.save_project_estimates(project_id, state)
        else:
            return self.controller.save_project_actuals(project_id, state)


handlers = (CoreRPCHandler, )
