# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy import and_, text

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator

from iqportal.weee.core.models import WEEEProductLevel


class WEEEController(AbstractController):

    def get_product_level_tree(self, _id=None, priv=WEEEProductLevel.PRIV_VIEW, parent_id=None, locale_id=None):
        self.assert_permissions('view_weee_product_levels')
        # TODO: translations
        rv = []

        _where = WEEEProductLevel.parent_level_id == parent_id
        if _id is not None:
            _where = and_(_where, WEEEProductLevel.id == _id)

        order_by = WEEEProductLevel.position
        p = self._get_privileged_object(WEEEProductLevel, where=_where, priv=priv, order_by=order_by)
        for product in p:

            rv.append({
                'id': product.id,
                'label': product.label,
                'description': product.description,
                'children': self.get_product_level_tree(None, priv=priv, parent_id=product.id, locale_id=locale_id)
            })

        return rv

    def get_product_level(self, product_id):
        self.assert_permissions('view_weee_product_levels')
        return self.get_privileged_object_by_id(WEEEProductLevel, product_id)

    @transaction_decorator
    def add_product_level(self, label, description, parent_level_id):
        self.assert_permissions('manage_weee_product_levels')

        parent_level = None if parent_level_id is None else self.get_product_level(parent_level_id).pop()
        position = 1 if parent_level is None or len(parent_level.sublevels) == 0 else parent_level.sublevels[-1].position+1

        p = WEEEProductLevel()
        p.label = label
        p.description = description
        p.parent_level = parent_level
        p.position = position
        self.save_default_privilege_object_group(p)
        self.sa_session.add(p)

        return p

    def populate_closure_table(self):
        self.sa_session.execute(text('CALL populateWEEEProductLevelClosure').execution_options(autocommit=True))
        self.logger.info('repopulate of WEEE product level closure table done.')