# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy.orm.query import aliased
from sqlalchemy import func, case, cast, Integer

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryModel, CustomQueryField
from iqportal.weee.core.models import WEEEProductLevel
from sqlalchemy.sql.expression import desc

ParentLevel1 = aliased(WEEEProductLevel)
ParentLevel2 = aliased(WEEEProductLevel)


class WEEEProductLevelCustomQuery(CustomQuery):

    permissions = ('view_weee_product_levels', )

    def tables(self):
        return (
            CustomQueryModel(WEEEProductLevel),
            CustomQueryModel(ParentLevel1, onclause=WEEEProductLevel.parent_level_id == ParentLevel1.id, use_outerjoin=True),
            CustomQueryModel(ParentLevel2, onclause=ParentLevel1.parent_level_id == ParentLevel2.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(
                func.concat(
                    case([(ParentLevel2.position == None, '')], else_=func.concat(ParentLevel2.position, '.')),
                    case([(ParentLevel1.position == None, '')], else_=func.concat(ParentLevel1.position, '.')),
                    WEEEProductLevel.position
                ), _('Category'), key='cat'),

            CustomQueryField(
                func.ifnull(ParentLevel2.position, func.ifnull(ParentLevel1.position, WEEEProductLevel.position)),
                "SORT", key='sort1'
            ),
            CustomQueryField(
                case([(ParentLevel1.position == None, 0)], else_=case([(ParentLevel2.position == None, WEEEProductLevel.position)], else_=ParentLevel1.position)),
                "SORT2", key='sort2'
            ),

            CustomQueryField(
                case([(ParentLevel2.position == None, 0)], else_=case([(ParentLevel1.position == None, 0)], else_=WEEEProductLevel.position)),
                "SORT3", key='sort3'
            ),

            CustomQueryField(WEEEProductLevel.id, _('ID'), key='id'),
            CustomQueryField(WEEEProductLevel.label, _('Label'), key='label', is_filter=True),
            CustomQueryField(WEEEProductLevel.description, _('Description'), key='description', is_filter=True),
        )

    def _get_data(self, split_by, sort, asc, offset, limit):
        if sort is None or sort == 'cat':
            self.order_by = ('sort1', 'sort2', 'sort3', )
        if asc is False and (sort is None or sort == 'cat'):
            self.order_by = (desc('sort1'), desc('sort2'), desc('sort3'), )
        return CustomQuery._get_data(self, split_by, sort, asc, offset, limit)

    group_by = (WEEEProductLevel.id, )


custom_queries = (
    WEEEProductLevelCustomQuery,
)
