# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy import and_, text

from basesoft.web.contrib.license.models import License, Group
from basesoft.web.core.util.controllers import AbstractController

from iqportal.weee.core import EEE_CATEGORIES, BAT_CATEGORIES, PAC_CATEGORIES
from iqportal.weee.core.models import WEEEProductLevel


def sync(session):

    ac = AbstractController(session)
    group = session.query(Group).join(License).filter(and_(Group.is_default == True, License.name == 'BaseSoft Solutions B.V.')).first()

    def get_or_add_level(description, pos, parent, prefix=None):
        label = '%s-%d' % (prefix, pos) if parent is None else '%s.%02d' % (parent.label, pos)
        level = session.query(WEEEProductLevel).filter(and_(
            WEEEProductLevel.label == label,
            WEEEProductLevel.parent_level == parent
        )).first()
        if level is None:
            level = WEEEProductLevel()
            level.description = description
            level.label = label
            level.position = pos
            level.parent_level = parent
            session.add(level)
            session.flush()
            ac.grant_privilege_to(level, privileges=[WEEEProductLevel.PRIV_VIEW], target_group=group, check_own_privs=False, world=True)

            return level
        elif level.position == pos:
            return level

        level.position = pos
        session.add(level)

    level1 = [{
        'label': 'Electrical and electronic equipment', 'children': EEE_CATEGORIES, 'prefix': 'EEE',
    }, {
        'label': 'Batteries', 'children': BAT_CATEGORIES, 'prefix': 'BAT',
    }, {
        'label': 'Packaging', 'children': PAC_CATEGORIES, 'prefix': 'PAC',
    }]

    l1pos = 1
    for l1 in level1:
        name = l1['label']
        level2 = l1['children']
        l1parent = get_or_add_level(name, l1pos, None, l1['prefix'])

        l2pos = 1
        for l2 in level2:
            l2parent = get_or_add_level(l2['label'], l2pos, l1parent)

            l3pos = 1

            for l3 in l2['subcategories']:
                l3parent = get_or_add_level(l3, l3pos, l2parent)
                l3pos += 1

            l2pos += 1

        l1pos += 1

def procedures():

    return [

        """
        DROP PROCEDURE IF EXISTS `populateWEEEProductLevelClosure`
        """,
        """
        CREATE PROCEDURE populateWEEEProductLevelClosure()
        BEGIN
            DECLARE distance int;
            TRUNCATE TABLE closureWEEEProductLevel;
            SET distance = 0;

            -- seed closure with self-pairs (distance 0)
            INSERT INTO closureWEEEProductLevel (parent_level_id, product_level_id, distance)
                SELECT id, id, distance
                FROM tblWEEEProductLevel;

            -- for each pair (root, leaf) in the closure,
            -- add (root, leaf->child) from the base table
            REPEAT
                SET distance = distance + 1;

                INSERT INTO closureWEEEProductLevel (parent_level_id, product_level_id, distance)
                SELECT c.parent_level_id, p.id, distance
                FROM closureWEEEProductLevel AS c, tblWEEEProductLevel AS p
                    WHERE c.product_level_id = p.parent_level_id
                    AND c.distance = distance - 1;
            UNTIL (ROW_COUNT() = 0) END REPEAT;
        END
        """

    ]

def post_sync(sa_session, app_name):
    if app_name != 'iqportal.weee.core':
        return

    sa_session.execute(text('CALL populateWEEEProductLevelClosure').execution_options(autocommit=True))

if __name__ == '__main__':
    from basesoft.web.core.util.db import Session
    sync(Session())