# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _
from sqlalchemy import func

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryModel, CustomQueryField
from iqportal.xbrl.core.models import XBRLProject, XBRLElement, XBRLContext

class XBRLProjectCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(XBRLProject),
            CustomQueryModel(XBRLContext, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(XBRLProject.id, 'id', key='project_id'),
            CustomQueryField(XBRLProject.name, _('XBRL Project'), key='project_name', is_filter=True),
            CustomQueryField(func.count(XBRLContext.id.distinct()), _('Contexts'), key='contexts'),
            CustomQueryField(XBRLProject.id, _('Elements'), key='elements'),
            CustomQueryField(XBRLProject.id, _('XBRL'), key='xbrl'),
        )

class ProjectElementCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(XBRLElement),
        )

    def fields(self):
        return (
            CustomQueryField(XBRLElement.project_id, _('Project ID'), key='project_id'),
            CustomQueryField(XBRLElement.id, _('Element ID'), key='element_id'),
            CustomQueryField(XBRLElement.handle, _('Handle'), key='element_handle', is_filter=True),
            CustomQueryField(XBRLElement.name, _('Name'), key='element_name')
        )

custom_queries = (
    XBRLProjectCustomQuery, ProjectElementCustomQuery
)