# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

import os

from iqportal.xbrl.core.controllers import XBRLController
from django.conf import settings

class XBRLRpcHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']

        self.controller = XBRLController(self.sa_session, account_id=self.account_id)

    def get_identifiers(self, project_id):
        return [{'value': i.id, 'label': i.identifier} for i in self.controller.get_identifier(project_id)]

    def get_years(self, project_id):
        periods = self.controller.get_period(project_id)
        ret = []
        years = []
        for p in periods:
            year = p.end_date.year
            if year in years:
                continue
            ret.append({'value': year, 'label': year})
            years.append(year)
        return ret

    def create_project(self, name, file):
        f = open(os.path.join(settings.TMP_FILE_REPOSITORY, file[0].file_hash))
        xsd_data = f.read()
        return self.controller.create_project(name, xsd_data).id

    def update_element_name(self, id, name):
        self.controller.update_element_name(id, name)

    def import_xbrl(self, project_id, files):
        file_paths = [os.path.join(settings.TMP_FILE_REPOSITORY, f.file_hash) for f in files]
        self.controller.import_xbrl(project_id, file_paths)

handlers = (XBRLRpcHandler, )
